@@ -2,7 +2,7 @@
  * ProFTPD: mod_ifsession -- a module supporting conditional
  *                            per-user/group/class configuration contexts.
  *
- * Copyright (c) 2002-2003 TJ Saunders
+ * Copyright (c) 2002-2004 TJ Saunders
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,7 +26,7 @@
  * This is mod_ifsession, contrib software for proftpd 1.2 and above.
  * For more information contact TJ Saunders <tj@castaglia.org>.
  *
- * $Id: mod_ifsession.c,v 1.14 2004/07/08 21:38:24 castaglia Exp $
+ * $Id: mod_ifsession.c,v 1.18 2004/12/17 18:24:57 castaglia Exp $
  */
 
 #include "conf.h"
@@ -106,9 +106,10 @@ static void ifsess_dup_param(pool *dst_pool, xaset_t **dst, config_rec *c,
 }
 
 static void ifsess_dup_set(pool *dst_pool, xaset_t *dst, xaset_t *src) {
-  config_rec *c = NULL;
+  config_rec *c, *next;
 
-  for (c = (config_rec *) src->xas_list; c; c = c->next) {
+  for (c = (config_rec *) src->xas_list; c; c = next) {
+    next = c->next;
 
     /* Skip the context lists. */
     if (c->config_type == IFSESS_CLASS_NUMBER ||
@@ -144,7 +145,7 @@ MODRET start_ifctxt(cmd_rec *cmd) {
 
   CHECK_CONF(cmd, CONF_ROOT|CONF_VIRTUAL|CONF_GLOBAL);
 
-  c = start_sub_config(cmd->argv[0]);
+  c = pr_parser_config_ctxt_open(cmd->argv[0]);
 
   /* "Inherit" the parent's context type. */
   c->config_type = (cmd->config && cmd->config->config_type != CONF_PARAM ?
@@ -152,17 +153,17 @@ MODRET start_ifctxt(cmd_rec *cmd) {
     cmd->server->config_type : CONF_ROOT);
 
   if (strcmp(cmd->argv[0], IFSESS_CLASS_TEXT) == 0) {
-    name = "IfClassList";
+    name = "_IfClassList";
     config_type = IFSESS_CLASS_NUMBER;
     eval_type = PR_EXPR_EVAL_OR;
 
   } else if (strcmp(cmd->argv[0], IFSESS_GROUP_TEXT) == 0) {
-    name = "IfGroupList";
+    name = "_IfGroupList";
     config_type = IFSESS_GROUP_NUMBER;
     eval_type = PR_EXPR_EVAL_AND;
 
   } else if (strcmp(cmd->argv[0], IFSESS_USER_TEXT) == 0) {
-    name = "IfUserList";
+    name = "_IfUserList";
     config_type = IFSESS_USER_NUMBER;
     eval_type = PR_EXPR_EVAL_OR;
   }
@@ -250,7 +251,7 @@ MODRET start_ifctxt(cmd_rec *cmd) {
 }
 
 MODRET end_ifctxt(cmd_rec *cmd) {
-  end_sub_config(NULL);
+  pr_parser_config_ctxt_close(NULL);
   return HANDLED(cmd);
 }
 
@@ -473,7 +474,6 @@ static int ifsess_sess_init(void) {
          */
         *((config_rec **) push_array(class_remove_list)) = c;
 
-        resolve_deferred_dirs(main_server);
         fixup_dirs(main_server, CF_DEFER);
 
         ifsess_merged = TRUE;
