@@ -1,6 +1,7 @@
 /*
  * ProFTPD: mod_sql_mysql -- Support for connecting to MySQL databases.
  * Copyright (c) 2001 Andrew Houghton
+ * Copyright (c) 2004-2005 TJ Saunders
  *  
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,11 +22,11 @@
  * the resulting executable, without including the source code for OpenSSL in
  * the source distribution.
  *
- * $Id: mod_sql_mysql.c,v 1.30 2004/05/08 03:21:46 castaglia Exp $
+ * $Id: mod_sql_mysql.c,v 1.39 2005/06/27 20:41:14 castaglia Exp $
  */
 
 /*
- * $Libraries: -lm -lz -lmysqlclient $
+ * $Libraries: -lm -lmysqlclient -lz $
  */
 
 /* INTRO:
@@ -88,9 +89,6 @@
  * connection policy, connections may be closed at any time and may need
  * to be reopened for any call.
  *
- * All backends should register an event handler for "core.exit", to close
- * any open connections.  See the function sql_mysql_exit_ev() as an example.
- *
  * CONNECTION TIMERS
  *
  * Backends are required to handle connection timers; when a connection is 
@@ -130,7 +128,7 @@
  * Internal define used for debug and logging.  All backends are encouraged
  * to use the same format.
  */
-#define _MOD_VERSION "mod_sql_mysql/4.04"
+#define MOD_SQL_MYSQL_VERSION		"mod_sql_mysql/4.05"
 
 #define _MYSQL_PORT "3306"
 
@@ -141,7 +139,7 @@
 /* 
  * timer-handling code adds the need for a couple of forward declarations
  */
-MODRET cmd_close( cmd_rec *cmd );
+MODRET cmd_close(cmd_rec *cmd);
 module sql_mysql_module;
 
 /* 
@@ -192,8 +190,8 @@ typedef struct conn_entry_struct conn_entry_t;
 
 #define DEF_CONN_POOL_SIZE 10
 
-pool *conn_pool = NULL;
-array_header *conn_cache = NULL;
+static pool *conn_pool = NULL;
+static array_header *conn_cache = NULL;
 
 /*
  *  _sql_get_connection: walks the connection cache looking for the named
@@ -252,9 +250,10 @@ static void *_sql_add_connection(pool *p, char *name, db_conn_t *conn)
  *  shutdown.
  */
 static void _sql_check_cmd(cmd_rec *cmd, char *msg) {
-  if ((!cmd) || (!cmd->tmp_pool)) {
-    pr_log_pri(PR_LOG_ERR, _MOD_VERSION ": '%s' was passed an invalid cmd_rec. "
-	    "Shutting down.", msg);
+  if (!cmd || 
+      !cmd->tmp_pool) {
+    pr_log_pri(PR_LOG_ERR, MOD_SQL_MYSQL_VERSION
+      ": '%s' was passed an invalid cmd_rec. Shutting down.", msg);
     sql_log(DEBUG_WARN, "'%s' was passed an invalid cmd_rec. Shutting down.",
       msg);
     end_login(1);
@@ -288,39 +287,16 @@ static int _sql_timer_callback(CALLBACK_FRAME) {
   return 0;
 }
 
-/* sql_mysql_exit_ev: walks the connection cache and closes every
- *  open connection, resetting their connection counts to 0.
- */
-static void sql_mysql_exit_ev(const void *event_data, void *user_data) {
-  register unsigned int i = 0;
-
-  for (i = 0; i < conn_cache->nelts; i++) {
-    conn_entry_t *entry = ((conn_entry_t **) conn_cache->elts)[i];
-
-    if (entry->connections > 0) {
-      cmd_rec *cmd = _sql_make_cmd(conn_pool, 2, entry->name, "1");
-      cmd_close(cmd);
-      SQL_FREE_CMD(cmd);
-    }
-  }
-
-  return;
-}
-
-/* 
- * _build_error: constructs a modret_t filled with error information;
+/* _build_error: constructs a modret_t filled with error information;
  *  mod_sql_mysql calls this function and returns the resulting mod_ret_t
  *  whenever a call to the database results in an error.  Other backends
  *  may want to use a different method to return error information.
- *
- *  If a backend database does not provide error information, please use
- *  the PR_ERR_SQL_UNKNOWN macro.
  */
 static modret_t *_build_error(cmd_rec *cmd, db_conn_t *conn) {
   char num[20] = {'\0'};
 
   if (!conn)
-    return PR_ERR_SQL_BADCMD(cmd);
+    return ERROR_MSG(cmd, MOD_SQL_MYSQL_VERSION, "badly formed request");
 
   snprintf(num, 20, "%u", mysql_errno(conn->mysql) );
 
@@ -344,7 +320,7 @@ static modret_t *_build_data(cmd_rec *cmd, db_conn_t *conn) {
   unsigned long i = 0;
 
   if (!conn) 
-    return PR_ERR_SQL_BADCMD(cmd);
+    return ERROR_MSG(cmd, MOD_SQL_MYSQL_VERSION, "badly formed request");
 
   mysql = conn->mysql;
 
@@ -412,14 +388,14 @@ MODRET cmd_open(cmd_rec *cmd) {
 
   if (cmd->argc < 1) {
     sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_open");
-    return PR_ERR_SQL_BADCMD(cmd);
+    return ERROR_MSG(cmd, MOD_SQL_MYSQL_VERSION, "badly formed request");
   }    
 
   /* get the named connection */
 
   if (!(entry = _sql_get_connection( cmd->argv[0]))) {
     sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_open");
-    return PR_ERR_SQL_UNDEF(cmd);
+    return ERROR_MSG(cmd, MOD_SQL_MYSQL_VERSION, "unknown named connection");
   } 
 
   conn = (db_conn_t *) entry->data;
@@ -429,9 +405,9 @@ MODRET cmd_open(cmd_rec *cmd) {
    */
   if ((entry->connections > 0) && (!mysql_ping(conn->mysql))) {
     entry->connections++;
-    if (entry->timer) {
-      reset_timer( entry->timer, &sql_mysql_module );
-    }
+    if (entry->timer)
+      pr_timer_reset(entry->timer, &sql_mysql_module);
+
     sql_log(DEBUG_INFO, "connection '%s' count is now %d", entry->name,
       entry->connections);
     sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_open");
@@ -442,8 +418,8 @@ MODRET cmd_open(cmd_rec *cmd) {
   conn->mysql = mysql_init(NULL);
 
   if (!conn->mysql) {
-    pr_log_pri(PR_LOG_ERR, _MOD_VERSION ": failed to allocate memory for "
-	    "MYSQL structure.  Shutting down.");
+    pr_log_pri(PR_LOG_ERR, MOD_SQL_MYSQL_VERSION
+      ": failed to allocate memory for MYSQL structure.  Shutting down.");
     sql_log(DEBUG_WARN, "%s", "failed to allocate memory for MYSQL structure. "
       "Shutting down.");
     end_login(1);
@@ -468,9 +444,8 @@ MODRET cmd_open(cmd_rec *cmd) {
 
   /* set up our timer if necessary */
   if (entry->ttl > 0) {
-    entry->timer = add_timer(entry->ttl, -1, 
-			     &sql_mysql_module, 
-			     _sql_timer_callback);
+    entry->timer = pr_timer_add(entry->ttl, -1, &sql_mysql_module,
+      _sql_timer_callback);
     sql_log(DEBUG_INFO, "connection '%s' - %d second timer started",
       entry->name, entry->ttl);
 
@@ -514,8 +489,7 @@ MODRET cmd_open(cmd_rec *cmd) {
  *  If argv[1] exists and is not NULL, the connection should be immediately
  *  closed and the connection count should be reset to 0.
  */
-MODRET cmd_close(cmd_rec *cmd)
-{
+MODRET cmd_close(cmd_rec *cmd) {
   conn_entry_t *entry = NULL;
   db_conn_t *conn = NULL;
 
@@ -525,13 +499,13 @@ MODRET cmd_close(cmd_rec *cmd)
 
   if ((cmd->argc < 1) || (cmd->argc > 2)) {
     sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_close");
-    return PR_ERR_SQL_BADCMD(cmd);
+    return ERROR_MSG(cmd, MOD_SQL_MYSQL_VERSION, "badly formed request");
   }
 
   /* get the named connection */
   if (!(entry = _sql_get_connection( cmd->argv[0] ))) {
     sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_close");
-    return PR_ERR_SQL_UNDEF(cmd);
+    return ERROR_MSG(cmd, MOD_SQL_MYSQL_VERSION, "unknown named connection");
   }
 
   conn = (db_conn_t *) entry->data;
@@ -555,7 +529,7 @@ MODRET cmd_close(cmd_rec *cmd)
     entry->connections = 0;
 
     if (entry->timer) {
-      remove_timer( entry->timer, &sql_mysql_module );
+      pr_timer_remove(entry->timer, &sql_mysql_module);
       entry->timer = 0;
       sql_log(DEBUG_INFO, "connection '%s' - timer stopped", entry->name);
     }
@@ -614,7 +588,7 @@ MODRET cmd_defineconnection(cmd_rec *cmd)
 
   if ((cmd->argc < 4) || (cmd->argc > 5) || (!cmd->argv[0])) {
     sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_defineconnection");
-    return PR_ERR_SQL_BADCMD(cmd);
+    return ERROR_MSG(cmd, MOD_SQL_MYSQL_VERSION, "badly formed request");
   }
 
   conn = (db_conn_t *) palloc(conn_pool, sizeof(db_conn_t));
@@ -659,7 +633,8 @@ MODRET cmd_defineconnection(cmd_rec *cmd)
   /* insert the new conn_info into the connection hash */
   if (!(entry = _sql_add_connection(conn_pool, name, (void *) conn))) {
     sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_defineconnection");
-    return PR_ERR_SQL_REDEF(cmd);
+    return ERROR_MSG(cmd, MOD_SQL_MYSQL_VERSION,
+      "named connection already exists");
   }
 
   entry->ttl = (cmd->argc == 5) ? 
@@ -681,6 +656,35 @@ MODRET cmd_defineconnection(cmd_rec *cmd)
   return HANDLED(cmd);
 }
 
+/*
+ * cmd_exit: closes all open connections.
+ *
+ * Inputs:
+ *  None
+ *
+ * Returns:
+ *  A simple non-error modret_t.
+ */
+static modret_t *cmd_exit(cmd_rec *cmd) {
+  register unsigned int i = 0;
+
+  sql_log(DEBUG_FUNC, "%s", "entering \tmysql cmd_exit");
+
+  for (i = 0; i < conn_cache->nelts; i++) {
+    conn_entry_t *entry = ((conn_entry_t **) conn_cache->elts)[i];
+
+    if (entry->connections > 0) {
+      cmd_rec *close_cmd = _sql_make_cmd(conn_pool, 2, entry->name, "1");
+      cmd_close(close_cmd);
+      destroy_pool(close_cmd->pool);
+    }
+  }
+
+  sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_exit");
+
+  return HANDLED(cmd);
+}
+
 /*
  * cmd_select: executes a SELECT query. properly constructing the query
  *  based on the inputs.  See mod_sql.h for the definition of the _sql_data
@@ -727,8 +731,7 @@ MODRET cmd_defineconnection(cmd_rec *cmd)
  *  certain selects could return huge amounts of data.  do whatever is
  *  possible to minimize the amount of data copying here.
  */
-MODRET cmd_select(cmd_rec *cmd)
-{
+MODRET cmd_select(cmd_rec *cmd) {
   conn_entry_t *entry = NULL;
   db_conn_t *conn = NULL;
   modret_t *cmr = NULL;
@@ -743,14 +746,14 @@ MODRET cmd_select(cmd_rec *cmd)
 
   if (cmd->argc < 2) {
     sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_select");
-    return PR_ERR_SQL_BADCMD(cmd);
+    return ERROR_MSG(cmd, MOD_SQL_MYSQL_VERSION, "badly formed request");
   }
 
   /* get the named connection */
   entry = _sql_get_connection( cmd->argv[0] );
   if (!entry) {
     sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_select");
-    return PR_ERR_SQL_UNDEF(cmd);
+    return ERROR_MSG(cmd, MOD_SQL_MYSQL_VERSION, "unknown named connection");
   }
  
   conn = (db_conn_t *) entry->data;
@@ -860,8 +863,7 @@ MODRET cmd_select(cmd_rec *cmd)
  * Notes:
  *  none
  */
-MODRET cmd_insert(cmd_rec *cmd)
-{
+MODRET cmd_insert(cmd_rec *cmd) {
   conn_entry_t *entry = NULL;
   db_conn_t *conn = NULL;
   modret_t *cmr = NULL;
@@ -875,14 +877,14 @@ MODRET cmd_insert(cmd_rec *cmd)
 
   if ((cmd->argc != 2) && (cmd->argc != 4)) {
     sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_insert");
-    return PR_ERR_SQL_BADCMD(cmd);
+    return ERROR_MSG(cmd, MOD_SQL_MYSQL_VERSION, "badly formed request");
   }
 
   /* get the named connection */
-  entry = _sql_get_connection( cmd->argv[0] );
+  entry = _sql_get_connection(cmd->argv[0]);
   if (!entry) {
     sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_insert");
-    return PR_ERR_SQL_UNDEF(cmd);
+    return ERROR_MSG(cmd, MOD_SQL_MYSQL_VERSION, "unknown named connection");
   }
 
   conn = (db_conn_t *) entry->data;
@@ -959,8 +961,7 @@ MODRET cmd_insert(cmd_rec *cmd)
  *  argv[3] is optional -- it may be NULL, or it may not exist at all.  
  *  make sure this is handled correctly. 
  */
-MODRET cmd_update(cmd_rec *cmd)
-{
+MODRET cmd_update(cmd_rec *cmd) {
   conn_entry_t *entry = NULL;
   db_conn_t *conn = NULL;
   modret_t *cmr = NULL;
@@ -974,14 +975,14 @@ MODRET cmd_update(cmd_rec *cmd)
 
   if ((cmd->argc < 2) || (cmd->argc > 4)) {
     sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_update");
-    return PR_ERR_SQL_BADCMD(cmd);
+    return ERROR_MSG(cmd, MOD_SQL_MYSQL_VERSION, "badly formed request");
   }
 
   /* get the named connection */
-  entry = _sql_get_connection( cmd->argv[0] );
+  entry = _sql_get_connection(cmd->argv[0]);
   if (!entry) {
     sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_update");
-    return PR_ERR_SQL_UNDEF(cmd);
+    return ERROR_MSG(cmd, MOD_SQL_MYSQL_VERSION, "unknown named connection");
   }
 
   conn = (db_conn_t *) entry->data;
@@ -1046,22 +1047,22 @@ MODRET cmd_update(cmd_rec *cmd)
  *  not support stored procedures should return an error with a descriptive
  *  error message (something like 'backend does not support procedures').
  */
-MODRET cmd_procedure(cmd_rec *cmd)
-{
+MODRET cmd_procedure(cmd_rec *cmd) {
   sql_log(DEBUG_FUNC, "%s", "entering \tmysql cmd_procedure");
 
   _sql_check_cmd(cmd, "cmd_procedure");
 
   if (cmd->argc != 3) {
     sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_procedure");
-    return PR_ERR_SQL_BADCMD(cmd);
+    return ERROR_MSG(cmd, MOD_SQL_MYSQL_VERSION, "badly formed request");
   }
 
   /* MySQL does not support procedures.  Nothing to do. */
 
   sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_procedure");
 
-  return ERROR_MSG(cmd, _MOD_VERSION, "backend does not support procedures");
+  return ERROR_MSG(cmd, MOD_SQL_MYSQL_VERSION,
+    "backend does not support procedures");
 }
 
 /*
@@ -1083,8 +1084,7 @@ MODRET cmd_procedure(cmd_rec *cmd)
  * Notes:
  *  None.
  */
-MODRET cmd_query(cmd_rec *cmd)
-{
+MODRET cmd_query(cmd_rec *cmd) {
   conn_entry_t *entry = NULL;
   db_conn_t *conn = NULL;
   modret_t *cmr = NULL;
@@ -1098,14 +1098,14 @@ MODRET cmd_query(cmd_rec *cmd)
 
   if (cmd->argc != 2) {
     sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_query");
-    return PR_ERR_SQL_BADCMD(cmd);
+    return ERROR_MSG(cmd, MOD_SQL_MYSQL_VERSION, "badly formed request");
   }
 
   /* get the named connection */
-  entry = _sql_get_connection( cmd->argv[0] );
+  entry = _sql_get_connection(cmd->argv[0]);
   if (!entry) {
     sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_query");
-    return PR_ERR_SQL_UNDEF(cmd);
+    return ERROR_MSG(cmd, MOD_SQL_MYSQL_VERSION, "unknown named connection");
   }
 
   conn = (db_conn_t *) entry->data;
@@ -1183,8 +1183,7 @@ MODRET cmd_query(cmd_rec *cmd)
  *  copying the data from argv[0] into the data field of the modret allows
  *  for possible SQL injection attacks when this backend is used.
  */
-MODRET cmd_escapestring(cmd_rec * cmd)
-{
+MODRET cmd_escapestring(cmd_rec * cmd) {
   conn_entry_t *entry = NULL;
   db_conn_t *conn = NULL;
   char *unescaped = NULL;
@@ -1196,14 +1195,14 @@ MODRET cmd_escapestring(cmd_rec * cmd)
 
   if (cmd->argc != 2) {
     sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_escapestring");
-    return PR_ERR_SQL_BADCMD(cmd);
+    return ERROR_MSG(cmd, MOD_SQL_MYSQL_VERSION, "badly formed request");
   }
 
   /* get the named connection */
-  entry = _sql_get_connection( cmd->argv[0] );
+  entry = _sql_get_connection(cmd->argv[0]);
   if (!entry) {
     sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_escapestring");
-    return PR_ERR_SQL_UNDEF(cmd);
+    return ERROR_MSG(cmd, MOD_SQL_MYSQL_VERSION, "unknown named connection");
   }
 
   conn = (db_conn_t *) entry->data;
@@ -1248,8 +1247,7 @@ MODRET cmd_escapestring(cmd_rec * cmd)
  *  If this backend does not provide this functionality, this cmd *must*
  *  return ERROR.
  */
-MODRET cmd_checkauth(cmd_rec * cmd)
-{
+MODRET cmd_checkauth(cmd_rec * cmd) {
   conn_entry_t *entry = NULL;
   db_conn_t *conn = NULL;
   char scrambled[256]={'\0'};
@@ -1263,14 +1261,14 @@ MODRET cmd_checkauth(cmd_rec * cmd)
 
   if (cmd->argc != 3) {
     sql_log(DEBUG_FUNC, "exiting \tmysql cmd_checkauth");
-    return PR_ERR_SQL_BADCMD(cmd);
+    return ERROR_MSG(cmd, MOD_SQL_MYSQL_VERSION, "badly formed request");
   }
 
   /* get the named connection -- not used in this case, but for consistency */
   entry = _sql_get_connection( cmd->argv[0] );
   if (!entry) {
     sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_checkauth");
-    return PR_ERR_SQL_UNDEF(cmd);
+    return ERROR_MSG(cmd, MOD_SQL_MYSQL_VERSION, "unknown named connection");
   }
 
   if (cmd->argv[1] == NULL) {
@@ -1291,6 +1289,26 @@ MODRET cmd_checkauth(cmd_rec * cmd)
 
   success = !strcmp(scrambled, c_hash); 
 
+  if (!success) {
+
+#if MYSQL_VERSION_ID >= 40101
+    /* Try to work around MySQL's stupid handling of password length
+     * changes in 4.1, and the stupidity and whining of admins who
+     * cannot deal with those changes.
+     */
+    memset(scrambled, '\0', sizeof(scrambled));
+    make_scrambled_password_323(scrambled, c_clear);
+
+    sql_log(DEBUG_FUNC, "%s",
+      "checking again using deprecated legacy MySQL password algorithm");
+    success = !strcmp(scrambled, c_hash);
+    if (!success)
+      sql_log(DEBUG_FUNC, "%s", "password mismatch");
+#else
+    sql_log(DEBUG_FUNC, "%s", "password mismatch");
+#endif
+  }
+
   sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_checkauth");
 
   return success ? HANDLED(cmd) : ERROR_INT(cmd, PR_AUTH_BADPWD);
@@ -1327,22 +1345,22 @@ MODRET cmd_identify(cmd_rec * cmd) {
   sd->rnum = 1;
   sd->fnum = 2;
 
-  sd->data[0] = _MOD_VERSION;
+  sd->data[0] = MOD_SQL_MYSQL_VERSION;
   sd->data[1] = MOD_SQL_API_V1;
 
   return mod_create_data(cmd, (void *) sd);
 }  
 
-/* 
- * sql_cmdtable: mod_sql requires each backend module to define a cmdtable
+/* SQL cmdtable: mod_sql requires each backend module to define a cmdtable
  *  with this exact name. ALL these functions must be defined; mod_sql checks
  *  that they all exist on startup and ProFTPD will refuse to start if they
  *  aren't defined.
  */
-cmdtable sql_cmdtable[] = {
+static cmdtable sql_mysql_cmdtable[] = {
   { CMD, "sql_open",             G_NONE, cmd_open,             FALSE, FALSE },
   { CMD, "sql_close",            G_NONE, cmd_close,            FALSE, FALSE },
   { CMD, "sql_defineconnection", G_NONE, cmd_defineconnection, FALSE, FALSE },
+  { CMD, "sql_exit",             G_NONE, cmd_exit,             FALSE, FALSE },
   { CMD, "sql_select",           G_NONE, cmd_select,           FALSE, FALSE },
   { CMD, "sql_insert",           G_NONE, cmd_insert,           FALSE, FALSE },
   { CMD, "sql_update",           G_NONE, cmd_update,           FALSE, FALSE },
@@ -1355,17 +1373,55 @@ cmdtable sql_cmdtable[] = {
   { 0, NULL }
 };
 
-/*
- * sql_mysql_sess_init: Used to initialize the connection cache and register
- *  the exit handler.
+/* Event handlers
  */
+
+static void sql_mysql_mod_load_ev(const void *event_data, void *user_data) {
+
+  if (strcmp("mod_sql_mysql.c", (const char *) event_data) == 0) {
+    /* Register ourselves with mod_sql. */
+    if (sql_register_backend("mysql", sql_mysql_cmdtable) < 0) {
+      pr_log_pri(PR_LOG_NOTICE, MOD_SQL_MYSQL_VERSION
+        ": notice: error registering backend: %s", strerror(errno));
+      end_login(1);
+    }
+  }
+}
+
+static void sql_mysql_mod_unload_ev(const void *event_data, void *user_data) {
+
+  if (strcmp("mod_sql_mysql.c", (const char *) event_data) == 0) {
+    /* Unegister ourselves with mod_sql. */
+    if (sql_unregister_backend("mysql") < 0) {
+      pr_log_pri(PR_LOG_NOTICE, MOD_SQL_MYSQL_VERSION
+        ": notice: error unregistering backend: %s", strerror(errno));
+      end_login(1);
+    }
+
+    /* Unregister ourselves from all events. */
+    pr_event_unregister(&sql_mysql_module, NULL, NULL);
+  }
+}
+
+/* Initialization routines
+ */
+
+static int sql_mysql_init(void) {
+
+  /* Register listeners for the load and unload events. */
+  pr_event_register(&sql_mysql_module, "core.module-load",
+    sql_mysql_mod_load_ev, NULL);
+  pr_event_register(&sql_mysql_module, "core.module-unload",
+    sql_mysql_mod_unload_ev, NULL);
+
+  return 0;
+}
+
 static int sql_mysql_sess_init(void) {
   conn_pool = make_sub_pool(session.pool);
   conn_cache = make_array(make_sub_pool(session.pool), DEF_CONN_POOL_SIZE,
     sizeof(conn_entry_t));
 
-  pr_event_register(&sql_mysql_module, "core.exit", sql_mysql_exit_ev, NULL);  
-
   return 0;
 }
 
@@ -1395,8 +1451,11 @@ module sql_mysql_module = {
   NULL,
 
   /* Module initialization */
-  NULL,
+  sql_mysql_init,
 
   /* Session initialization */
-  sql_mysql_sess_init
+  sql_mysql_sess_init,
+
+  /* Module version */
+  MOD_SQL_MYSQL_VERSION
 };
