@@ -2,6 +2,7 @@
  * ProFTPD: mod_sql_postgres -- Support for connecting to Postgres databases.
  * Time-stamp: <1999-10-04 03:21:21 root>
  * Copyright (c) 2001 Andrew Houghton
+ * Copyright (c) 2004 TJ Saunders
  *  
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,7 +23,7 @@
  * the resulting executable, without including the source code for OpenSSL in
  * the source distribution.
  *
- * $Id: mod_sql_postgres.c,v 1.20 2004/05/08 03:21:46 castaglia Exp $
+ * $Id: mod_sql_postgres.c,v 1.27 2005/04/11 17:02:33 castaglia Exp $
  */
 
 /*
@@ -33,7 +34,7 @@
  * Internal define used for debug and logging.  All backends are encouraged
  * to use the same format.
  */
-#define _MOD_VERSION "mod_sql_postgres/4.02"
+#define MOD_SQL_POSTGRES_VERSION	"mod_sql_postgres/4.02"
 
 #define _POSTGRES_PORT "5432"
 
@@ -94,8 +95,8 @@ typedef struct conn_entry_struct conn_entry_t;
 
 #define DEF_CONN_POOL_SIZE 10
 
-pool *conn_pool = NULL;
-array_header *conn_cache = NULL;
+static pool *conn_pool = NULL;
+static array_header *conn_cache = NULL;
 
 /*
  *  _sql_get_connection: walks the connection cache looking for the named
@@ -155,8 +156,8 @@ static void *_sql_add_connection(pool *p, char *name, db_conn_t *conn)
  */
 static void _sql_check_cmd(cmd_rec *cmd, char *msg) {
   if ((!cmd) || (!cmd->tmp_pool)) {
-    pr_log_pri(PR_LOG_ERR, _MOD_VERSION ": '%s' was passed an invalid cmd_rec. "
-	    "Shutting down.", msg);
+    pr_log_pri(PR_LOG_ERR, MOD_SQL_POSTGRES_VERSION
+      ": '%s' was passed an invalid cmd_rec. Shutting down.", msg);
     sql_log(DEBUG_WARN, "'%s' was passed an invalid cmd_rec. Shutting down.",
       msg);
     end_login(1);
@@ -190,36 +191,17 @@ static int _sql_timer_callback(CALLBACK_FRAME) {
   return 0;
 }
 
-/* sql_postgres_exit_ev: walks the connection cache and closes every
- *  open connection, resetting their connection counts to 0.
- */
-static void sql_postgres_exit_ev(const void *event_data, void *user_data) {
-  register unsigned int i = 0;
-
-  for (i = 0; i < conn_cache->nelts; i++) {
-    conn_entry_t *entry = ((conn_entry_t **) conn_cache->elts)[i];
-
-    if (entry->connections > 0) {
-      cmd_rec *cmd = _sql_make_cmd(conn_pool, 2, entry->name, "1");
-      cmd_close(cmd);
-      SQL_FREE_CMD(cmd);
-    }
-  }
-
-  return;
-}
-
 /* 
  * _build_error: constructs a modret_t filled with error information;
  *  mod_sql_postgres calls this function and returns the resulting mod_ret_t
  *  whenever a call to the database results in an error.
  */
-static modret_t *_build_error( cmd_rec *cmd, db_conn_t *conn )
-{
+static modret_t *_build_error(cmd_rec *cmd, db_conn_t *conn) {
   if (!conn)
-    return PR_ERR_SQL_BADCMD(cmd);
+    return ERROR_MSG(cmd, MOD_SQL_POSTGRES_VERSION, "badly formed request");
 
-  return ERROR_MSG(cmd, "unknown", PQerrorMessage(conn->postgres));
+  return ERROR_MSG(cmd, MOD_SQL_POSTGRES_VERSION,
+    PQerrorMessage(conn->postgres));
 }
 
 /*
@@ -228,8 +210,7 @@ static modret_t *_build_error( cmd_rec *cmd, db_conn_t *conn )
  *  that data.  This is Postgres specific; other backends may choose 
  *  to do things differently.
  */
-static modret_t *_build_data( cmd_rec *cmd, db_conn_t *conn )
-{
+static modret_t *_build_data(cmd_rec *cmd, db_conn_t *conn) {
   PGresult *result = NULL;
   sql_data_t *sd = NULL;
   char **data = NULL;
@@ -238,7 +219,7 @@ static modret_t *_build_data( cmd_rec *cmd, db_conn_t *conn )
   int row =0;
 
   if (!conn) 
-    return PR_ERR_SQL_BADCMD(cmd);
+    return ERROR_MSG(cmd, MOD_SQL_POSTGRES_VERSION, "badly formed request");
 
   result = conn->result;
 
@@ -278,8 +259,7 @@ static modret_t *_build_data( cmd_rec *cmd, db_conn_t *conn )
  *  compatibility; a connection should not be closed unless the count
  *  reaches 0, and ideally will not need to be re-opened for counts > 1.
  */
-MODRET cmd_open(cmd_rec *cmd)
-{
+MODRET cmd_open(cmd_rec *cmd) {
   conn_entry_t *entry = NULL;
   db_conn_t *conn = NULL;
 
@@ -289,14 +269,14 @@ MODRET cmd_open(cmd_rec *cmd)
 
   if (cmd->argc < 1) {
     sql_log(DEBUG_FUNC, "%s", "exiting \tpostgres cmd_open");
-    return PR_ERR_SQL_BADCMD(cmd);
+    return ERROR_MSG(cmd, MOD_SQL_POSTGRES_VERSION, "badly formed request");
   }    
 
   /* get the named connection */
 
-  if (!(entry = _sql_get_connection( cmd->argv[0]))) {
+  if (!(entry = _sql_get_connection(cmd->argv[0]))) {
     sql_log(DEBUG_FUNC, "%s", "exiting \tpostgres cmd_open");
-    return PR_ERR_SQL_UNDEF(cmd);
+    return ERROR_MSG(cmd, MOD_SQL_POSTGRES_VERSION, "unknown named connection");
   } 
 
   conn = (db_conn_t *) entry->data;
@@ -307,9 +287,9 @@ MODRET cmd_open(cmd_rec *cmd)
   if ((entry->connections > 0) && 
       (PQstatus(conn->postgres) == CONNECTION_OK)) {
     entry->connections++;
-    if (entry->timer) {
-      reset_timer( entry->timer, &sql_postgres_module );
-    }
+    if (entry->timer) 
+      pr_timer_reset(entry->timer, &sql_postgres_module);
+
     sql_log(DEBUG_INFO, "connection '%s' count is now %d", entry->name,
       entry->connections);
     sql_log(DEBUG_FUNC, "%s", "exiting \tpostgres cmd_open");
@@ -330,9 +310,8 @@ MODRET cmd_open(cmd_rec *cmd)
 
   /* set up our timer if necessary */
   if (entry->ttl > 0) {
-    entry->timer = add_timer(entry->ttl, -1, 
-			     &sql_postgres_module, 
-			     _sql_timer_callback);
+    entry->timer = pr_timer_add(entry->ttl, -1, &sql_postgres_module,
+      _sql_timer_callback);
     sql_log(DEBUG_INFO, "connection '%s' - %d second timer started",
       entry->name, entry->ttl);
 
@@ -376,8 +355,7 @@ MODRET cmd_open(cmd_rec *cmd)
  *  If argv[1] exists and is not NULL, the connection should be immediately
  *  closed and the connection count should be reset to 0.
  */
-MODRET cmd_close(cmd_rec *cmd)
-{
+MODRET cmd_close(cmd_rec *cmd) {
   conn_entry_t *entry = NULL;
   db_conn_t *conn = NULL;
 
@@ -387,13 +365,13 @@ MODRET cmd_close(cmd_rec *cmd)
 
   if ((cmd->argc < 1) || (cmd->argc > 2)) {
     sql_log(DEBUG_FUNC, "%s", "exiting \tpostgres cmd_close");
-    return PR_ERR_SQL_BADCMD(cmd);
+    return ERROR_MSG(cmd, MOD_SQL_POSTGRES_VERSION, "badly formed request");
   }
 
   /* get the named connection */
-  if (!(entry = _sql_get_connection( cmd->argv[0] ))) {
+  if (!(entry = _sql_get_connection(cmd->argv[0]))) {
     sql_log(DEBUG_FUNC, "%s", "exiting \tpostgres cmd_close");
-    return PR_ERR_SQL_UNDEF(cmd);
+    return ERROR_MSG(cmd, MOD_SQL_POSTGRES_VERSION, "unknown named connection");
   }
 
   conn = (db_conn_t *) entry->data;
@@ -417,7 +395,7 @@ MODRET cmd_close(cmd_rec *cmd)
     entry->connections = 0;
 
     if (entry->timer) {
-      remove_timer( entry->timer, &sql_postgres_module );
+      pr_timer_remove(entry->timer, &sql_postgres_module);
       entry->timer = 0;
       sql_log(DEBUG_INFO, "connection '%s' - timer stopped", entry->name);
     }
@@ -455,8 +433,7 @@ MODRET cmd_close(cmd_rec *cmd)
  *  a number or ttl is negative, the connection will be assumed to have no
  *  associated timer.
  */
-MODRET cmd_defineconnection(cmd_rec *cmd)
-{
+MODRET cmd_defineconnection(cmd_rec *cmd) {
   char *info = NULL;
   char *name = NULL;
 
@@ -477,7 +454,7 @@ MODRET cmd_defineconnection(cmd_rec *cmd)
 
   if ((cmd->argc < 4) || (cmd->argc > 5) || (!cmd->argv[0])) {
     sql_log(DEBUG_FUNC, "%s", "exiting \tpostgres cmd_defineconnection");
-    return PR_ERR_SQL_BADCMD(cmd);
+    return ERROR_MSG(cmd, MOD_SQL_POSTGRES_VERSION, "badly formed request");
   }
 
   conn = (db_conn_t *) palloc(conn_pool, sizeof(db_conn_t));
@@ -529,7 +506,8 @@ MODRET cmd_defineconnection(cmd_rec *cmd)
   /* insert the new conn_info into the connection hash */
   if (!(entry = _sql_add_connection(conn_pool, name, (void *) conn))) {
     sql_log(DEBUG_FUNC, "%s", "exiting \tpostgres cmd_defineconnection");
-    return PR_ERR_SQL_REDEF(cmd);
+    return ERROR_MSG(cmd, MOD_SQL_POSTGRES_VERSION,
+      "named connection already exists");
   }
 
   entry->ttl = (cmd->argc == 5) ? 
@@ -551,6 +529,35 @@ MODRET cmd_defineconnection(cmd_rec *cmd)
   return HANDLED(cmd);
 }
 
+/*
+ * cmd_exit: closes all open connections.
+ *
+ * Inputs:
+ *  None
+ *
+ * Returns:
+ *  A simple non-error modret_t.
+ */
+static modret_t *cmd_exit(cmd_rec *cmd) {
+  register unsigned int i = 0;
+
+  sql_log(DEBUG_FUNC, "%s", "entering \tpostgres cmd_exit");
+
+  for (i = 0; i < conn_cache->nelts; i++) {
+    conn_entry_t *entry = ((conn_entry_t **) conn_cache->elts)[i];
+
+    if (entry->connections > 0) {
+      cmd_rec *close_cmd = _sql_make_cmd(conn_pool, 2, entry->name, "1");
+      cmd_close(close_cmd);
+      destroy_pool(close_cmd->pool);
+    }
+  }
+
+  sql_log(DEBUG_FUNC, "%s", "exiting \tpostgres cmd_exit");
+
+  return HANDLED(cmd);
+}
+
 /*
  * cmd_select: executes a SELECT query. properly constructing the query
  *  based on the inputs.  See mod_sql.h for the definition of the _sql_data
@@ -597,8 +604,7 @@ MODRET cmd_defineconnection(cmd_rec *cmd)
  *  certain selects could return huge amounts of data.  do whatever is
  *  possible to minimize the amount of data copying here.
  */
-MODRET cmd_select(cmd_rec *cmd)
-{
+MODRET cmd_select(cmd_rec *cmd) {
   conn_entry_t *entry = NULL;
   db_conn_t *conn = NULL;
   modret_t *cmr = NULL;
@@ -613,14 +619,14 @@ MODRET cmd_select(cmd_rec *cmd)
 
   if (cmd->argc < 2) {
     sql_log(DEBUG_FUNC, "%s", "exiting \tpostgres cmd_select");
-    return PR_ERR_SQL_BADCMD(cmd);
+    return ERROR_MSG(cmd, MOD_SQL_POSTGRES_VERSION, "badly formed request");
   }
 
   /* get the named connection */
-  entry = _sql_get_connection( cmd->argv[0] );
+  entry = _sql_get_connection(cmd->argv[0]);
   if (!entry) {
     sql_log(DEBUG_FUNC, "%s", "exiting \tpostgres cmd_select");
-    return PR_ERR_SQL_UNDEF(cmd);
+    return ERROR_MSG(cmd, MOD_SQL_POSTGRES_VERSION, "unknown named connection");
   }
   
   conn = (db_conn_t *) entry->data;
@@ -736,8 +742,7 @@ MODRET cmd_select(cmd_rec *cmd)
  * Notes:
  *  none
  */
-MODRET cmd_insert(cmd_rec *cmd)
-{
+MODRET cmd_insert(cmd_rec *cmd) {
   conn_entry_t *entry = NULL;
   db_conn_t *conn = NULL;
   modret_t *cmr = NULL;
@@ -751,14 +756,14 @@ MODRET cmd_insert(cmd_rec *cmd)
 
   if ((cmd->argc != 2) && (cmd->argc != 4)) {
     sql_log(DEBUG_FUNC, "%s", "exiting \tpostgres cmd_insert");
-    return PR_ERR_SQL_BADCMD(cmd);
+    return ERROR_MSG(cmd, MOD_SQL_POSTGRES_VERSION, "badly formed request");
   }
 
   /* get the named connection */
-  entry = _sql_get_connection( cmd->argv[0] );
+  entry = _sql_get_connection(cmd->argv[0]);
   if (!entry) {
     sql_log(DEBUG_FUNC, "%s", "exiting \tpostgres cmd_insert");
-    return PR_ERR_SQL_UNDEF(cmd);
+    return ERROR_MSG(cmd, MOD_SQL_POSTGRES_VERSION, "unknown named connection");
   }
 
   conn = (db_conn_t *) entry->data;
@@ -839,8 +844,7 @@ MODRET cmd_insert(cmd_rec *cmd)
  *  argv[3] is optional -- it may be NULL, or it may not exist at all.  
  *  make sure this is handled correctly. 
  */
-MODRET cmd_update(cmd_rec *cmd)
-{
+MODRET cmd_update(cmd_rec *cmd) {
   conn_entry_t *entry = NULL;
   db_conn_t *conn = NULL;
   modret_t *cmr = NULL;
@@ -854,14 +858,14 @@ MODRET cmd_update(cmd_rec *cmd)
 
   if ((cmd->argc < 2) || (cmd->argc > 4)) {
     sql_log(DEBUG_FUNC, "%s", "exiting \tpostgres cmd_update");
-    return PR_ERR_SQL_BADCMD(cmd);
+    return ERROR_MSG(cmd, MOD_SQL_POSTGRES_VERSION, "badly formed request");
   }
 
   /* get the named connection */
-  entry = _sql_get_connection( cmd->argv[0] );
+  entry = _sql_get_connection(cmd->argv[0]);
   if (!entry) {
     sql_log(DEBUG_FUNC, "%s", "exiting \tpostgres cmd_update");
-    return PR_ERR_SQL_UNDEF(cmd);
+    return ERROR_MSG(cmd, MOD_SQL_POSTGRES_VERSION, "unknown named connection");
   }
 
   conn = (db_conn_t *) entry->data;
@@ -931,22 +935,22 @@ MODRET cmd_update(cmd_rec *cmd)
  *  not support stored procedures should return an error with a descriptive
  *  error message (something like 'backend does not support procedures').
  */
-MODRET cmd_procedure(cmd_rec *cmd)
-{
+MODRET cmd_procedure(cmd_rec *cmd) {
   sql_log(DEBUG_FUNC, "%s", "entering \tpostgres cmd_procedure");
 
   _sql_check_cmd(cmd, "cmd_procedure");
 
   if (cmd->argc != 3) {
     sql_log(DEBUG_FUNC, "%s", "exiting \tpostgres cmd_procedure");
-    return PR_ERR_SQL_BADCMD(cmd);
+    return ERROR_MSG(cmd, MOD_SQL_POSTGRES_VERSION, "badly formed request");
   }
 
   /* PostgreSQL supports procedures, but the backend doesn't. */
 
   sql_log(DEBUG_FUNC, "%s", "exiting \tpostgres cmd_procedure");
 
-  return ERROR_MSG(cmd, _MOD_VERSION, "backend does not support procedures");
+  return ERROR_MSG(cmd, MOD_SQL_POSTGRES_VERSION,
+    "backend does not support procedures");
 }
 
 /*
@@ -968,8 +972,7 @@ MODRET cmd_procedure(cmd_rec *cmd)
  * Notes:
  *  None.
  */
-MODRET cmd_query(cmd_rec *cmd)
-{
+MODRET cmd_query(cmd_rec *cmd) {
   conn_entry_t *entry = NULL;
   db_conn_t *conn = NULL;
   modret_t *cmr = NULL;
@@ -983,14 +986,14 @@ MODRET cmd_query(cmd_rec *cmd)
 
   if (cmd->argc != 2) {
     sql_log(DEBUG_FUNC, "%s", "exiting \tpostgres cmd_query");
-    return PR_ERR_SQL_BADCMD(cmd);
+    return ERROR_MSG(cmd, MOD_SQL_POSTGRES_VERSION, "badly formed request");
   }
 
   /* get the named connection */
-  entry = _sql_get_connection( cmd->argv[0] );
+  entry = _sql_get_connection(cmd->argv[0]);
   if (!entry) {
     sql_log(DEBUG_FUNC, "%s", "exiting \tpostgres cmd_query");
-    return PR_ERR_SQL_UNDEF(cmd);
+    return ERROR_MSG(cmd, MOD_SQL_POSTGRES_VERSION, "unknown named connection");
   }
 
   conn = (db_conn_t *) entry->data;
@@ -1087,14 +1090,14 @@ MODRET cmd_escapestring(cmd_rec * cmd) {
 
   if (cmd->argc != 2) {
     sql_log(DEBUG_FUNC, "%s", "exiting \tpostgres cmd_escapestring");
-    return PR_ERR_SQL_BADCMD(cmd);
+    return ERROR_MSG(cmd, MOD_SQL_POSTGRES_VERSION, "badly formed request");
   }
 
   /* get the named connection */
-  entry = _sql_get_connection( cmd->argv[0] );
+  entry = _sql_get_connection(cmd->argv[0]);
   if (!entry) {
     sql_log(DEBUG_FUNC, "%s", "exiting \tpostgres cmd_escapestring");
-    return PR_ERR_SQL_UNDEF(cmd);
+    return ERROR_MSG(cmd, MOD_SQL_POSTGRES_VERSION, "unknown named connection");
   }
 
   conn = (db_conn_t *) entry->data;
@@ -1134,8 +1137,7 @@ MODRET cmd_escapestring(cmd_rec * cmd) {
  *  If this backend does not provide this functionality, this cmd *must*
  *  return ERROR.
  */
-MODRET cmd_checkauth(cmd_rec * cmd)
-{
+MODRET cmd_checkauth(cmd_rec * cmd) {
   conn_entry_t *entry = NULL;
   db_conn_t *conn = NULL;
 
@@ -1145,14 +1147,14 @@ MODRET cmd_checkauth(cmd_rec * cmd)
 
   if (cmd->argc != 3) {
     sql_log(DEBUG_FUNC, "%s", "exiting \tpostgres cmd_checkauth");
-    return PR_ERR_SQL_BADCMD(cmd);
+    return ERROR_MSG(cmd, MOD_SQL_POSTGRES_VERSION, "badly formed request");
   }
 
   /* get the named connection -- not used in this case, but for consistency */
-  entry = _sql_get_connection( cmd->argv[0] );
+  entry = _sql_get_connection(cmd->argv[0]);
   if (!entry) {
     sql_log(DEBUG_FUNC, "%s", "exiting \tpostgres cmd_checkauth");
-    return PR_ERR_SQL_UNDEF(cmd);
+    return ERROR_MSG(cmd, MOD_SQL_POSTGRES_VERSION, "unknown named connection");
   }
 
   conn = (db_conn_t *) entry->data;
@@ -1195,22 +1197,22 @@ MODRET cmd_identify(cmd_rec * cmd) {
   sd->rnum = 1;
   sd->fnum = 2;
 
-  sd->data[0] = _MOD_VERSION;
+  sd->data[0] = MOD_SQL_POSTGRES_VERSION;
   sd->data[1] = MOD_SQL_API_V1;
 
   return mod_create_data(cmd, (void *) sd);
 }  
 
-/* 
- * sql_cmdtable: mod_sql requires each backend module to define a cmdtable
+/* SQL cmdtable: mod_sql requires each backend module to define a cmdtable
  *  with this exact name. ALL these functions must be defined; mod_sql checks
  *  that they all exist on startup and ProFTPD will refuse to start if they
  *  aren't defined.
  */
-cmdtable sql_cmdtable[] = {
+static cmdtable sql_postgres_cmdtable[] = {
   { CMD, "sql_open",             G_NONE, cmd_open,             FALSE, FALSE },
   { CMD, "sql_close",            G_NONE, cmd_close,            FALSE, FALSE },
   { CMD, "sql_defineconnection", G_NONE, cmd_defineconnection, FALSE, FALSE },
+  { CMD, "sql_exit",             G_NONE, cmd_exit,             FALSE, FALSE },
   { CMD, "sql_select",           G_NONE, cmd_select,           FALSE, FALSE },
   { CMD, "sql_insert",           G_NONE, cmd_insert,           FALSE, FALSE },
   { CMD, "sql_update",           G_NONE, cmd_update,           FALSE, FALSE },
@@ -1223,17 +1225,57 @@ cmdtable sql_cmdtable[] = {
   { 0, NULL }
 };
 
-/*
- * sql_postgres_init: Used to initialize the connection cache and register
- *  the exit handler.
+/* Event handlers
  */
+
+static void sql_postgres_mod_load_ev(const void *event_data,
+    void *user_data) {
+
+  if (strcmp("mod_sql_postgres.c", (const char *) event_data) == 0) {
+    /* Register ourselves with mod_sql. */
+    if (sql_register_backend("postgres", sql_postgres_cmdtable) < 0) {
+      pr_log_pri(PR_LOG_NOTICE, MOD_SQL_POSTGRES_VERSION
+        ": notice: error registering backend: %s", strerror(errno));
+      end_login(1);
+    }
+  }
+}
+
+static void sql_postgres_mod_unload_ev(const void *event_data,
+    void *user_data) {
+
+  if (strcmp("mod_sql_postgres.c", (const char *) event_data) == 0) {
+    /* Unegister ourselves with mod_sql. */
+    if (sql_unregister_backend("postgres") < 0) {
+      pr_log_pri(PR_LOG_NOTICE, MOD_SQL_POSTGRES_VERSION
+        ": notice: error unregistering backend: %s", strerror(errno));
+      end_login(1);
+    }
+
+    /* Unregister ourselves from all events. */
+    pr_event_unregister(&sql_postgres_module, NULL, NULL);
+  }
+}
+
+/* Initialization routines
+ */
+
+static int sql_postgres_init(void) {
+
+  /* Register listeners for the load and unload events. */
+  pr_event_register(&sql_postgres_module, "core.module-load",
+    sql_postgres_mod_load_ev, NULL);
+  pr_event_register(&sql_postgres_module, "core.module-unload",
+    sql_postgres_mod_unload_ev, NULL);
+
+  return 0;
+}
+
 static int sql_postgres_sess_init(void) {
   conn_pool = make_sub_pool(session.pool);
   conn_cache = make_array(make_sub_pool(session.pool), DEF_CONN_POOL_SIZE,
     sizeof(conn_entry_t));
 
-  pr_event_register(&sql_postgres_module, "core.exit", sql_postgres_exit_ev,
-    NULL);
   return 0;
 }
 
@@ -1263,8 +1305,11 @@ module sql_postgres_module = {
   NULL,
 
   /* Module initialization */
-  NULL,
+  sql_postgres_init,
 
   /* Session initialization */
-  sql_postgres_sess_init
+  sql_postgres_sess_init,
+
+  /* Module version */
+  MOD_SQL_POSTGRES_VERSION
 };
