@@ -2,7 +2,7 @@
  * ProFTPD - FTP server daemon
  * Copyright (c) 1997, 1998 Public Flood Software
  * Copyright (c) 1999, 2000 MacGyver aka Habeeb J. Dihu <macgyver@tos.net>
- * Copyright (c) 2001, 2002, 2003 The ProFTPD Project team
+ * Copyright (c) 2001, 2002, 2003, 2004 The ProFTPD Project team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@
  */
 
 /* Generic configuration and standard header file includes.
- * $Id: conf.h,v 1.42 2003/09/11 06:36:11 castaglia Exp $
+ * $Id: conf.h,v 1.62 2005/06/30 00:39:16 castaglia Exp $
  */
 
 #ifndef PR_CONF_H
@@ -38,6 +38,15 @@
 #include "version.h"
 #include "config.h"
 
+/* Manually override the socklen_t type on HP-UX 11 boxes.  For more
+ * details on why, see:
+ *  http://nagoya.apache.org/bugzilla/show_bug.cgi?id=16317
+ */
+#if defined(HPUX11) && !defined(_XOPEN_SOURCE_EXTENDED)
+# undef socklen_t
+# define socklen_t      int
+#endif
+
 #include "default_paths.h"
 
 #include <stdlib.h>
@@ -54,6 +63,10 @@
 # include <sys/types.h>
 #endif
 
+#ifdef HAVE_INTTYPES_H
+# include <inttypes.h>
+#endif
+
 #ifdef HAVE_UNISTD_H
 # include <unistd.h>
 #endif
@@ -62,6 +75,10 @@
 # include <limits.h>
 #endif
 
+#ifdef HAVE_SYS_ACL_H
+# include <sys/acl.h>
+#endif
+
 #ifdef HAVE_SYS_SELECT_H
 # include <sys/select.h>
 #endif
@@ -199,6 +216,10 @@ char *strchr(),*strrchr();
 # include <fnmatch.h>
 #endif
 
+#ifdef HAVE_UTIME_H
+# include <utime.h>
+#endif
+
 #ifdef HAVE_UTMP_H
 # include <utmp.h>
 #endif
@@ -306,9 +327,11 @@ extern char *alloca();
 #endif
 
 #if defined(HAVE_LLU) && SIZEOF_OFF_T == 8
-# define PR_LU	"llu"
+# define PR_LU		"llu"
+# define pr_off_t	unsigned long long
 #else
-# define PR_LU	"lu"
+# define PR_LU		"lu"
+# define pr_off_t	unsigned long
 #endif
 
 /********************************************************************
@@ -339,11 +362,11 @@ typedef struct {
   char na_dnsstr[256];
   int na_have_dnsstr;
 
-#ifdef USE_IPV6
+#ifdef PR_USE_IPV6
   char na_ipstr[INET6_ADDRSTRLEN];
 #else
   char na_ipstr[INET_ADDRSTRLEN];
-#endif /* USE_IPV6 */
+#endif /* PR_USE_IPV6 */
   int na_have_ipstr;
 
   /* Note: at some point, this union might/should be replaced with
@@ -354,35 +377,48 @@ typedef struct {
 
   union {
     struct sockaddr_in v4;
-#ifdef USE_IPV6
+#ifdef PR_USE_IPV6
     struct sockaddr_in6 v6;
-#endif /* USE_IPV6 */
+#endif /* PR_USE_IPV6 */
   } na_addr;
 
 } pr_netaddr_t;
 
 #include "pool.h"
 #include "regexp.h"
+#include "table.h"
 #include "proftpd.h"
 #include "support.h"
 #include "sets.h"
 #include "dirtree.h"
 #include "netio.h"
 #include "modules.h"
+#include "auth.h"
 #include "response.h"
 #include "timers.h"
 #include "inet.h"
+#include "child.h"
 #include "netaddr.h"
+#include "netacl.h"
+#include "class.h"
 #include "bindings.h"
 #include "ident.h"
+#include "help.h"
 #include "feat.h"
 #include "ftp.h"
 #include "log.h"
+#include "parser.h"
+#include "xferlog.h"
 #include "scoreboard.h"
 #include "data.h"
+#include "display.h"
 #include "libsupp.h"
 #include "fsio.h"
 #include "mkhome.h"
+#include "ctrls.h"
+#include "event.h"
+#include "var.h"
+#include "compat.h"
 #include "pr-syslog.h"
 
 # ifdef HAVE_SETPASSENT
