@@ -25,7 +25,7 @@
  */
 
 /* Data connection management prototypes
- * $Id: data.h,v 1.15 2003/04/08 18:47:07 castaglia Exp $
+ * $Id: data.h,v 1.17 2004/11/04 22:48:17 castaglia Exp $
  */
 
 #ifndef PR_DATACONN_H
@@ -38,6 +38,7 @@ void pr_data_close(int);
 void pr_data_abort(int, int);
 int pr_data_xfer(char *, int);
 void pr_data_reset(void);
+void pr_data_set_linger(long);
 
 #ifdef HAVE_SENDFILE
 typedef
@@ -53,7 +54,7 @@ off_t
 
 pr_sendfile_t;
 
-pr_sendfile_t pr_data_sendfile(int retr_fd, off_t *offset, size_t count);
+pr_sendfile_t pr_data_sendfile(int retr_fd, off_t *offset, off_t count);
 #endif /* HAVE_SENDFILE */
 
 #endif /* PR_DATACONN_H */
