@@ -27,7 +27,7 @@
 /*
  * Configuration structure, server, command and associated prototypes.
  *
- * $Id: dirtree.h,v 1.44 2003/09/08 00:41:37 castaglia Exp $
+ * $Id: dirtree.h,v 1.60 2005/06/12 00:36:12 castaglia Exp $
  */
 
 #ifndef PR_DIRTREE_H
@@ -35,14 +35,14 @@
 
 #include "pool.h"
 #include "sets.h"
+#include "table.h"
 
 typedef struct config_struc config_rec;
-typedef struct privdata privdata_t;
 
 struct conn_struc;
 
 typedef struct server_struc {
-  struct server_struc *next,*prev;
+  struct server_struc *next, *prev;
 
   pool *pool;			/* Memory pool for this server */
   xaset_t *set;			/* Set holding all servers */
@@ -70,9 +70,11 @@ typedef struct server_struc {
   int tcp_sndbuf_len;
   unsigned char tcp_sndbuf_override;
 
-  char *ServerAdmin;		/* Administrator's name */
+  /* Administrator name */
+  char *ServerAdmin;
 
-  pr_netaddr_t *addr;		/* Internal address of this server */
+  /* Internal address of this server */
+  pr_netaddr_t *addr;
 
   /* The listener for this server.  Note that this listener, and that
    * pointed to by ipbind->ib_listener (where ipbind->ib_server points to
@@ -82,12 +84,14 @@ typedef struct server_struc {
    */
   struct conn_struc *listen;
 
-  xaset_t *conf;		/* Configuration details */
-
+  /* Configuration details */
+  xaset_t *conf;
   int config_type;
-} server_rec;
 
-#define CLASS_USER
+  /* Internal server ID, automatically assigned */
+  unsigned int sid;
+
+} server_rec;
 
 typedef struct cmd_struc {
   pool *pool;
@@ -104,8 +108,7 @@ typedef struct cmd_struc {
 
   int  class;			/* The command class */
   int  stash_index;		/* hack to speed up symbol hashing in modules.c */
-  privdata_t *private;		/* Private data for passing/retaining between handlers */
-  array_header *privarr;	/* Internal use */
+  pr_table_t *notes;		/* Private data for passing/retaining between handlers */
 } cmd_rec;
 
 struct config_struc {
@@ -132,7 +135,8 @@ struct config_struc {
 #define CONF_VIRTUAL		(1 << 4) /* Virtual host */
 #define CONF_DYNDIR		(1 << 5) /* .ftpaccess file */
 #define CONF_GLOBAL		(1 << 6) /* "Global" context (applies to main server and ALL virtualhosts */
-#define CONF_NAMED		(1 << 7)  /* Named virtual host */
+#define CONF_CLASS		(1 << 7) /* Class context */
+#define CONF_NAMED		(1 << 8) /* Named virtual host */
 #define CONF_USERDATA		(1 << 14) /* Runtime user data */
 #define CONF_PARAM		(1 << 15) /* config/args pair */
 
@@ -141,6 +145,7 @@ struct config_struc {
 #define CF_MERGEDOWN_MULTI	(1 << 1) /* Merge down, allowing multiple instances */
 #define CF_DYNAMIC		(1 << 2) /* Dynamically added entry */
 #define CF_DEFER		(1 << 3) /* Defer hashing until authentication */
+#define CF_SILENT		(1 << 4) /* Do not print a config dump when merging */
 
 /* Operation codes for dir_* funcs */
 #define OP_HIDE			1	/* Op for hiding dirs/files */
@@ -150,6 +155,11 @@ struct config_struc {
 #define ORDER_ALLOWDENY		0
 #define ORDER_DENYALLOW		1
 
+/* For the different types of expressions: AND, OR, and REGEX. */
+#define PR_EXPR_EVAL_AND	0
+#define PR_EXPR_EVAL_OR		1
+#define PR_EXPR_EVAL_REGEX	2
+
 /* The following macro determines the "highest" level available for
  * configuration directives.  If a current dir_config is available, it's
  * subset is used, otherwise anon config or main server
@@ -211,31 +221,31 @@ void kludge_enable_umask(void);
 
 unsigned char define_exists(const char *);
 void init_config(void);
-int fixup_servers(void);
-void set_config_stream(FILE *, unsigned int);
-char *get_config_line(char *, size_t);
-int parse_config_file(const char *);
+int fixup_servers(xaset_t *list);
+int parse_config_path(pool *, const char *);
 config_rec *add_config_set(xaset_t **, const char *);
-config_rec *add_config(const char *);
+config_rec *add_config(server_rec *, const char *);
 config_rec *add_config_param(const char *, int, ...);
 config_rec *add_config_param_str(const char *, int, ...);
 config_rec *add_config_param_set(xaset_t **, const char *, int, ...);
+config_rec *pr_conf_add_server_config_param_str(server_rec *, const char *,
+  int, ...);
 config_rec *find_config_next(config_rec *, config_rec *, int,
   const char *, int);
 config_rec *find_config(xaset_t *, int, const char *, int);
 void find_config_set_top(config_rec *);
 int remove_config(xaset_t *, const char *, int);
-class_t *find_class(pr_netaddr_t *, const char *);
 
 cmd_rec *pr_cmd_alloc(pool *, int, ...);
 
-array_header *pr_parse_expression(pool *, int *, char **);
-unsigned char pr_class_and_expression(char **);
-unsigned char pr_class_or_expression(char **);
-unsigned char pr_group_and_expression(char **);
-unsigned char pr_group_or_expression(char **);
-unsigned char pr_user_and_expression(char **);
-unsigned char pr_user_or_expression(char **);
+/* Expression API.  (XXX These should be in their own header/src files). */
+array_header *pr_expr_create(pool *, int *, char **);
+unsigned char pr_expr_eval_class_and(char **);
+unsigned char pr_expr_eval_class_or(char **);
+unsigned char pr_expr_eval_group_and(char **);
+unsigned char pr_expr_eval_group_or(char **);
+unsigned char pr_expr_eval_user_and(char **);
+unsigned char pr_expr_eval_user_or(char **);
 
 long get_param_int(xaset_t *, const char *, int);
 long get_param_int_next(const char *, int);
@@ -243,23 +253,19 @@ void *get_param_ptr(xaset_t *, const char *, int);
 void *get_param_ptr_next(const char *, int);
 xaset_t *get_dir_ctxt(pool *, char *);
 
-void init_conf_stacks(void);
-void free_conf_stacks(void);
-server_rec *start_new_server(const char *);
-server_rec *end_new_server(void);
-config_rec *start_sub_config(const char *);
-config_rec *end_sub_config(unsigned char *);
-char *get_word(char **, unsigned char);
+char *pr_str_get_word(char **, int);
+#define PR_STR_FL_PRESERVE_COMMENTS		0x0001
+#define PR_STR_FL_PRESERVE_WHITESPACE		0x0002
 
 config_rec *dir_match_path(pool *, char *);
 void build_dyn_config(pool *, char *, struct stat *, unsigned char);
 unsigned char dir_hide_file(const char *);
-int dir_check_op_mode(pool *, char *, int, uid_t, gid_t, mode_t);
 int dir_check_full(pool *, char *, char *, char *, int *);
 int dir_check_limits(config_rec *, char *, int);
 int dir_check(pool *, char *, char *, char *, int *);
 int dir_check_canon(pool *, char *, char *, char *, int *);
 int is_dotdir(const char *);
+int is_fnmatch(const char *);
 int login_check_limits(xaset_t *, int, int, int *);
 char *path_subst_uservar(pool *, char **);
 void resolve_anonymous_dirs(xaset_t *);
@@ -267,8 +273,10 @@ void resolve_deferred_dirs(server_rec *);
 void fixup_dirs(server_rec *, int);
 unsigned char check_context(cmd_rec *, int);
 char *get_context_name(cmd_rec *);
+int pr_is_boolean(const char *);
 int get_boolean(cmd_rec *, int);
 char *get_full_cmd(cmd_rec *);
-int match_ip(pr_netaddr_t *, const char *, const char *);
+
+void pr_config_dump(void (*)(const char *, ...), xaset_t *, char *);
 
 #endif /* PR_DIRTREE_H */
