@@ -27,7 +27,7 @@
 /* Logging, either to syslog or stderr, as well as debug logging
  * and debug levels.
  *
- * $Id: log.h,v 1.19 2003/08/06 22:03:32 castaglia Exp $
+ * $Id: log.h,v 1.25 2005/06/14 01:23:44 castaglia Exp $
  */
 
 #ifndef PR_LOG_H
@@ -45,6 +45,7 @@
  * info.  DEBUG0 (the default) prints nothing.
  */
 
+#define DEBUG10		10
 #define DEBUG9		9
 #define DEBUG8		8
 #define DEBUG7		7
@@ -56,53 +57,56 @@
 #define DEBUG1		1
 #define DEBUG0		0
 
-/* log_openfile() return values */
-#define LOG_WRITEABLE_DIR	-2
-#define LOG_SYMLINK		-3
+/* pr_log_openfile() return values */
+#define PR_LOG_WRITABLE_DIR	-2
+#define PR_LOG_SYMLINK		-3
 
 /* Log file modes */
-#define LOG_SYSTEM_MODE         0640
-#define LOG_XFER_MODE           0644
+#define PR_LOG_SYSTEM_MODE	0640
+#define PR_LOG_XFER_MODE	0644
 
-char *fmt_time(time_t);
 int log_wtmp(char *, const char *, const char *, pr_netaddr_t *);
-void log_setfacility(int);
-int log_openfile(const char *, int *, mode_t);
-int log_opensyslog(const char *);
+
+/* file-based logging functions */
+int pr_log_openfile(const char *, int *, mode_t);
+int pr_log_writefile(int, const char *, const char *, ...);
+
+/* syslog-based logging functions.  Note that the open/close functions are
+ * not part of the public API; use the pr_log_pri() function to log via
+ * syslog.
+ */
 void log_closesyslog(void);
+int log_opensyslog(const char *);
+void log_setfacility(int);
 
 /* Utilize gcc's __attribute__ pragma for signalling that it should perform
  * printf-style checking of this function's arguments.
  */
-void log_pri(int, const char *, ...)
+void pr_log_pri(int, const char *, ...)
 #ifdef __GNUC__
        __attribute__ ((format (printf, 2, 3)));
 #else
        ;
 #endif
 
-void log_auth(int, const char *, ...);
+void pr_log_auth(int, const char *, ...);
+
+/* Utilize gcc's __attribute__ pragma for signalling that it should perform
+ * printf-style checking of this function's arguments.
+ */
+void pr_log_debug(int, const char *, ...)
+#ifdef __GNUC__
+       __attribute__ ((format (printf, 2, 3)));
+#else
+       ;
+#endif
+
+int  pr_log_setdebuglevel(int);
+
 void log_stderr(int);
-int  log_setdebuglevel(int);
-
-/* Utilize gcc's __attribute__ pragma for signalling that it should perform
- * printf-style checking of this function's arguments.
- */
-void log_debug(int, const char *, ...)
-#ifdef __GNUC__
-       __attribute__ ((format (printf, 2, 3)));
-#else
-       ;
-#endif
-
 void log_discard(void);
 void init_log(void);
 
-int log_str2sysloglevel(const char *);
-
-/* TransferLog routines */
-int log_open_xfer(const char *);
-void log_close_xfer(void);
-int log_xfer(long, const char *, off_t, char *, char, char, char, char *, char);
+int pr_log_str2sysloglevel(const char *);
 
 #endif /* PR_LOG_H */
