@@ -1,6 +1,6 @@
 /*
  * ProFTPD - FTP server daemon
- * Copyright (c) 2003 The ProFTPD Project team
+ * Copyright (c) 2003-2005 The ProFTPD Project team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
  */
 
 /* Network address API
- * $Id: netaddr.h,v 1.17 2004/01/29 03:19:08 castaglia Exp $
+ * $Id: netaddr.h,v 1.19 2005/09/19 21:35:38 castaglia Exp $
  */
 
 #ifndef PR_NETADDR_H
@@ -60,7 +60,7 @@ struct addrinfo {
 };
 #endif /* HAVE_STRUCT_ADDRINFO */
 
-#if defined(HAVE_GETADDRINFO) && !defined(USE_GETADDRINFO)
+#if defined(HAVE_GETADDRINFO) && !defined(PR_USE_GETADDRINFO)
 /* Use the system getaddrinfo(2) and freeaddrinfo(2) by redefining the
  * 'pr_getaddrinfo' and 'pr_freeaddrinfo' symbols to be 'getaddrinfo' and
  * 'freeaddrinfo', respectively.
@@ -71,7 +71,7 @@ struct addrinfo {
 int pr_getaddrinfo(const char *, const char *, const struct addrinfo *,
   struct addrinfo **);
 void pr_freeaddrinfo(struct addrinfo *);
-#endif /* HAVE_GETNAMEINFO and !USE_GETNAMEINFO */
+#endif /* HAVE_GETNAMEINFO and !PR_USE_GETNAMEINFO */
 
 /* These AI_ defines are for use by getaddrinfo(3). */
 
@@ -142,7 +142,7 @@ void pr_freeaddrinfo(struct addrinfo *);
 # define EAI_SYSTEM     -11
 #endif /* EAI_SYSTEM */
 
-#if defined(HAVE_GETNAMEINFO) && !defined(USE_GETNAMEINFO)
+#if defined(HAVE_GETNAMEINFO) && !defined(PR_USE_GETNAMEINFO)
 /* Use the system getnameinfo(2) by redefining the 'pr_getnameinfo' symbol
  * to be simply 'getnameinfo'.
  */
@@ -150,7 +150,7 @@ void pr_freeaddrinfo(struct addrinfo *);
 #else
 int pr_getnameinfo(const struct sockaddr *, socklen_t, char *, size_t,
   char *, size_t, int);
-#endif /* HAVE_GETNAMEINFO and !USE_GETNAMEINFO */
+#endif /* HAVE_GETNAMEINFO and !PR_USE_GETNAMEINFO */
 
 /* These NI_ defines are for use by getnameinfo(3). */
 
@@ -352,4 +352,13 @@ int pr_netaddr_loopback(const pr_netaddr_t *);
  */
 int pr_netaddr_is_v4mappedv6(const pr_netaddr_t *);
 
+/* Return pointers to static memory which contains the local and remote
+ * netaddr information for the sesssion.  DO NOT MODIFY the pointed-to
+ * memory!  Returns NULL if no such session information exists.
+ */
+pr_netaddr_t *pr_netaddr_get_sess_local_addr(void);
+pr_netaddr_t *pr_netaddr_get_sess_remote_addr(void);
+const char *pr_netaddr_get_sess_remote_name(void);
+void pr_netaddr_set_sess_addrs(void);
+
 #endif /* PR_NETADDR_H */
