@@ -26,7 +26,7 @@
 
 /* User configurable defaults and tunable parameters.
  *
- * $Id: options.h,v 1.14 2003/01/02 17:28:17 castaglia Exp $
+ * $Id: options.h,v 1.21 2004/10/31 19:09:35 castaglia Exp $
  */
 
 #ifndef PR_OPTIONS_H
@@ -69,16 +69,19 @@
  * "tcpBackLog" configuration directive, this value is just the default.
  */
 
-#define PR_TUNABLE_DEFAULT_BACKLOG 5
+#define PR_TUNABLE_DEFAULT_BACKLOG	5
 
-/* The next two define the default receive/send tcp windows (and
- * internal ProFTPD buffer sizes.  These can be configured per server
- * or per virtual-server via the tcpReceiveWindow and tcpSendWindow
- * directives.
+/* The next two define the default receive/send TCP windows.  These can be
+ * configured on a per-vhost basis via the SocketOptions directive.
  */
 
-#define PR_TUNABLE_DEFAULT_RWIN    8192
-#define PR_TUNABLE_DEFAULT_SWIN    8192
+#ifndef PR_TUNABLE_RCVBUFSZ
+# define PR_TUNABLE_RCVBUFSZ		8192
+#endif
+
+#ifndef PR_TUNABLE_SNDBUFSZ
+# define PR_TUNABLE_SNDBUFSZ		8192
+#endif
 
 /* Default internal buffer size used for data transfers and other
  * miscellaneous tasks.
@@ -87,6 +90,26 @@
 # define PR_TUNABLE_BUFFER_SIZE	1024
 #endif
 
+/* This one is specifically for data transfers.
+ */
+#ifndef PR_TUNABLE_XFER_BUFFER_SIZE
+# define PR_TUNABLE_XFER_BUFFER_SIZE	PR_TUNABLE_BUFFER_SIZE
+#endif
+
+/* Maximum path length.  GNU HURD (and some others) do not define
+ * MAXPATHLEN.  POSIX' PATH_MAX is mandated to be at least 256 
+ * (according to some), so 1K, in the absense of MAXPATHLEN, should be
+ * a reasonable default.
+ */
+
+#ifndef PR_TUNABLE_PATH_MAX
+# ifdef MAXPATHLEN
+#  define PR_TUNABLE_PATH_MAX           MAXPATHLEN
+# else
+#  define PR_TUNABLE_PATH_MAX           1024
+# endif
+#endif
+
 /* Default timeouts, if not explicitly configured via
  * the TimeoutLogin, TimeoutIdle, etc directives.
  */
@@ -96,7 +119,7 @@
 #endif
 
 #ifndef PR_TUNABLE_TIMEOUTIDLE
-# define PR_TUNABLE_TIMEOUTIDLE	600
+# define PR_TUNABLE_TIMEOUTIDLE		600
 #endif
 
 #ifndef PR_TUNABLE_TIMEOUTLINGER
@@ -134,6 +157,24 @@
 # define PR_TUNABLE_SCOREBOARD_BUFFER_SIZE	80
 #endif
 
+/* Number of seconds between scoreboard scrubs, where the scoreboard is
+ * scanned for slots containing invalid PIDs.  Defaults to 30 seconds.
+ */
+
+#ifndef PR_TUNABLE_SCOREBOARD_SCRUB_TIMER
+# define PR_TUNABLE_SCOREBOARD_SCRUB_TIMER	30
+#endif
+
+/* Maximum number of attempted updates to the scoreboard during a
+ * file transfer before an actual write is done.  This is to allow
+ * an optimization where the scoreboard is not updated on every loop
+ * through the transfer buffer.
+ */
+
+#ifndef PR_TUNABLE_XFER_SCOREBOARD_UPDATES
+# define PR_TUNABLE_XFER_SCOREBOARD_UPDATES	10
+#endif
+
 /* Max number of recursion/directory levels to support when globbing.
  */
 
@@ -141,6 +182,18 @@
 # define PR_TUNABLE_GLOBBING_MAX	8
 #endif
 
+/* Maximum length of login name.
+ */
+
+#ifndef PR_TUNABLE_LOGIN_MAX
+/* Ideally, we'd use _POSIX_LOGIN_NAME_MAX here, if it was defined.  However,
+ * doing so would cause trouble for those sites that use databases for
+ * storing user information; such sites often use email addresses as
+ * login names.  Given that, let's use 256 as a login name size.
+ */
+# define PR_TUNABLE_LOGIN_MAX		256
+#endif
+
 /* Loopback network, this should generally not need to be changed,
  * although you can set a specific address by setting it to say
  * "127.0.0.1" and the netmask to "255.255.255.255".
