@@ -2,7 +2,7 @@
  * ProFTPD - FTP server daemon
  * Copyright (c) 1997, 1998 Public Flood Software
  * Copyright (c) 1999, 2000 MacGyver aka Habeeb J. Dihu <macgyver@tos.net>
- * Copyright (c) 2001, 2002, 2003 The ProFTPD Project team
+ * Copyright (c) 2001, 2002, 2003, 2004 The ProFTPD Project team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@
  */
 
 /* General options
- * $Id: proftpd.h,v 1.48 2004/01/29 22:20:52 castaglia Exp $
+ * $Id: proftpd.h,v 1.53 2005/02/26 17:28:58 castaglia Exp $
  */
 
 #ifndef PR_PROFTPD_H
@@ -41,6 +41,17 @@
 # define FALSE				0
 #endif
 
+/* Definition of root user/group IDs (non-Unix platforms may have these as
+ * different from 0/0).
+ */
+#ifdef __CYGWIN__
+# define PR_ROOT_UID    18
+# define PR_ROOT_GID    544
+#else
+# define PR_ROOT_UID    0
+# define PR_ROOT_GID    0
+#endif /* __CYGWIN__ */
+
 #define CALLBACK_FRAME	\
   unsigned long p1, unsigned long p2, unsigned long p3, void *data
 
@@ -51,8 +62,6 @@ typedef int (*callback_t)(CALLBACK_FRAME);
 
 struct conn_struc;
 struct cmd_struc;
-struct IO_Request;
-struct IO_File;
 struct config_struc;
 
 typedef struct {
@@ -108,6 +117,8 @@ typedef struct {
                                          * session.gid is changed
                                          */
 
+  pr_table_t *notes;			/* Session notes table */
+
   pr_class_t *class;			/* Session class */
   char *proc_prefix;			/* The "prefix" of our process name */
 
@@ -145,6 +156,12 @@ typedef struct {
     char *bufstart,*buf;
   } xfer;
 
+  /* Total number of bytes uploaded in this session. */
+  off_t total_bytes_in;
+
+  /* Total number of bytes downloaded in this session. */
+  off_t total_bytes_out;
+
   /* Total number of bytes transfered in this session. */
   off_t total_bytes;
 
@@ -173,8 +190,9 @@ extern array_header *daemon_gids;
 #define STOR_UNIQUE	3
 
 extern session_t	session;
-extern char		ServerType;
-extern char		MultilineRFC2228;
+extern char AddressCollisionCheck;
+extern char ServerType;
+extern char MultilineRFC2228;
 extern const char	*pwdfname,*grpfname;
 
 /* Session/State flags */
