@@ -24,7 +24,7 @@
 
 /* Scoreboard routines.
  *
- * $Id: scoreboard.h,v 1.8 2003/01/02 17:28:18 castaglia Exp $
+ * $Id: scoreboard.h,v 1.13 2004/11/02 18:18:58 castaglia Exp $
  */
 
 #ifndef PR_SCOREBOARD_H
@@ -32,7 +32,7 @@
 
 /* PR_SCOREBOARD_VERSION is used for checking for scoreboard compatibility
  */
-#define PR_SCOREBOARD_VERSION        		0x01040001
+#define PR_SCOREBOARD_VERSION        		0x01040002
 
 /* Structure used as a header for scoreboard files.
  */
@@ -62,15 +62,25 @@ typedef struct {
   uid_t sce_uid;
   gid_t sce_gid;
   char sce_user[32];
-  p_in_addr_t *sce_server_ip;
+
   int sce_server_port;
   char sce_server_addr[80], sce_server_label[32];
-  char sce_client_addr[16];
+
+#ifdef PR_USE_IPV6
+  char sce_client_addr[INET6_ADDRSTRLEN];
+#else
+  char sce_client_addr[INET_ADDRSTRLEN];
+#endif /* PR_USE_IPV6 */
   char sce_client_name[PR_TUNABLE_SCOREBOARD_BUFFER_SIZE];
+
   char sce_class[32];
   char sce_cwd[PR_TUNABLE_SCOREBOARD_BUFFER_SIZE];
-  char sce_cmd[PR_TUNABLE_SCOREBOARD_BUFFER_SIZE];
+
+  char sce_cmd[5];
+  char sce_cmd_arg[PR_TUNABLE_SCOREBOARD_BUFFER_SIZE];
+
   time_t sce_begin_idle, sce_begin_session;
+
   off_t sce_xfer_size, sce_xfer_done, sce_xfer_len;
   unsigned long sce_xfer_elapsed;
 
@@ -83,10 +93,10 @@ typedef struct {
 #define PR_SCORE_USER		1
 #define PR_SCORE_CLIENT_ADDR	2
 #define PR_SCORE_CLIENT_NAME	3
-#define PR_SCORE_CLASS		4	
+#define PR_SCORE_CLASS		4
 #define PR_SCORE_CWD		5
 #define PR_SCORE_CMD		6
-#define PR_SCORE_SERVER_IP	7
+#define PR_SCORE_CMD_ARG	7
 #define PR_SCORE_SERVER_PORT	8
 #define PR_SCORE_SERVER_ADDR	9
 #define PR_SCORE_SERVER_LABEL	10
@@ -107,12 +117,14 @@ int pr_set_scoreboard(const char *);
 
 int pr_close_scoreboard(void);
 void pr_delete_scoreboard(void);
-int pr_open_scoreboard(int, pid_t *);
+int pr_open_scoreboard(int);
 int pr_restore_scoreboard(void);
 int pr_rewind_scoreboard(void);
 
 int pr_scoreboard_add_entry(void);
 int pr_scoreboard_del_entry(unsigned char);
+pid_t pr_scoreboard_get_daemon_pid(void);
+time_t pr_scoreboard_get_daemon_uptime(void);
 pr_scoreboard_entry_t *pr_scoreboard_read_entry(void);
 int pr_scoreboard_update_entry(pid_t, ...);
 
