@@ -25,7 +25,7 @@
  */
 
 /* Non-specific support functions.
- * $Id: support.h,v 1.23 2004/06/07 22:58:44 castaglia Exp $
+ * $Id: support.h,v 1.25 2005/02/26 17:28:58 castaglia Exp $
  */
 
 #ifndef PR_SUPPORT_H
@@ -72,10 +72,6 @@ char *dir_canonical_path(pool *, const char *);
 char *dir_canonical_vpath(pool *, const char *);
 char *dir_best_path(pool *, const char *);
 
-void pr_exit_register_handler(void (*)(void));
-void remove_exit_handlers(void);
-void run_exit_handlers(void);
-
 void schedule(void (*f)(void *, void *, void *, void *), int, void *, void *,
   void *, void *);
 void run_schedule(void);
@@ -86,7 +82,6 @@ mode_t file_mode(char *);
 int file_exists(char *);
 int dir_exists(char *);
 int exists(char *);
-int access_check(char *, int);
 char *make_arg_str(pool *, int, char **);
 
 char *pr_str_strip(pool *, char *);
