@@ -2,7 +2,7 @@
  * ProFTPD: mod_auth_file - file-based authentication module that supports
  *                          restrictions on the file contents
  *
- * Copyright (c) 2002, 2003 The ProFTPD Project team
+ * Copyright (c) 2002-2005 The ProFTPD Project team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
  * distribute the resulting executable, without including the source code for
  * OpenSSL in the source distribution.
  *
- * $Id: mod_auth_file.c,v 1.9 2003/01/18 06:06:54 castaglia Exp $
+ * $Id: mod_auth_file.c,v 1.25 2005/07/03 18:52:02 castaglia Exp $
  */
 
 #include "conf.h"
@@ -31,11 +31,11 @@
 /* AIX has some rather stupid function prototype inconsistencies between
  * their crypt.h and stdlib.h's setkey() declarations.
  */
-#if defined(HAVE_CRYPT_H) && !defined(AIX4)
+#if defined(HAVE_CRYPT_H) && !defined(AIX4) && !defined(AIX5)
 # include <crypt.h>
 #endif
 
-#define MOD_AUTH_FILE_VERSION	"mod_auth_file/0.8.2"
+#define MOD_AUTH_FILE_VERSION	"mod_auth_file/0.8.3"
 
 /* Make sure the version of proftpd is as necessary. */
 #if PROFTPD_VERSION_NUMBER < 0x0001020702
@@ -167,6 +167,7 @@ static char *af_getgrentline(char **buf, int *buflen, FILE *fp) {
   char *cp = *buf;
 
   while (fgets(cp, (*buflen) - (cp - *buf), fp) != NULL) {
+    pr_signals_handle();
 
     /* Is this a full line? */
     if (strchr(cp, '\n'))
@@ -238,7 +239,7 @@ static struct group *af_getgrp(const char *buf) {
   }
 
   if (i < (NGRPFIELDS - 1)) {
-    log_pri(PR_LOG_ERR, "Malformed entry in group file: %s", buf);
+    pr_log_pri(PR_LOG_ERR, "Malformed entry in group file: %s", buf);
     return NULL;
   }
 
@@ -286,14 +287,14 @@ static unsigned char af_allow_grent(authfile_file_t *groupf,
   if (groupf->af_restricted_ids) {
 
     if (grp->gr_gid < groupf->af_min_id.gid) {
-      log_debug(DEBUG3, MOD_AUTH_FILE_VERSION ": skipping group '%s': "
+      pr_log_debug(DEBUG3, MOD_AUTH_FILE_VERSION ": skipping group '%s': "
         "GID (%u) below the minimum allowed (%u)", grp->gr_name,
         (unsigned int) grp->gr_gid, (unsigned int) groupf->af_min_id.gid);
       return FALSE;
     }
 
     if (grp->gr_gid > groupf->af_max_id.gid) {
-      log_debug(DEBUG3, MOD_AUTH_FILE_VERSION ": skipping group '%s': "
+      pr_log_debug(DEBUG3, MOD_AUTH_FILE_VERSION ": skipping group '%s': "
         "GID (%u) above the maximum allowed (%u)", grp->gr_name,
         (unsigned int) grp->gr_gid, (unsigned int) groupf->af_max_id.gid);
       return FALSE;
@@ -307,7 +308,7 @@ static unsigned char af_allow_grent(authfile_file_t *groupf,
 
     if ((res != 0 && !groupf->af_name_regex_inverted) ||
         (res == 0 && groupf->af_name_regex_inverted)) {
-      log_debug(DEBUG3, MOD_AUTH_FILE_VERSION ": skipping group '%s': "
+      pr_log_debug(DEBUG3, MOD_AUTH_FILE_VERSION ": skipping group '%s': "
         "name '%s' does not meet allowed filter '%s'", grp->gr_name,
         grp->gr_name, groupf->af_name_filter);
       return FALSE;
@@ -330,11 +331,15 @@ static struct group *af_getgrent(authfile_file_t *groupf) {
 
   while (TRUE) {
 #ifdef HAVE_FGETGRENT
+    pr_signals_handle();
     grp = fgetgrent(groupf->af_file);
 #else
-    char *cp = NULL, *buf = malloc(BUFSIZ);
+    char *cp = NULL, *buf = NULL;
     int buflen = BUFSIZ;
 
+    pr_signals_handle();
+
+    buf = malloc(BUFSIZ);
     if (!buf)
       return NULL;
 
@@ -401,7 +406,7 @@ static unsigned char af_setgrent(void) {
 
     if (!af_open_file(af_current_group_file)) {
       /* Log the error */
-      log_pri(PR_LOG_ERR, "error: unable to open group file '%s': %s",
+      pr_log_pri(PR_LOG_ERR, "error: unable to open group file '%s': %s",
         af_current_group_file->af_path, strerror(errno));
 
       /* Move to the next file in the list. */
@@ -409,7 +414,7 @@ static unsigned char af_setgrent(void) {
       continue;
 
     } else {
-      log_debug(DEBUG7, MOD_AUTH_FILE_VERSION ": using group file '%s'",
+      pr_log_debug(DEBUG7, MOD_AUTH_FILE_VERSION ": using group file '%s'",
         af_current_group_file->af_path);
       return TRUE;
     }
@@ -425,14 +430,14 @@ static unsigned char af_allow_pwent(authfile_file_t *passwdf,
   if (passwdf->af_restricted_ids) {
 
     if (pwd->pw_uid < passwdf->af_min_id.uid) {
-      log_debug(DEBUG3, MOD_AUTH_FILE_VERSION ": skipping user '%s': "
+      pr_log_debug(DEBUG3, MOD_AUTH_FILE_VERSION ": skipping user '%s': "
         "UID (%u) below the minimum allowed (%u)", pwd->pw_name,
         (unsigned int) pwd->pw_uid, (unsigned int) passwdf->af_min_id.uid);
       return FALSE;
     }
 
     if (pwd->pw_uid > passwdf->af_max_id.gid) {
-      log_debug(DEBUG3, MOD_AUTH_FILE_VERSION ": skipping user '%s': "
+      pr_log_debug(DEBUG3, MOD_AUTH_FILE_VERSION ": skipping user '%s': "
         "UID (%u) above the maximum allowed (%u)", pwd->pw_name,
         (unsigned int) pwd->pw_uid, (unsigned int) passwdf->af_max_id.uid);
       return FALSE;
@@ -446,7 +451,7 @@ static unsigned char af_allow_pwent(authfile_file_t *passwdf,
 
     if ((res != 0 && !passwdf->af_name_regex_inverted) ||
         (res == 0 && passwdf->af_name_regex_inverted)) {
-      log_debug(DEBUG3, MOD_AUTH_FILE_VERSION ": skipping user '%s': "
+      pr_log_debug(DEBUG3, MOD_AUTH_FILE_VERSION ": skipping user '%s': "
         "name '%s' does not meet allowed filter '%s'", pwd->pw_name,
         pwd->pw_name, passwdf->af_name_filter);
       return FALSE;
@@ -460,7 +465,7 @@ static unsigned char af_allow_pwent(authfile_file_t *passwdf,
 
     if ((res != 0 && !passwdf->af_home_regex_inverted) ||
         (res == 0 && passwdf->af_home_regex_inverted)) {
-      log_debug(DEBUG3, MOD_AUTH_FILE_VERSION ": skipping user '%s': "
+      pr_log_debug(DEBUG3, MOD_AUTH_FILE_VERSION ": skipping user '%s': "
         "home '%s' does not meet allowed filter '%s'", pwd->pw_name,
         pwd->pw_dir, passwdf->af_home_filter);
       return FALSE;
@@ -483,11 +488,14 @@ static struct passwd *af_getpwent(authfile_file_t *passwdf) {
 
   while (TRUE) {
 #ifdef HAVE_FGETPWENT
+    pr_signals_handle();
     pwd = fgetpwent(passwdf->af_file);
 #else
     char buf[BUFSIZ] = {'\0'};
 
+    pr_signals_handle();
     while (fgets(buf, sizeof(buf), passwdf->af_file) != (char*) 0) {
+      pr_signals_handle();
 
       /* Ignore empty and comment lines */
       if (buf[0] == '\0' || buf[0] == '#')
@@ -552,7 +560,7 @@ static unsigned char af_setpwent(void) {
 
     if (!af_open_file(af_current_user_file)) {
       /* Log the error */
-      log_pri(PR_LOG_ERR, "error: unable to open passwd file '%s': %s",
+      pr_log_pri(PR_LOG_ERR, "error: unable to open passwd file '%s': %s",
         af_current_user_file->af_path, strerror(errno));
 
       /* Move to the next file in the list. */
@@ -560,7 +568,7 @@ static unsigned char af_setpwent(void) {
       continue;
 
     } else {
-      log_debug(DEBUG7, MOD_AUTH_FILE_VERSION ": using passwd file '%s'",
+      pr_log_debug(DEBUG7, MOD_AUTH_FILE_VERSION ": using passwd file '%s'",
         af_current_user_file->af_path);
       return TRUE;
     }
@@ -580,7 +588,7 @@ MODRET authfile_endpwent(cmd_rec *cmd) {
 
   af_endpwent();
 
-  return HANDLED(cmd);
+  return DECLINED(cmd);
 }
 
 MODRET authfile_getpwent(cmd_rec *cmd) {
@@ -621,7 +629,7 @@ MODRET authfile_getpwnam(cmd_rec *cmd) {
 
 MODRET authfile_getpwuid(cmd_rec *cmd) {
   struct passwd *pwd = NULL;
-  uid_t uid = (uid_t) cmd->argv[0];
+  uid_t uid = *((uid_t *) cmd->argv[0]);
 
   /* Do not handle *pw* requests unless we can do so. */
   if (!af_handle_pw)
@@ -647,7 +655,7 @@ MODRET authfile_name2uid(cmd_rec *cmd) {
 
   pwd = af_getpwnam(af_current_user_file, cmd->argv[0]);
 
-  return pwd ? mod_create_data(cmd, (void *) pwd->pw_uid) : DECLINED(cmd);
+  return pwd ? mod_create_data(cmd, (void *) &pwd->pw_uid) : DECLINED(cmd);
 }
 
 MODRET authfile_setpwent(cmd_rec *cmd) {
@@ -657,9 +665,9 @@ MODRET authfile_setpwent(cmd_rec *cmd) {
     return DECLINED(cmd);
 
   if (af_setpwent())
-    return HANDLED(cmd);
+    return DECLINED(cmd);
 
-  log_debug(DEBUG2,
+  pr_log_debug(DEBUG2,
     MOD_AUTH_FILE_VERSION ": unable to find useable AuthUserFile");
 
   return DECLINED(cmd);
@@ -675,7 +683,7 @@ MODRET authfile_uid2name(cmd_rec *cmd) {
   if (!af_setpwent())
     return DECLINED(cmd);
 
-  pwd = af_getpwuid(af_current_user_file, (uid_t) cmd->argv[0]);
+  pwd = af_getpwuid(af_current_user_file, *((uid_t *) cmd->argv[0]));
 
   return pwd ? mod_create_data(cmd, pwd->pw_name) : DECLINED(cmd);
 }
@@ -688,7 +696,7 @@ MODRET authfile_endgrent(cmd_rec *cmd) {
 
   af_endgrent();
 
-  return HANDLED(cmd);
+  return DECLINED(cmd);
 }
 
 MODRET authfile_getgrent(cmd_rec *cmd) {
@@ -708,7 +716,7 @@ MODRET authfile_getgrent(cmd_rec *cmd) {
 
 MODRET authfile_getgrgid(cmd_rec *cmd) {
   struct group *grp = NULL;
-  gid_t gid = (gid_t) cmd->argv[0];
+  gid_t gid = *((gid_t *) cmd->argv[0]);
 
   /* Do not handle *gr* requests unless we can do so. */
   if (!af_handle_gr)
@@ -775,7 +783,19 @@ MODRET authfile_getgroups(cmd_rec *cmd) {
 
   if (groups &&
       (grp = af_getgrgid(af_current_group_file, pwd->pw_gid)) != NULL)
-    *((char **) push_array(groups)) = pstrdup(permanent_pool, grp->gr_name);
+    *((char **) push_array(groups)) = pstrdup(session.pool, grp->gr_name);
+
+  /* The above call to af_getgrgid() will position the file pointer in
+   * the AuthGroupFile just after the group with the primary GID.
+   * Subsequently, the below af_getgrent() starts from that position, and
+   * goes to the end of the file.  The problem is that there may be groups
+   * before the primary GID for the current group.  So, ideally, the
+   * getgrent() loop would continue until we're back to where we are now,
+   * rather than stopping at the end of the file.  Conversely, we could
+   * just simply rewind to the start of the AuthGroupFile (which is easier).
+   * The core auth code will remove duplicate IDs as needed.
+   */
+  af_open_file(af_current_group_file);
 
   /* This is where things get slow, expensive, and ugly.  Loop through
    * everything, checking to make sure we haven't already added it.
@@ -794,18 +814,17 @@ MODRET authfile_getgroups(cmd_rec *cmd) {
         if (gids)
           *((gid_t *) push_array(gids)) = grp->gr_gid;
 
-        if (groups && pwd->pw_gid != grp->gr_gid)
-          *((char **) push_array(groups)) = pstrdup(permanent_pool,
-            grp->gr_name);
+        if (groups)
+          *((char **) push_array(groups)) = pstrdup(session.pool, grp->gr_name);
       }
     }
   }
 
   if (gids && gids->nelts > 0)
-    return mod_create_data(cmd, (void *) gids->nelts);
+    return mod_create_data(cmd, (void *) &gids->nelts);
 
   else if (groups && groups->nelts > 0)
-    return mod_create_data(cmd, (void *) groups->nelts);
+    return mod_create_data(cmd, (void *) &groups->nelts);
 
   return DECLINED(cmd);
 }
@@ -820,7 +839,7 @@ MODRET authfile_gid2name(cmd_rec *cmd) {
   if (!af_setgrent())
     return DECLINED(cmd);
 
-  grp = af_getgrgid(af_current_group_file, (gid_t) cmd->argv[0]);
+  grp = af_getgrgid(af_current_group_file, *((gid_t *) cmd->argv[0]));
 
   return grp ? mod_create_data(cmd, grp->gr_name) : DECLINED(cmd);
 }
@@ -837,7 +856,7 @@ MODRET authfile_name2gid(cmd_rec *cmd) {
 
   grp = af_getgrnam(af_current_group_file, cmd->argv[0]);
 
-  return grp ? mod_create_data(cmd, (void *) grp->gr_gid) : DECLINED(cmd);
+  return grp ? mod_create_data(cmd, (void *) &grp->gr_gid) : DECLINED(cmd);
 }
 
 MODRET authfile_setgrent(cmd_rec *cmd) {
@@ -847,9 +866,9 @@ MODRET authfile_setgrent(cmd_rec *cmd) {
     return DECLINED(cmd);
 
   if (af_setgrent())
-    return HANDLED(cmd);
+    return DECLINED(cmd);
 
-  log_debug(DEBUG2,
+  pr_log_debug(DEBUG2,
     MOD_AUTH_FILE_VERSION ": unable to find useable AuthGroupFile");
 
   return DECLINED(cmd);
@@ -868,12 +887,26 @@ MODRET authfile_auth(cmd_rec *cmd) {
     return DECLINED(cmd);
 
   /* Lookup the cleartxt password for this user. */
-  if ((tmp = af_getpwpass(af_current_user_file, name)) == NULL)
+  if ((tmp = af_getpwpass(af_current_user_file, name)) == NULL) {
+
+    /* For now, return DECLINED.  Ideally, we could stash an auth module
+     * identifier in the session structure, so that all auth modules could
+     * coordinate/use their methods as long as they matched the auth module
+     * used.
+     */
+    return DECLINED(cmd);
+
+#if 0
+    /* When the above is implemented, and if the user being checked was
+     * provided by mod_auth_file, we'd return this.
+     */
     return ERROR_INT(cmd, PR_AUTH_NOPWD);
+#endif
+  }
 
   cleartxt_pass = pstrdup(cmd->tmp_pool, tmp);
 
-  if (auth_check(cmd->tmp_pool, cleartxt_pass, name, cmd->argv[1]))
+  if (pr_auth_check(cmd->tmp_pool, cleartxt_pass, name, cmd->argv[1]))
     return ERROR_INT(cmd, PR_AUTH_BADPWD);
 
   if (lstchg > 0 && max > 0 && inact > 0 && now > (lstchg + max + inact))
@@ -882,6 +915,7 @@ MODRET authfile_auth(cmd_rec *cmd) {
   if (disable > 0 && now > disable)
     return ERROR_INT(cmd, PR_AUTH_DISABLEDPWD);
 
+  session.auth_mech = "mod_auth_file.c";
   return HANDLED(cmd);
 }
 
@@ -893,8 +927,10 @@ MODRET authfile_chkpass(cmd_rec *cmd) {
   if (!af_handle_pw)
     return DECLINED(cmd);
 
-  if (!strcmp(crypt(cleartxt_pass, ciphertxt_pass), ciphertxt_pass))
+  if (strcmp(crypt(cleartxt_pass, ciphertxt_pass), ciphertxt_pass) == 0) {
+    session.auth_mech = "mod_auth_file.c";
     return HANDLED(cmd);
+  }  
 
   return DECLINED(cmd);
 }
@@ -1158,8 +1194,10 @@ static int authfile_sess_init(void) {
  */
 c->name = "";
 
-    file->af_next = af_user_file_list;
-    af_user_file_list = file;
+    if (!af_user_file_list) {
+      file->af_next = af_user_file_list;
+      af_user_file_list = file;
+    }
 
     c = find_config_next(c, c->next, CONF_PARAM, "AuthUserFile", FALSE);
   }
@@ -1176,8 +1214,10 @@ c->name = "";
  */
 c->name = "";
 
-    file->af_next = af_group_file_list;
-    af_group_file_list = file;
+    if (!af_group_file_list) {
+      file->af_next = af_group_file_list;
+      af_group_file_list = file;
+    }
 
     c = find_config_next(c, c->next, CONF_PARAM, "AuthGroupFile", FALSE);
   }
@@ -1207,9 +1247,9 @@ static authtable authfile_authtab[] = {
   { 0, "getpwent",	authfile_getpwent },
   { 0, "getpwnam",	authfile_getpwnam },
   { 0, "getpwuid",	authfile_getpwuid },
-  { 0, "name_uid",	authfile_name2uid },
+  { 0, "name2uid",	authfile_name2uid },
   { 0, "setpwent",	authfile_setpwent },
-  { 0, "uid_name",	authfile_uid2name },
+  { 0, "uid2name",	authfile_uid2name },
 
   /* Group information callbacks */
   { 0, "endgrent",	authfile_endgrent },
@@ -1217,8 +1257,8 @@ static authtable authfile_authtab[] = {
   { 0, "getgrgid",	authfile_getgrgid },
   { 0, "getgrnam",	authfile_getgrnam },
   { 0, "getgroups",	authfile_getgroups },
-  { 0, "gid_name",	authfile_gid2name },
-  { 0, "name_gid",	authfile_name2gid },
+  { 0, "gid2name",	authfile_gid2name },
+  { 0, "name2gid",	authfile_name2gid },
   { 0, "setgrent",	authfile_setgrent },
 
   /* Miscellaneous callbacks */
