@@ -2,7 +2,7 @@
  * ProFTPD - FTP server daemon
  * Copyright (c) 1997, 1998 Public Flood Software
  * Copyright (c) 1999, 2000 MacGyver aka Habeeb J. Dihu <macgyver@tos.net>
- * Copyright (c) 2001, 2002, 2003 The ProFTPD Project team
+ * Copyright (c) 2001-2005 The ProFTPD Project team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@
  */
 
 /* Unix authentication module for ProFTPD
- * $Id: mod_auth_unix.c,v 1.11 2003/09/28 23:56:22 castaglia Exp $
+ * $Id: mod_auth_unix.c,v 1.25 2005/07/03 18:52:02 castaglia Exp $
  */
 
 #include "conf.h"
@@ -33,11 +33,11 @@
 /* AIX has some rather stupid function prototype inconsistencies between
  * their crypt.h and stdlib.h's setkey() declarations.  *sigh*
  */
-#if defined(HAVE_CRYPT_H) && !defined(AIX4)
+#if defined(HAVE_CRYPT_H) && !defined(AIX4) && !defined(AIX5)
 # include <crypt.h>
 #endif
 
-#ifdef USE_SHADOW
+#ifdef PR_USE_SHADOW
 # include <shadow.h>
 #endif
 
@@ -60,14 +60,14 @@
 # include <prot.h>
 #endif
 
-#ifdef USE_SIA
+#ifdef PR_USE_SIA
 # ifdef HAVE_SIA_H
 #  include <sia.h>
 # endif
 # ifdef HAVE_SIAD_H
 #  include <siad.h>
 # endif
-#endif /* USE_SIA */
+#endif /* PR_USE_SIA */
 
 #ifdef CYGWIN
 typedef void *HANDLE;
@@ -113,6 +113,8 @@ typedef struct _idmap {
   char *name;			/* user or group name */
 } idmap_t;
 
+module auth_unix_module;
+
 static xaset_t *uid_table[HASH_TABLE_SIZE];
 static xaset_t *gid_table[HASH_TABLE_SIZE];
 
@@ -131,14 +133,14 @@ static int persistent_passwdf = 0, persistent_groupf = 0;
 #undef GROUP
 #define	GROUP		grpfname
 
-#ifdef USE_SHADOW
+#ifdef PR_USE_SHADOW
 
 /* Shadow password entries are stored as number of days, not seconds
  * and are -1 if unused
  */
 #define SP_CVT_DAYS(x)	((x) == (time_t)-1 ? (x) : ((x) * 86400))
 
-#endif /* USE_SHADOW */
+#endif /* PR_USE_SHADOW */
 
 static void p_setpwent(void) {
   if (pwdf)
@@ -146,7 +148,7 @@ static void p_setpwent(void) {
 
   else
     if ((pwdf = fopen(PASSWD,"r")) == NULL)
-      log_pri(PR_LOG_ERR, "Unable to open password file %s for reading: %s",
+      pr_log_pri(PR_LOG_ERR, "Unable to open password file %s for reading: %s",
         PASSWD, strerror(errno));
 }
 
@@ -163,7 +165,7 @@ static RETSETGRENTTYPE p_setgrent(void) {
 
   else
     if ((grpf = fopen(GROUP,"r")) == NULL)
-      log_pri(PR_LOG_ERR, "Unable to open group file %s for reading: %s",
+      pr_log_pri(PR_LOG_ERR, "Unable to open group file %s for reading: %s",
         GROUP, strerror(errno));
 
 #ifndef SETGRENT_VOID
@@ -289,7 +291,7 @@ static idmap_t *_auth_lookup_id(xaset_t **id_table, idauth_t id) {
 
   if (!m || !_compare_id(id_table, m->id, id)) {
     /* Isn't in the table */
-    m = (idmap_t *) pcalloc(id_table[hash]->mempool, sizeof(idmap_t));
+    m = (idmap_t *) pcalloc(id_table[hash]->pool, sizeof(idmap_t));
 
     if (id_table == uid_table)
       m->id.uid = id.uid;
@@ -308,7 +310,7 @@ MODRET pw_setpwent(cmd_rec *cmd) {
   else
     setpwent();
 
-  return HANDLED(cmd);
+  return DECLINED(cmd);
 }
 
 MODRET pw_endpwent(cmd_rec *cmd) {
@@ -317,7 +319,7 @@ MODRET pw_endpwent(cmd_rec *cmd) {
   else
     endpwent();
 
-  return HANDLED(cmd);
+  return DECLINED(cmd);
 }
 
 MODRET pw_setgrent(cmd_rec *cmd) {
@@ -326,7 +328,7 @@ MODRET pw_setgrent(cmd_rec *cmd) {
   else
     setgrent();
 
-  return HANDLED(cmd);
+  return DECLINED(cmd);
 }
 
 MODRET pw_endgrent(cmd_rec *cmd) {
@@ -335,7 +337,7 @@ MODRET pw_endgrent(cmd_rec *cmd) {
   else
     endgrent();
 
-  return HANDLED(cmd);
+  return DECLINED(cmd);
 }
 
 MODRET pw_getgrent(cmd_rec *cmd) {
@@ -346,10 +348,7 @@ MODRET pw_getgrent(cmd_rec *cmd) {
   else
     gr = getgrent();
 
-  if (gr)
-    return mod_create_data(cmd,gr);
-  else
-    return ERROR(cmd);
+  return gr ? mod_create_data(cmd, gr) : DECLINED(cmd);
 }
 
 MODRET pw_getpwent(cmd_rec *cmd) {
@@ -360,26 +359,20 @@ MODRET pw_getpwent(cmd_rec *cmd) {
   else
     pw = getpwent();
 
-  if (pw)
-    return mod_create_data(cmd,pw);
-  else
-    return ERROR(cmd);
+  return pw ? mod_create_data(cmd, pw) : DECLINED(cmd);
 }
 
 MODRET pw_getpwuid(cmd_rec *cmd) {
   struct passwd *pw;
   uid_t uid;
 
-  uid = (uid_t)cmd->argv[0];
+  uid = *((uid_t *) cmd->argv[0]);
   if (PERSISTENT_PASSWD)
     pw = p_getpwuid(uid);
   else
     pw = getpwuid(uid);
 
-  if (pw)
-    return mod_create_data(cmd,pw);
-  else
-    return ERROR(cmd);
+  return pw ? mod_create_data(cmd, pw) : DECLINED(cmd);
 }
 
 MODRET pw_getpwnam(cmd_rec *cmd) {
@@ -392,10 +385,7 @@ MODRET pw_getpwnam(cmd_rec *cmd) {
   else
     pw = getpwnam(name);
 
-  if (pw)
-    return mod_create_data(cmd,pw);
-  else
-    return ERROR(cmd);
+  return pw ? mod_create_data(cmd, pw) : DECLINED(cmd);
 }
 
 MODRET pw_getgrnam(cmd_rec *cmd) {
@@ -408,35 +398,33 @@ MODRET pw_getgrnam(cmd_rec *cmd) {
   else
     gr = getgrnam(name);
 
-  if (gr)
-    return mod_create_data(cmd,gr);
-  else
-    return ERROR(cmd);
+  return gr ? mod_create_data(cmd, gr) : DECLINED(cmd);
 }
 
 MODRET pw_getgrgid(cmd_rec *cmd) {
   struct group *gr;
   gid_t gid;
 
-  gid = (gid_t)cmd->argv[0];
+  gid = *((gid_t *) cmd->argv[0]);
   if (PERSISTENT_GROUP)
     gr = p_getgrgid(gid);
   else
     gr = getgrgid(gid);
 
-  if (gr)
-    return mod_create_data(cmd,gr);
-  else
-    return ERROR(cmd);
+  return gr ? mod_create_data(cmd, gr) : DECLINED(cmd);
 }
 
-#ifdef USE_SHADOW
+#ifdef PR_USE_SHADOW
 static char *_get_pw_info(pool *p, const char *u, time_t *lstchg, time_t *min,
     time_t *max, time_t *warn, time_t *inact, time_t *expire) {
   struct spwd *sp;
   char *cpw = NULL;
 
   PRIVS_ROOT
+#ifdef HAVE_SETSPENT
+  setspent();
+#endif /* HAVE_SETSPENT */
+
   sp = getspnam(u);
 
   if (sp) {
@@ -466,7 +454,7 @@ static char *_get_pw_info(pool *p, const char *u, time_t *lstchg, time_t *min,
       *expire = SP_CVT_DAYS(sp->sp_expire);
 #endif /* HAVE_SPWD_SP_EXPIRE */
   }
-#ifdef USE_AUTO_SHADOW
+#ifdef PR_USE_AUTO_SHADOW
   else {
     struct passwd *pw;
 
@@ -498,11 +486,11 @@ static char *_get_pw_info(pool *p, const char *u, time_t *lstchg, time_t *min,
 #else
   endspent();
   PRIVS_RELINQUISH
-#endif /* USE_AUTO_SHADOW */
+#endif /* PR_USE_AUTO_SHADOW */
   return cpw;
 }
 
-#else /* USE_SHADOW */
+#else /* PR_USE_SHADOW */
 
 static char *_get_pw_info(pool *p, const char *u, time_t *lstchg, time_t *min,
     time_t *max, time_t *warn, time_t *inact, time_t *expire) {
@@ -607,7 +595,7 @@ static char *_get_pw_info(pool *p, const char *u, time_t *lstchg, time_t *min,
   return cpw;
 }
 
-#endif /* USE_SHADOW */
+#endif /* PR_USE_SHADOW */
 
 static char *_get_ppw_info(pool *p, const char *u) {
   struct passwd *pw;
@@ -630,7 +618,7 @@ static char *_get_ppw_info(pool *p, const char *u) {
 MODRET pw_auth(cmd_rec *cmd) {
   time_t now;
   char *cpw;
-  time_t lstchg = -1,max = -1,inact = -1,disable = -1;
+  time_t lstchg = -1, max = -1, inact = -1, disable = -1;
   const char *name;
 
   name = cmd->argv[0];
@@ -643,9 +631,9 @@ MODRET pw_auth(cmd_rec *cmd) {
       &disable);
 
   if (!cpw)
-    return ERROR_INT(cmd, PR_AUTH_NOPWD);
+    return DECLINED(cmd);
 
-  if (auth_check(cmd->tmp_pool, cpw, cmd->argv[0], cmd->argv[1]))
+  if (pr_auth_check(cmd->tmp_pool, cpw, cmd->argv[0], cmd->argv[1]))
     return ERROR_INT(cmd, PR_AUTH_BADPWD);
 
   if (lstchg > (time_t) 0 &&
@@ -658,11 +646,11 @@ MODRET pw_auth(cmd_rec *cmd) {
       now > disable)
     return ERROR_INT(cmd, PR_AUTH_DISABLEDPWD);
 
+  session.auth_mech = "mod_auth_unix.c";
   return HANDLED(cmd);
 }
 
-/*
- * cmd->argv[0] = hashed password,
+/* cmd->argv[0] = hashed password,
  * cmd->argv[1] = user,
  * cmd->argv[2] = cleartext
  */
@@ -671,7 +659,7 @@ MODRET pw_check(cmd_rec *cmd) {
   const char *cpw = cmd->argv[0];
   const char *pw = cmd->argv[2];
 
-#ifdef USE_SIA
+#ifdef PR_USE_SIA
   SIAENTITY *ent = NULL;
   int res = SIASUCCESS;
   char *info[2];
@@ -681,41 +669,60 @@ MODRET pw_check(cmd_rec *cmd) {
 
 #ifdef COMSEC
   if (iscomsec()) {
-    if (strcmp(bigcrypt(pw, cpw), cpw) != 0)
+    if (strcmp(bigcrypt((char *) pw, (char *) cpw), cpw) != 0)
       return ERROR(cmd);
 
   } else {
 #endif /* COMSEC */
 
-#ifdef USE_SIA
+#ifdef PR_USE_SIA
   /* Use Tru64's C2 SIA subsystem for authenticating this user. */
   user = cmd->argv[1];
 
+  pr_log_auth(PR_LOG_NOTICE, "using SIA for user '%s'", user);
+
   info[0] = "ProFTPD";
   info[1] = NULL;
 
   /* Prepare the SIA subsystem. */
   PRIVS_ROOT
-  if ((res = sia_ses_init(&ent, 1, info, NULL, user, NULL, 0,
-      NULL)) != SIASUCCESS) {
-    log_auth(PR_LOG_NOTICE, "sia_ses_init() returned %d for user '%s'", res,
+  res = sia_ses_init(&ent, 1, info, NULL, user, NULL, 0, NULL);
+  if (res != SIASUCCESS) {
+    pr_log_auth(PR_LOG_NOTICE, "sia_ses_init() returned %d for user '%s'", res,
       user);
 
   } else {
 
-    if ((res = sia_ses_authent(NULL, pw, ent)) != SIASUCCESS)
-      log_auth(PR_LOG_NOTICE, "sia_ses_authent() returned %d for user '%s'",
+    res = sia_ses_authent(NULL, pw, ent);
+    if (res != SIASUCCESS) {
+      sia_ses_release(&ent);
+      PRIVS_RELINQUISH
+      pr_log_auth(PR_LOG_NOTICE, "sia_ses_authent() returned %d for user '%s'",
         res, user);
+      return ERROR(cmd);
+    }
 
-    if ((res = sia_ses_release(&ent)) != SIASUCCESS)
-      log_auth(PR_LOG_NOTICE, "sia_ses_release() returned %d", res);
+    res = sia_ses_estab(NULL, ent);
+    if (res != SIASUCCESS) {
+      PRIVS_RELINQUISH
+      pr_log_auth(PR_LOG_NOTICE, "sia_ses_estab() returned %d for user '%s'",
+        res, user);
+      return ERROR(cmd);
+    }
+
+    res = sia_ses_release(&ent);
+    if (res != SIASUCCESS) {
+      PRIVS_RELINQUISH
+      pr_log_auth(PR_LOG_NOTICE, "sia_ses_release() returned %d", res);
+      return ERROR(cmd);
+    }
   }
   PRIVS_RELINQUISH
 
   if (res != SIASUCCESS)
     return ERROR(cmd);
 
-#else /* !USE_SIA */
+#else /* !PR_USE_SIA */
 
 # ifdef CYGWIN
   /* We have to do special Windows NT voodoo with Cygwin in order to be
@@ -734,7 +741,7 @@ MODRET pw_check(cmd_rec *cmd) {
     tmp_cmd = pr_cmd_alloc(cmd->tmp_pool, 1, cmd->argv[1]);
 
     /* pw_getpwnam() returns a MODRET, so we need to handle that.  Yes, this
-     * might have been easier if we'd used auth_getpwnam(), but that would
+     * might have been easier if we'd used pr_auth_getpwnam(), but that would
      * dispatch through other auth modules, which is _not_ what we want.
      */
     mr = pw_getpwnam(tmp_cmd);
@@ -748,7 +755,7 @@ MODRET pw_check(cmd_rec *cmd) {
 
       if ((token = cygwin_logon_user((const struct passwd *) pwent,
           pw)) == INVALID_HANDLE_VALUE) {
-        log_pri(PR_LOG_NOTICE, "error authenticating Cygwin user: %s",
+        pr_log_pri(PR_LOG_NOTICE, "error authenticating Cygwin user: %s",
           strerror(errno));
         return ERROR(cmd);
       }
@@ -756,32 +763,33 @@ MODRET pw_check(cmd_rec *cmd) {
       cygwin_set_impersonation_token(token);
 
     } else
-      return ERROR(cmd);
+      return DECLINED(cmd);
 
   } else
 # endif /* CYGWIN */
 
   if (strcmp(crypt(pw, cpw), cpw) != 0)
     return ERROR(cmd);
-#endif /* USE_SIA */
+#endif /* PR_USE_SIA */
 
 #ifdef COMSEC
   }
 #endif /* COMSEC */
 
+  session.auth_mech = "mod_auth_unix.c";
   return HANDLED(cmd);
 }
 
-MODRET pw_uid_name(cmd_rec *cmd) {
+MODRET pw_uid2name(cmd_rec *cmd) {
   idmap_t *m;
   idauth_t id;
   struct passwd *pw;
 
-  id.uid = (uid_t) cmd->argv[0];
+  id.uid = *((uid_t *) cmd->argv[0]);
   m = _auth_lookup_id(uid_table, id);
 
   if (!m->name) {
-    /* wasn't cached, so perform a lookup */
+    /* Wasn't cached, so perform a lookup */
 
     if (PERSISTENT_PASSWD)
       pw = p_getpwuid(id.uid);
@@ -791,24 +799,21 @@ MODRET pw_uid_name(cmd_rec *cmd) {
     if (pw) {
       m->name = pstrdup(session.pool ? session.pool : permanent_pool,
         pw->pw_name);
-
-    } else {
-      char buf[10] = {'\0'};
-
-      snprintf(buf, sizeof(buf), "%lu", (unsigned long) id.uid);
-      m->name = pstrdup(session.pool ? session.pool : permanent_pool, buf);
+      return mod_create_data(cmd, m->name);
     }
+
+    return DECLINED(cmd);
   }
 
   return mod_create_data(cmd, m->name);
 }
 
-MODRET pw_gid_name(cmd_rec *cmd) {
+MODRET pw_gid2name(cmd_rec *cmd) {
   idmap_t *m;
   idauth_t id;
   struct group *gr;
 
-  id.gid = (gid_t) cmd->argv[0];
+  id.gid = *((gid_t *) cmd->argv[0]);
 
   m = _auth_lookup_id(gid_table, id);
 
@@ -818,26 +823,19 @@ MODRET pw_gid_name(cmd_rec *cmd) {
     else
       gr = getgrgid(id.gid);
 
-    if (gr)
+    if (gr) {
       m->name = pstrdup(session.pool ? session.pool : permanent_pool,
         gr->gr_name);
-
-    else {
-      char buf[10] = {'\0'};
-
-      /* Removed cast to unsigned long long here, as it presents a problem
-       * passed to snprintf because there is no ANSI standard for the format
-       * string modifier used for long long (is it %llu or %Lu, etc).
-       */
-      snprintf(buf, sizeof(buf), "%lu", (unsigned long) id.gid);
-      m->name = pstrdup(session.pool ? session.pool : permanent_pool, buf);
+      return mod_create_data(cmd, m->name);
     }
+
+    return DECLINED(cmd);
   }
 
   return mod_create_data(cmd, m->name);
 }
 
-MODRET pw_name_uid(cmd_rec *cmd) {
+MODRET pw_name2uid(cmd_rec *cmd) {
   struct passwd *pw;
   const char *name;
 
@@ -848,13 +846,10 @@ MODRET pw_name_uid(cmd_rec *cmd) {
   else
     pw = getpwnam(name);
 
-  if (pw)
-    return mod_create_data(cmd, (void *) pw->pw_uid);
-
-  return ERROR(cmd);
+  return pw ? mod_create_data(cmd, (void *) &pw->pw_uid) : DECLINED(cmd);
 }
 
-MODRET pw_name_gid(cmd_rec *cmd) {
+MODRET pw_name2gid(cmd_rec *cmd) {
   struct group *gr;
   const char *name;
 
@@ -865,10 +860,7 @@ MODRET pw_name_gid(cmd_rec *cmd) {
   else
     gr = getgrnam(name);
 
-  if (gr)
-    return mod_create_data(cmd, (void *) gr->gr_gid);
-
-  return ERROR(cmd);
+  return gr ? mod_create_data(cmd, (void *) &gr->gr_gid) : DECLINED(cmd);
 }
 
 /* cmd->argv[0] = name
@@ -913,7 +905,7 @@ MODRET pw_getgroups(cmd_rec *cmd) {
 
   /* Retrieve the necessary info. */
   if (!name || !(pw = my_getpwnam(name)))
-    return mod_create_error(cmd, -1);
+    return DECLINED(cmd);
 
   /* Populate the first group ID and name. */
   if (gids)
@@ -947,10 +939,10 @@ MODRET pw_getgroups(cmd_rec *cmd) {
   }
 
   if (gids && gids->nelts > 0)
-    return mod_create_data(cmd, (void *) gids->nelts);
+    return mod_create_data(cmd, (void *) &gids->nelts);
 
   else if (groups && groups->nelts > 0)
-    return mod_create_data(cmd, (void *) groups->nelts);
+    return mod_create_data(cmd, (void *) &groups->nelts);
 
   return DECLINED(cmd);
 }
@@ -969,14 +961,19 @@ MODRET set_persistentpasswd(cmd_rec *cmd) {
   return HANDLED(cmd);
 }
 
-/* Initialization routines
+/* Events handlers
  */
 
-static void auth_unix_sess_exit(void) {
-  auth_endpwent(session.pool);
-  auth_endgrent(session.pool);
+static void auth_unix_exit_ev(const void *event_data, void *user_data) {
+  pr_auth_endpwent(session.pool);
+  pr_auth_endgrent(session.pool);
+
+  return;
 }
 
+/* Initialization routines
+ */
+
 static int auth_unix_init(void) {
   memset(uid_table, 0 ,sizeof(uid_table));
   memset(gid_table, 0, sizeof(gid_table));
@@ -1007,7 +1004,7 @@ static int auth_unix_sess_init(void) {
     p_setgrent();
   }
 
-  pr_exit_register_handler(auth_unix_sess_exit);
+  pr_event_register(&auth_unix_module, "core.exit", auth_unix_exit_ev, NULL);
   return 0;
 }
 
@@ -1032,10 +1029,10 @@ static authtable auth_unix_authtab[] = {
   { 0,  "getgrgid",     pw_getgrgid },
   { 0,  "auth",         pw_auth	},
   { 0,  "check",	pw_check },
-  { 0,  "uid_name",	pw_uid_name },
-  { 0,  "gid_name",	pw_gid_name },
-  { 0,  "name_uid",	pw_name_uid },
-  { 0,  "name_gid",	pw_name_gid },
+  { 0,  "uid2name",	pw_uid2name },
+  { 0,  "gid2name",	pw_gid2name },
+  { 0,  "name2uid",	pw_name2uid },
+  { 0,  "name2gid",	pw_name2gid },
   { 0,  "getgroups",	pw_getgroups },
   { 0,  NULL }
 };
