@@ -2,7 +2,7 @@
  * ProFTPD - FTP server daemon
  * Copyright (c) 1997, 1998 Public Flood Software
  * Copyright (c) 1999, 2000 MacGyver aka Habeeb J. Dihu <macgyver@tos.net>
- * Copyright (c) 2001, 2002, 2003 The ProFTPD Project team
+ * Copyright (c) 2001-2005 The ProFTPD Project team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,13 +25,15 @@
  */
 
 /* Flexible logging module for proftpd
- * $Id: mod_log.c,v 1.57 2003/10/20 07:28:57 castaglia Exp $
+ * $Id: mod_log.c,v 1.73 2006/03/15 03:56:23 castaglia Exp $
  */
 
 #include "conf.h"
 #include "privs.h"
 
-extern pr_response_t *resp_list,*resp_err_list;
+extern pr_response_t *resp_list, *resp_err_list;
+
+module log_module;
 
 #define EXTENDED_LOG_BUFFER_SIZE		1025
 #define EXTENDED_LOG_MODE			0644
@@ -343,24 +345,40 @@ static int _parse_classes(char *s) {
         *nextp++ = '\0';
     }
 
-    if (!strcasecmp(s,"NONE"))
-      { classes = CL_NONE; break; }
-    if (!strcasecmp(s,"ALL"))
-      { classes = CL_ALL; break; }
-    else if (!strcasecmp(s,"AUTH"))
+    if (strcasecmp(s, "NONE") == 0) {
+      classes = CL_NONE;
+      break;
+    }
+
+    if (strcasecmp(s, "ALL") == 0) {
+      classes = CL_ALL;
+      break;
+
+    } else if (strcasecmp(s, "AUTH") == 0) {
       classes |= CL_AUTH;
-    else if (!strcasecmp(s,"INFO"))
+
+    } else if (strcasecmp(s, "INFO") == 0) {
       classes |= CL_INFO;
-    else if (!strcasecmp(s,"DIRS"))
+
+    } else if (strcasecmp(s, "DIRS") == 0) {
       classes |= CL_DIRS;
-    else if (!strcasecmp(s,"READ"))
+
+    } else if (strcasecmp(s, "READ") == 0) {
       classes |= CL_READ;
-    else if (!strcasecmp(s,"WRITE"))
+
+    } else if (strcasecmp(s, "WRITE") == 0) {
       classes |= CL_WRITE;
-    else if (!strcasecmp(s,"MISC"))
+
+    } else if (strcasecmp(s, "MISC") == 0) {
       classes |= CL_MISC;
-    else
-      log_pri(PR_LOG_NOTICE, "ExtendedLog class '%s' is not defined.", s);
+
+    } else if (strcasecmp(s, "SEC") == 0 ||
+               strcasecmp(s, "SECURE") == 0) {
+      classes |= CL_SEC;
+
+    } else
+      pr_log_pri(PR_LOG_NOTICE, "ExtendedLog class '%s' is not defined.", s);
+
   } while ((s = nextp));
 
   return classes;
@@ -383,7 +401,7 @@ MODRET set_extendedlog(cmd_rec *cmd) {
   if (strncasecmp(cmd->argv[1], "syslog:", 7) == 0) {
     char *tmp = strchr(cmd->argv[1], ':');
 
-    if (log_str2sysloglevel(++tmp) < 0) {
+    if (pr_log_str2sysloglevel(++tmp) < 0) {
       CONF_ERROR(cmd, pstrcat(cmd->tmp_pool, "unknown syslog level: '",
         tmp, "'", NULL));
 
@@ -416,7 +434,7 @@ MODRET set_allowlogsymlinks(cmd_rec *cmd) {
   CHECK_CONF(cmd, CONF_ROOT|CONF_VIRTUAL|CONF_GLOBAL);
 
   if ((bool = get_boolean(cmd, 1)) == -1)
-    CONF_ERROR(cmd, "expected boolean argument.");
+    CONF_ERROR(cmd, "expected Boolean parameter");
 
   c = add_config_param(cmd->argv[0], 1, NULL);
   c->argv[0] = pcalloc(c->pool, sizeof(unsigned char));
@@ -438,7 +456,7 @@ MODRET set_serverlog(cmd_rec *cmd) {
 /* Syntax: SystemLog <filename> */
 MODRET set_systemlog(cmd_rec *cmd) {
   char *syslogfn = NULL;
-  int ret;
+  int res;
 
   CHECK_ARGS(cmd, 1);
   CHECK_CONF(cmd, CONF_ROOT);
@@ -457,16 +475,17 @@ MODRET set_systemlog(cmd_rec *cmd) {
 
   pr_signals_block();
 
-  if ((ret = log_opensyslog(syslogfn)) < 0) {
+  res = log_opensyslog(syslogfn);
+  if (res < 0) {
     int xerrno = errno;
 
     pr_signals_unblock();
 
-    if (ret == LOG_WRITEABLE_DIR) {
+    if (res == PR_LOG_WRITABLE_DIR) {
       CONF_ERROR(cmd,
         "you are attempting to log to a world writeable directory");
 
-    } else if (ret == LOG_SYMLINK) {
+    } else if (res == PR_LOG_SYMLINK) {
       CONF_ERROR(cmd, "you are attempting to log to a symbolic link");
 
     } else {
@@ -547,7 +566,8 @@ static char *get_next_meta(pool *p, cmd_rec *cmd, unsigned char **f) {
   case META_BYTES_SENT:
     argp = arg;
     if (session.xfer.p)
-      snprintf(argp, sizeof(arg), "%" PR_LU, session.xfer.total_bytes);
+      snprintf(argp, sizeof(arg), "%" PR_LU,
+        (pr_off_t) session.xfer.total_bytes);
     else
       sstrncpy(argp, "-", sizeof(arg));
 
@@ -555,31 +575,25 @@ static char *get_next_meta(pool *p, cmd_rec *cmd, unsigned char **f) {
     break;
 
   case META_CLASS:
-    {
-      unsigned char *classes = get_param_ptr(TOPLEVEL_CONF, "Classes", FALSE);
-      argp = arg;
-
-      if (classes && *classes == TRUE)
-        sstrncpy(argp, session.class->name, sizeof(arg));
-      else
-        sstrncpy(argp, "-", sizeof(arg));
-      m++;
-    }
+    argp = arg;
+    sstrncpy(argp, session.class ? session.class->cls_name : "-", sizeof(arg));
+    m++;
     break;
 
   case META_DIR_NAME:
     argp = arg;
 
-    if (!strcmp(cmd->argv[0], C_CDUP) || !strcmp(cmd->argv[0], C_CWD) ||
-        !strcmp(cmd->argv[0], C_MKD) || !strcmp(cmd->argv[0], C_RMD) ||
-        !strcmp(cmd->argv[0], C_XCWD) || !strcmp(cmd->argv[0], C_XCUP) ||
-        !strcmp(cmd->argv[0], C_XMKD) || !strcmp(cmd->argv[0], C_XRMD)) {
+    if (strcmp(cmd->argv[0], C_CDUP) == 0 ||
+        strcmp(cmd->argv[0], C_CWD) == 0 ||
+        strcmp(cmd->argv[0], C_MKD) == 0 ||
+        strcmp(cmd->argv[0], C_RMD) == 0 ||
+        strcmp(cmd->argv[0], C_XCWD) == 0 ||
+        strcmp(cmd->argv[0], C_XCUP) == 0 ||
+        strcmp(cmd->argv[0], C_XMKD) == 0 ||
+        strcmp(cmd->argv[0], C_XRMD) == 0) {
       char *tmp = strrchr(cmd->arg, '/');
 
-      if (tmp)
-        sstrncpy(argp, tmp, sizeof(arg));
-      else
-        sstrncpy(argp, cmd->arg, sizeof(arg));
+      sstrncpy(argp, tmp ? tmp : cmd->arg, sizeof(arg));
 
     } else {
       sstrncpy(argp, "", sizeof(arg));
@@ -591,12 +605,16 @@ static char *get_next_meta(pool *p, cmd_rec *cmd, unsigned char **f) {
   case META_DIR_PATH:
     argp = arg;
 
-    if (!strcmp(cmd->argv[0], C_CDUP) || !strcmp(cmd->argv[0], C_MKD) ||
-        !strcmp(cmd->argv[0], C_RMD) || !strcmp(cmd->argv[0], C_XCUP) ||
-        !strcmp(cmd->argv[0], C_XMKD) || !strcmp(cmd->argv[0], C_XRMD)) {
+    if (strcmp(cmd->argv[0], C_CDUP) == 0 ||
+        strcmp(cmd->argv[0], C_MKD) == 0 ||
+        strcmp(cmd->argv[0], C_RMD) == 0 ||
+        strcmp(cmd->argv[0], C_XCUP) == 0 ||
+        strcmp(cmd->argv[0], C_XMKD) == 0 ||
+        strcmp(cmd->argv[0], C_XRMD) == 0) {
       sstrncpy(argp, dir_abs_path(p, cmd->arg, TRUE), sizeof(arg));
 
-    } else if (!strcmp(cmd->argv[0], C_CWD) || !strcmp(cmd->argv[0], C_XCWD)) {
+    } else if (strcmp(cmd->argv[0], C_CWD) == 0 ||
+               strcmp(cmd->argv[0], C_XCWD) == 0) {
 
       /* Note: by this point in the dispatch cycle, the current working
        * directory has already been changed.  For the CWD/XCWD commands,
@@ -626,7 +644,11 @@ static char *get_next_meta(pool *p, cmd_rec *cmd, unsigned char **f) {
   case META_FILENAME:
     argp = arg;
 
-    if (session.xfer.p && session.xfer.path) {
+    if (strcmp(cmd->argv[0], C_RNTO) == 0) {
+      sstrncpy(argp, dir_abs_path(p, cmd->arg, TRUE), sizeof(arg));
+
+    } else if (session.xfer.p &&
+               session.xfer.path) {
       sstrncpy(argp, dir_abs_path(p, session.xfer.path, TRUE), sizeof(arg));
 
     } else {
@@ -635,9 +657,11 @@ static char *get_next_meta(pool *p, cmd_rec *cmd, unsigned char **f) {
        * filenames that are not stored in the session.xfer structure; these
        * should be expanded properly as well.
        */
-      if (!strcmp(cmd->argv[0], C_DELE) || !strcmp(cmd->argv[0], C_MKD) ||
-          !strcmp(cmd->argv[0], C_RMD) || !strcmp(cmd->argv[0], C_XMKD) ||
-          !strcmp(cmd->argv[0], C_XRMD))
+      if (strcmp(cmd->argv[0], C_DELE) == 0 ||
+          strcmp(cmd->argv[0], C_MKD) == 0 ||
+          strcmp(cmd->argv[0], C_RMD) == 0 ||
+          strcmp(cmd->argv[0], C_XMKD) == 0 ||
+          strcmp(cmd->argv[0], C_XRMD) == 0)
         sstrncpy(arg, dir_abs_path(p, cmd->arg, TRUE), sizeof(arg));
 
       else
@@ -659,7 +683,7 @@ static char *get_next_meta(pool *p, cmd_rec *cmd, unsigned char **f) {
        * stored in the session.xfer structure; these should be expanded
        * properly as well.
        */
-      if (!strcmp(cmd->argv[0], C_DELE))
+      if (strcmp(cmd->argv[0], C_DELE) == 0)
         sstrncpy(arg, cmd->arg, sizeof(arg));
 
       else
@@ -684,13 +708,14 @@ static char *get_next_meta(pool *p, cmd_rec *cmd, unsigned char **f) {
 
   case META_REMOTE_HOST:
     argp = arg;
-    sstrncpy(argp, session.c->remote_name, sizeof(arg));
+    sstrncpy(argp, pr_netaddr_get_sess_remote_name(), sizeof(arg));
     m++;
     break;
 
   case META_REMOTE_IP:
     argp = arg;
-    sstrncpy(argp, pr_netaddr_get_ipstr(session.c->remote_addr), sizeof(arg));
+    sstrncpy(argp, pr_netaddr_get_ipstr(pr_netaddr_get_sess_remote_addr()),
+      sizeof(arg));
     m++;
     break;
 
@@ -756,7 +781,7 @@ static char *get_next_meta(pool *p, cmd_rec *cmd, unsigned char **f) {
           snprintf(argp + strlen(argp), sizeof(arg) - strlen(argp),
             "%c%.2d%.2d]", sign, timz/60, timz%60);
         else
-          log_pri(PR_LOG_NOTICE, "notice: %%t expansion yields excessive "
+          pr_log_pri(PR_LOG_NOTICE, "notice: %%t expansion yields excessive "
             "string, ignoring");
       }
     }
@@ -785,8 +810,8 @@ static char *get_next_meta(pool *p, cmd_rec *cmd, unsigned char **f) {
           end_time.tv_sec--;
         }
 
-        snprintf(argp, sizeof(arg), "%ld.%03ld", (time_t) end_time.tv_sec,
-          (time_t) (end_time.tv_usec / 1000));
+        snprintf(argp, sizeof(arg), "%ld.%03ld", (long) end_time.tv_sec,
+          (long) (end_time.tv_usec / 1000));
 
       } else
         sstrncpy(argp, "-", sizeof(arg));
@@ -800,16 +825,27 @@ static char *get_next_meta(pool *p, cmd_rec *cmd, unsigned char **f) {
   case META_COMMAND:
     argp = arg;
 
-    if (!strcasecmp(cmd->argv[0], C_PASS) && session.hide_password)
+    if (strcasecmp(cmd->argv[0], C_PASS) == 0 &&
+        session.hide_password) {
       sstrncpy(argp, "PASS (hidden)", sizeof(arg));
-    else
+
+    } else {
       sstrncpy(argp, get_full_cmd(cmd), sizeof(arg));
+    }
+
     m++;
     break;
 
   case META_CMD_PARAMS:
     argp = arg;
-    sstrncpy(argp, cmd->arg, sizeof(arg));
+    if (strcasecmp(cmd->argv[0], C_PASS) == 0 &&
+        session.hide_password) {
+      sstrncpy(argp, "(hidden)", sizeof(arg));
+
+    } else {
+      sstrncpy(argp, cmd->arg, sizeof(arg));
+    }
+
     m++;
     break;
 
@@ -859,16 +895,18 @@ static char *get_next_meta(pool *p, cmd_rec *cmd, unsigned char **f) {
       argp = arg;
       r = (resp_list ? resp_list : resp_err_list);
 
-      for (; r && !r->num; r=r->next) ;
-      if (r && r->num)
-        sstrncpy(argp,r->num,sizeof(arg));
+      for (; r && !r->num; r = r->next) ;
+      if (r &&
+          r->num) {
+        sstrncpy(argp, r->num, sizeof(arg));
 
       /* Hack to add return code for proper logging of QUIT command. */
-      else if (!strcasecmp(cmd->argv[0], C_QUIT))
+      } else if (strcasecmp(cmd->argv[0], C_QUIT) == 0) {
         sstrncpy(argp, R_221, sizeof(arg));
 
-      else
-        sstrncpy(argp,"-",sizeof(arg));
+      } else {
+        sstrncpy(argp, "-", sizeof(arg));
+      }
     }
 
     m++;
@@ -944,11 +982,7 @@ MODRET log_any(cmd_rec *cmd) {
   return DECLINED(cmd);
 }
 
-/* log_rehash_cb() is called whenever the master server rehashes it's
- * config (in response to SIGHUP).
- */
-
-static void log_rehash_cb(void *d) {
+static void log_restart_ev(const void *event_data, void *user_data) {
   destroy_pool(log_pool);
 
   formats = NULL;
@@ -957,16 +991,21 @@ static void log_rehash_cb(void *d) {
   log_set = NULL;
 
   log_pool = make_sub_pool(permanent_pool);
+  pr_pool_tag(log_pool, "mod_log pool");
+
   logformat("", "%h %l %u %t \"%r\" %s %b");
+
+  return;
 }
 
 static int log_init(void) {
   log_pool = make_sub_pool(permanent_pool);
+  pr_pool_tag(log_pool, "mod_log pool");
 
   /* Add the "default" extendedlog format */
   logformat("", "%h %l %u %t \"%r\" %s %b");
 
-  pr_rehash_register_handler(NULL, log_rehash_cb);
+  pr_event_register(&log_module, "core.restart", log_restart_ev, NULL);
   return 0;
 }
 
@@ -1005,12 +1044,13 @@ static void find_extendedlogs(void) {
     if (logfmt_s) {
       /* search for the format-nickname */
       for (logfmt = formats; logfmt; logfmt = logfmt->next)
-        if (!strcmp(logfmt->lf_nickname, logfmt_s))
+        if (strcmp(logfmt->lf_nickname, logfmt_s) == 0)
           break;
 
       if (!logfmt) {
-        log_pri(PR_LOG_NOTICE, "ExtendedLog '%s' uses unknown format nickname "
-          "'%s'", logfname, logfmt_s);
+        pr_log_pri(PR_LOG_NOTICE,
+          "ExtendedLog '%s' uses unknown format nickname '%s'", logfname,
+          logfmt_s);
         goto loop_extendedlogs;
       }
 
@@ -1047,7 +1087,8 @@ MODRET log_post_pass(cmd_rec *cmd) {
     for (lf = logs; lf; lf = lf->next) {
       if (lf->lf_fd != -1 && lf->lf_fd != EXTENDED_LOG_SYSLOG &&
           lf->lf_conf && lf->lf_conf->config_type == CONF_ANON) {
-        log_debug(DEBUG7, "mod_log: closing ExtendedLog '%s'", lf->lf_filename);
+        pr_log_debug(DEBUG7, "mod_log: closing ExtendedLog '%s'",
+          lf->lf_filename);
         close(lf->lf_fd);
         lf->lf_fd = -1;
       }
@@ -1060,7 +1101,8 @@ MODRET log_post_pass(cmd_rec *cmd) {
     for (lf = logs; lf; lf = lf->next) {
       if (lf->lf_fd != -1 && lf->lf_fd != EXTENDED_LOG_SYSLOG &&
           lf->lf_conf && lf->lf_conf != session.anon_config) {
-        log_debug(DEBUG7, "mod_log: closing ExtendedLog '%s'", lf->lf_filename);
+        pr_log_debug(DEBUG7, "mod_log: closing ExtendedLog '%s'",
+          lf->lf_filename);
         close(lf->lf_fd);
         lf->lf_fd = -1;
       }
@@ -1077,9 +1119,11 @@ MODRET log_post_pass(cmd_rec *cmd) {
         logfile_t *lfi = NULL;
 
         for (lfi = logs; lfi; lfi = lfi->next) {
-          if (lfi->lf_fd != -1 && lfi->lf_fd != EXTENDED_LOG_SYSLOG &&
-              !lfi->lf_conf && !strcmp(lfi->lf_filename, lf->lf_filename)) {
-            log_debug(DEBUG7, "mod_log: closing ExtendedLog '%s'",
+          if (lfi->lf_fd != -1 &&
+              lfi->lf_fd != EXTENDED_LOG_SYSLOG &&
+              !lfi->lf_conf &&
+              strcmp(lfi->lf_filename, lf->lf_filename) == 0) {
+            pr_log_debug(DEBUG7, "mod_log: closing ExtendedLog '%s'",
               lf->lf_filename);
             close(lfi->lf_fd);
             lfi->lf_fd = -1;
@@ -1087,9 +1131,10 @@ MODRET log_post_pass(cmd_rec *cmd) {
         }
 
         /* Go ahead and close the log if it's CL_NONE */
-        if (lf->lf_fd != -1 && lf->lf_fd != EXTENDED_LOG_SYSLOG &&
+        if (lf->lf_fd != -1 &&
+            lf->lf_fd != EXTENDED_LOG_SYSLOG &&
             lf->lf_classes == CL_NONE) {
-          log_debug(DEBUG7, "mod_log: closing ExtendedLog '%s'",
+          pr_log_debug(DEBUG7, "mod_log: closing ExtendedLog '%s'",
             lf->lf_filename);
           close(lf->lf_fd);
           lf->lf_fd = -1;
@@ -1126,26 +1171,27 @@ static int log_sess_init(void) {
       if (strncasecmp(lf->lf_filename, "syslog:", 7) != 0) {
         int res = 0;
 
-        log_debug(DEBUG7, "mod_log: opening ExtendedLog '%s'", lf->lf_filename);
+        pr_log_debug(DEBUG7, "mod_log: opening ExtendedLog '%s'",
+          lf->lf_filename);
 
         pr_signals_block();
         PRIVS_ROOT
-        res = log_openfile(lf->lf_filename, &lf->lf_fd, EXTENDED_LOG_MODE);
+        res = pr_log_openfile(lf->lf_filename, &lf->lf_fd, EXTENDED_LOG_MODE);
         PRIVS_RELINQUISH
         pr_signals_unblock();
 
         if (res == -1) {
-          log_pri(PR_LOG_NOTICE, "unable to open ExtendedLog '%s': %s",
+          pr_log_pri(PR_LOG_NOTICE, "unable to open ExtendedLog '%s': %s",
             lf->lf_filename, strerror(errno));
           continue;
 
-        } else if (res == LOG_WRITEABLE_DIR) {
-          log_pri(PR_LOG_NOTICE, "unable to open ExtendedLog '%s': "
+        } else if (res == PR_LOG_WRITABLE_DIR) {
+          pr_log_pri(PR_LOG_NOTICE, "unable to open ExtendedLog '%s': "
             "containing directory is world writeable", lf->lf_filename);
           continue;
 
-        } else if (res == LOG_SYMLINK) {
-          log_pri(PR_LOG_NOTICE, "unable to open ExtendedLog '%s': "
+        } else if (res == PR_LOG_SYMLINK) {
+          pr_log_pri(PR_LOG_NOTICE, "unable to open ExtendedLog '%s': "
             "%s is a symbolic link", lf->lf_filename, lf->lf_filename);
           close(lf->lf_fd);
           lf->lf_fd = -1;
@@ -1155,7 +1201,7 @@ static int log_sess_init(void) {
       } else {
         char *tmp = strchr(lf->lf_filename, ':');
 
-        lf->lf_syslog_level = log_str2sysloglevel(++tmp);
+        lf->lf_syslog_level = pr_log_str2sysloglevel(++tmp);
         lf->lf_fd = EXTENDED_LOG_SYSLOG;
       }
     }
@@ -1177,7 +1223,6 @@ static conftable log_conftab[] = {
 };
 
 static cmdtable log_cmdtab[] = {
-  { PRE_CMD, 		C_QUIT,	G_NONE,	log_any,		FALSE, FALSE },
   { LOG_CMD,		C_ANY,	G_NONE,	log_any,		FALSE, FALSE },
   { LOG_CMD_ERR,	C_ANY,	G_NONE,	log_any,		FALSE, FALSE },
   { POST_CMD,		C_PASS,	G_NONE,	log_post_pass,		FALSE, FALSE },
