@@ -2,7 +2,7 @@
  * ProFTPD - FTP server daemon
  * Copyright (c) 1997, 1998 Public Flood Software
  * Copyright (c) 1999, 2000 MacGyver aka Habeeb J. Dihu <macgyver@tos.net>
- * Copyright (c) 2001, 2002, 2003 The ProFTPD Project team
+ * Copyright (c) 2001-2004 The ProFTPD Project team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@
  */
 
 /* Directory listing module for ProFTPD.
- * $Id: mod_ls.c,v 1.113 2004/05/07 21:31:30 castaglia Exp $
+ * $Id: mod_ls.c,v 1.128 2005/08/24 16:10:30 castaglia Exp $
  */
 
 #include "conf.h"
@@ -34,16 +34,18 @@
 #define GLOB_ABORTED GLOB_ABEND
 #endif
 
-#define MAP_UID(x)	(fakeuser ? fakeuser : auth_uid_name(cmd->tmp_pool,(x)))
-#define MAP_GID(x)	(fakegroup ? fakegroup : auth_gid_name(cmd->tmp_pool,(x)))
+#define MAP_UID(x) \
+  (fakeuser ? fakeuser : pr_auth_uid2name(cmd->tmp_pool, (x)))
 
-static void addfile(cmd_rec *, const char *, const char *, time_t);
+#define MAP_GID(x) \
+  (fakegroup ? fakegroup : pr_auth_gid2name(cmd->tmp_pool, (x)))
+
+static void addfile(cmd_rec *, const char *, const char *, time_t, off_t);
 static int outputfiles(cmd_rec *);
 
 static int listfile(cmd_rec *, pool *, const char *);
 static int listdir(cmd_rec *, pool *, const char *);
 
-static int matches = 0;
 static unsigned char list_strict_opts = FALSE;
 static char *list_options = NULL;
 static unsigned char list_show_symlinks = TRUE, list_times_gmt = TRUE;
@@ -79,6 +81,7 @@ static int
     opt_n = 0,
     opt_R = 0,
     opt_r = 0,
+    opt_S = 0,
     opt_t = 0,
     opt_STAT = 0;
 
@@ -98,8 +101,7 @@ static config_rec *_find_ls_limit(char *ftp_cmd) {
   if (!session.dir_config)
     return NULL;
 
-  /* determine whether this command is <Limit>'ed
-   */
+  /* Determine whether this command is <Limit>'ed. */
   for (c = session.dir_config; c; c = c->parent) {
 
     if (c->subset) {
@@ -277,7 +279,7 @@ static char units[6][2] =
 
 static void ls_fmt_filesize(char *buf, size_t buflen, off_t sz) {
   if (!opt_h || sz < 1000) {
-    snprintf(buf, buflen, "%8" PR_LU, sz);
+    snprintf(buf, buflen, "%8" PR_LU, (pr_off_t) sz);
 
   } else {
     register unsigned int i = 0;
@@ -333,7 +335,8 @@ static int listfile(cmd_rec *cmd, pool *p, const char *name) {
       if (pr_fsio_stat(name, &l_st) != -1) {
         memcpy(&st, &l_st, sizeof(struct stat));
 
-        if ((len = pr_fsio_readlink(name, m, sizeof(m))) < 0)
+        len = pr_fsio_readlink(name, m, sizeof(m));
+        if (len < 0)
           return 0;
 
         m[len] = '\0';
@@ -345,8 +348,8 @@ static int listfile(cmd_rec *cmd, pool *p, const char *name) {
         return 0;
 
     } else if (S_ISLNK(st.st_mode)) {
-
-      if ((len = pr_fsio_readlink(name, l, sizeof(l))) < 0)
+      len = pr_fsio_readlink(name, l, sizeof(l));
+      if (len < 0)
         return 0;
 
       l[len] = '\0';
@@ -513,12 +516,14 @@ static int listfile(cmd_rec *cmd, pool *p, const char *name) {
         if (opt_STAT)
           pr_response_add(R_211, "%s%s", nameline, suffix);
         else
-          addfile(cmd, nameline, suffix, mtime);
+          addfile(cmd, nameline, suffix, mtime, st.st_size);
       }
 
     } else {
-      if (S_ISREG(st.st_mode) || S_ISDIR(st.st_mode) || S_ISLNK(st.st_mode))
-           addfile(cmd, name, suffix, mtime);
+      if (S_ISREG(st.st_mode) ||
+          S_ISDIR(st.st_mode) ||
+          S_ISLNK(st.st_mode))
+           addfile(cmd, name, suffix, mtime, st.st_size);
     }
   }
 
@@ -531,13 +536,15 @@ static int filenames = 0;
 struct filename {
   struct filename *down;
   struct filename *right;
+  char *line;
   int top;
-  char line[1];
 };
 
 struct sort_filename {
   time_t mtime;
-  char *name,*suffix;
+  off_t size;
+  char *name;
+  char *suffix;
 };
 
 static struct filename *head = NULL;
@@ -546,9 +553,9 @@ static array_header *sort_arr = NULL;
 static pool *fpool = NULL;
 
 static void addfile(cmd_rec *cmd, const char *name, const char *suffix,
-    time_t mtime) {
+    time_t mtime, off_t size) {
   struct filename *p;
-  int l;
+  size_t l;
 
   if (!name || !suffix)
     return;
@@ -558,28 +565,27 @@ static void addfile(cmd_rec *cmd, const char *name, const char *suffix,
     pr_pool_tag(fpool, "mod_ls: addfile() fpool");
   }
 
-  if (opt_t) {
+  if (opt_S || opt_t) {
     struct sort_filename *s;
 
     if (!sort_arr)
-      sort_arr = make_array(fpool,50,sizeof(struct sort_filename));
+      sort_arr = make_array(fpool, 50, sizeof(struct sort_filename));
 
-    s = (struct sort_filename*)push_array(sort_arr);
+    s = (struct sort_filename *) push_array(sort_arr);
     s->mtime = mtime;
-    s->name = pstrdup(fpool,name);
-    s->suffix = pstrdup(fpool,suffix);
+    s->size = size;
+    s->name = pstrdup(fpool, name);
+    s->suffix = pstrdup(fpool, suffix);
 
     return;
   }
 
-  matches++;
-
   l = strlen(name) + strlen(suffix);
   if (l > colwidth)
     colwidth = l;
 
-  p = (struct filename*) pcalloc(fpool, sizeof(struct filename) + l + 1);
-
+  p = (struct filename *) pcalloc(fpool, sizeof(struct filename));
+  p->line = pcalloc(fpool, l + 2);
   snprintf(p->line, l + 1, "%s%s", name, suffix);
 
   if (tail)
@@ -592,7 +598,7 @@ static void addfile(cmd_rec *cmd, const char *name, const char *suffix,
   filenames++;
 }
 
-static int file_mtime_cmp( const struct sort_filename *f1,
+static int file_mtime_cmp(const struct sort_filename *f1,
     const struct sort_filename *f2) {
 
   if (f1->mtime > f2->mtime)
@@ -609,23 +615,63 @@ static int file_mtime_reverse_cmp(const struct sort_filename *f1,
   return -file_mtime_cmp(f1, f2);
 }
 
+static int file_size_cmp(const struct sort_filename *f1,
+    const struct sort_filename *f2) {
+
+  if (f1->size > f2->size)
+    return -1;
+
+  else if (f1->size < f2->size)
+    return 1;
+
+  return 0;
+}
+
+static int file_size_reverse_cmp(const struct sort_filename *f1,
+    const struct sort_filename *f2) {
+  return -file_size_cmp(f1, f2);
+}
+
 static void sortfiles(cmd_rec *cmd) {
-  struct sort_filename *s = NULL;
 
-  if (opt_t && sort_arr) {
-    register unsigned int i = 0;
+  if (sort_arr) {
 
-    qsort(sort_arr->elts, sort_arr->nelts, sizeof(struct sort_filename),
-      (int (*)(const void *, const void *))
-        (opt_r ? file_mtime_reverse_cmp : file_mtime_cmp));
+    /* Sort by modification time? */
+    if (opt_t) {
+      register unsigned int i = 0;
+      int setting = opt_S;
+      struct sort_filename *elts = sort_arr->elts;
 
-    opt_t = 0;
+      qsort(sort_arr->elts, sort_arr->nelts, sizeof(struct sort_filename),
+        (int (*)(const void *, const void *))
+          (opt_r ? file_mtime_reverse_cmp : file_mtime_cmp));
 
-    for (i = 0, s = (struct sort_filename *)sort_arr->elts;
-        i < sort_arr->nelts; i++, s++)
-      addfile(cmd, s->name, s->suffix, s->mtime);
+      opt_S = opt_t = 0;
 
-    opt_t = 1;
+      for (i = 0; i < sort_arr->nelts; i++)
+        addfile(cmd, elts[i].name, elts[i].suffix, elts[i].mtime, elts[i].size);
+
+      opt_S = setting;
+      opt_t = 1;
+
+    /* Sort by file size? */
+    } else if (opt_S) {
+      register unsigned int i = 0;
+      int setting = opt_t;
+      struct sort_filename *elts = sort_arr->elts;
+
+      qsort(sort_arr->elts, sort_arr->nelts, sizeof(struct sort_filename),
+        (int (*)(const void *, const void *))
+          (opt_r ? file_size_reverse_cmp : file_size_cmp));
+
+      opt_S = opt_t = 0;
+
+      for (i = 0; i < sort_arr->nelts; i++)
+        addfile(cmd, elts[i].name, elts[i].suffix, elts[i].mtime, elts[i].size);
+
+      opt_S = 1;
+      opt_t = setting;
+    }
   }
 
   sort_arr = NULL;
@@ -635,7 +681,7 @@ static int outputfiles(cmd_rec *cmd) {
   int n;
   struct filename *p = NULL, *q = NULL;
 
-  if (opt_t)
+  if (opt_S || opt_t)
     sortfiles(cmd);
 
   if (!head)		/* nothing to display */
@@ -924,7 +970,7 @@ static int listdir(cmd_rec *cmd, pool *workp, const char *name) {
 
         } else {
 
-          /* Make sure IgnoreHidden is properly honored. "." and * ".." are
+          /* Make sure IgnoreHidden is properly honored. "." and ".." are
            * not to be treated as hidden files, though.
            */
           d = listfile(cmd, workp, *s);
@@ -934,7 +980,17 @@ static int listdir(cmd_rec *cmd, pool *workp, const char *name) {
         d = listfile(cmd, workp, *s);
       }
 
-      if (d == 2)
+      if (opt_R && d == 0) {
+
+        /* This is a nasty hack.  If listfile() returns a zero, and we
+         * will be recursing (-R option), make sure we don't try to list
+         * this file again by changing the first character of the path
+         * to ".".  Such files are skipped later.
+         */
+        **s = '.';
+        *(*s + 1) = '\0';
+
+      } else if (d == 2)
         break;
 
       s++;
@@ -1001,10 +1057,10 @@ static int listdir(cmd_rec *cmd, pool *workp, const char *name) {
         char *subdir;
         int res = 0;
 
-        if (strcmp(name,".") == 0)
+        if (strcmp(name, ".") == 0)
           subdir = *r;
         else
-          subdir = pdircat(workp,name,*r,NULL);
+          subdir = pdircat(workp, name, *r, NULL);
 
         if (opt_STAT) {
           pr_response_add(R_211, "%s", "");
@@ -1151,6 +1207,10 @@ static void parse_list_opts(char **opt, int *glob_flags, int handle_plus_opts) {
           opt_r = 1;
           break;
 
+        case 'S':
+          opt_S = 1;
+          break;
+
         case 't':
           opt_t = 1;
           if (glob_flags)
@@ -1218,6 +1278,10 @@ static void parse_list_opts(char **opt, int *glob_flags, int handle_plus_opts) {
           opt_r = 0;
           break;
 
+        case 'S':
+          opt_S = 0;
+          break;
+
         case 't':
           opt_t = 0;
           if (glob_flags)
@@ -1237,14 +1301,13 @@ static void parse_list_opts(char **opt, int *glob_flags, int handle_plus_opts) {
 static int dolist(cmd_rec *cmd, const char *opt, int clearflags) {
   int skiparg = 0;
   int glob_flags = GLOB_PERIOD;
-  char *arg = (char*)opt;
+  char *arg = (char*) opt;
 
-  matches = 0;
   ls_curtime = time(NULL);
 
   if (clearflags)
     opt_a = opt_C = opt_d = opt_F = opt_h = opt_n = opt_r = opt_R =
-      opt_t = opt_STAT = opt_L = 0;
+      opt_S = opt_t = opt_STAT = opt_L = 0;
 
   if (!list_strict_opts) {
     parse_list_opts(&arg, &glob_flags, FALSE);
@@ -1274,21 +1337,16 @@ static int dolist(cmd_rec *cmd, const char *opt, int clearflags) {
   if (list_options)
     parse_list_opts(&list_options, &glob_flags, TRUE);
 
-  /* open data connection */
-  if (!opt_STAT) {
-    session.sf_flags |= SF_ASCII_OVERRIDE;
-    if (pr_data_open(NULL, "file list", PR_NETIO_IO_WR, 0) < 0)
-      return -1;
-  }
-
   if (arg && *arg) {
     int justone = 1;
     glob_t g;
-    int    a;
+    int globbed = FALSE;
+    int a;
     char   pbuffer[PR_TUNABLE_PATH_MAX + 1] = "";
+    char *target;
 
     /* Make sure the glob_t is initialized. */
-    memset(&g, '\0', sizeof(glob_t));
+    memset(&g, '\0', sizeof(g));
 
     if (*arg == '~') {
       struct passwd *pw;
@@ -1301,7 +1359,7 @@ static int dolist(cmd_rec *cmd, const char *opt, int clearflags) {
 
       pbuffer[i] = '\0';
 
-      pw = auth_getpwnam(cmd->tmp_pool, i ? pbuffer : session.user);
+      pw = pr_auth_getpwnam(cmd->tmp_pool, i ? pbuffer : session.user);
       if (pw) {
         snprintf(pbuffer, sizeof(pbuffer), "%s%s", pw->pw_dir, p);
 
@@ -1309,9 +1367,30 @@ static int dolist(cmd_rec *cmd, const char *opt, int clearflags) {
         pbuffer[0] = '\0';
     }
 
+    target = *pbuffer ? pbuffer : arg;
+
+    /* If there are no globbing characters in the given target,
+     * we can check to see if it even exists.
+     */
+    if (strpbrk(target, "{[*?") == NULL) {
+      struct stat st;
+
+      pr_fs_clear_cache();
+      if (pr_fsio_stat(target, &st) < 0) {
+        pr_response_add_err(R_450, "%s: %s", target, strerror(errno));
+        return -1;
+      }
+    }
+
+    /* Open data connection */
+    if (!opt_STAT) {
+      session.sf_flags |= SF_ASCII_OVERRIDE;
+      if (pr_data_open(NULL, "file list", PR_NETIO_IO_WR, 0) < 0)
+        return -1;
+    }
+
     /* Check perms on the directory/file we are about to scan. */
-    if (!ls_perms_full(cmd->tmp_pool, cmd,
-        (*pbuffer ? (char *) pbuffer : (char *) arg), NULL)) {
+    if (!ls_perms_full(cmd->tmp_pool, cmd, target, NULL)) {
       a = -1;
       skiparg = TRUE;
 
@@ -1319,16 +1398,17 @@ static int dolist(cmd_rec *cmd, const char *opt, int clearflags) {
 
       skiparg = FALSE;
 
-      if (use_globbing)
-        a = pr_fs_glob(*pbuffer ? pbuffer : arg, glob_flags, NULL, &g);
+      if (use_globbing &&
+          strpbrk(target, "{[*?") != NULL) {
+        a = pr_fs_glob(target, glob_flags, NULL, &g);
+        globbed = TRUE;
 
-      else {
+      } else {
 
         /* Trick the following code into using the non-glob() processed path */
         a = 0;
         g.gl_pathv = (char **) pcalloc(cmd->tmp_pool, 2 * sizeof(char *));
-        g.gl_pathv[0] = (char *) pstrdup(cmd->tmp_pool,
-          *pbuffer ? pbuffer : arg);
+        g.gl_pathv[0] = (char *) pstrdup(cmd->tmp_pool, target);
         g.gl_pathv[1] = NULL;
       }
     }
@@ -1342,7 +1422,7 @@ static int dolist(cmd_rec *cmd, const char *opt, int clearflags) {
        * directories as files for listing purposes.
        */
       if (use_globbing &&
-          strpbrk((*pbuffer ? pbuffer : arg), "{[*?") != NULL &&
+          strpbrk(target, "{[*?") != NULL &&
           !opt_R)
         list_dir_as_file = TRUE;
 
@@ -1369,7 +1449,7 @@ static int dolist(cmd_rec *cmd, const char *opt, int clearflags) {
               (S_ISDIR(target_mode) && list_dir_as_file)) {
             if (listfile(cmd, NULL, *path) < 0) {
               ls_terminate();
-              if (use_globbing)
+              if (use_globbing && globbed)
                 pr_fs_globfree(&g);
               return -1;
             }
@@ -1385,8 +1465,9 @@ static int dolist(cmd_rec *cmd, const char *opt, int clearflags) {
 
       if (outputfiles(cmd) < 0) {
         ls_terminate();
-        if (use_globbing)
+        if (use_globbing && globbed) {
           pr_fs_globfree(&g);
+        }
         return -1;
       }
 
@@ -1423,7 +1504,7 @@ static int dolist(cmd_rec *cmd, const char *opt, int clearflags) {
 
             } else if (res < 0) {
               ls_terminate();
-              if (use_globbing)
+              if (use_globbing && globbed)
                 pr_fs_globfree(&g);
               return -1;
             }
@@ -1432,7 +1513,7 @@ static int dolist(cmd_rec *cmd, const char *opt, int clearflags) {
 
         if (XFER_ABORTED) {
           discard_output();
-          if (use_globbing)
+          if (use_globbing && globbed)
             pr_fs_globfree(&g);
           return -1;
         }
@@ -1451,7 +1532,7 @@ static int dolist(cmd_rec *cmd, const char *opt, int clearflags) {
         pr_response_add(R_226, "Unknown error during globbing of %s", arg);
     }
 
-    if (!skiparg && use_globbing)
+    if (!skiparg && use_globbing && globbed)
       pr_fs_globfree(&g);
 
     if (XFER_ABORTED) {
@@ -1460,6 +1541,14 @@ static int dolist(cmd_rec *cmd, const char *opt, int clearflags) {
     }
 
   } else {
+
+    /* Open data connection */
+    if (!opt_STAT) {
+      session.sf_flags |= SF_ASCII_OVERRIDE;
+      if (pr_data_open(NULL, "file list", PR_NETIO_IO_WR, 0) < 0)
+        return -1;
+    }
+
     if (ls_perms_full(cmd->tmp_pool, cmd, ".", NULL)) {
 
       if (opt_d) {
@@ -1708,7 +1797,7 @@ MODRET genericlist(cmd_rec *cmd) {
     list_options = c->argv[0];
     list_strict_opts = *((unsigned char *) c->argv[1]);
 
-    list_ndepth.max = *((unsigned char *) c->argv[2]);
+    list_ndepth.max = *((unsigned int *) c->argv[2]);
 
     /* We add one to the configured maxdepth in order to allow it to
      * function properly: if one configures a maxdepth of 2, one should
@@ -1719,8 +1808,8 @@ MODRET genericlist(cmd_rec *cmd) {
     if (list_ndepth.max)
       list_ndepth.max += 1;
 
-    list_nfiles.max = *((unsigned char *) c->argv[3]);
-    list_ndirs.max = *((unsigned char *) c->argv[4]);
+    list_nfiles.max = *((unsigned int *) c->argv[3]);
+    list_ndirs.max = *((unsigned int *) c->argv[4]);
   }
 
   fakeuser = get_param_ptr(CURRENT_CONF, "DirFakeUser", FALSE);
@@ -1771,6 +1860,7 @@ MODRET ls_err_nlst(cmd_rec *cmd) {
 }
 
 MODRET ls_stat(cmd_rec *cmd) {
+  int res;
   char *arg = cmd->arg;
   unsigned char *tmp = NULL;
   mode_t *fake_mode = NULL;
@@ -1797,7 +1887,7 @@ MODRET ls_stat(cmd_rec *cmd) {
 
     if (session.total_bytes)
       pr_response_add(R_DUP, "Total bytes transferred for session: %" PR_LU,
-        session.total_bytes);
+        (pr_off_t) session.total_bytes);
 
     if (session.sf_flags & SF_XFER) {
 
@@ -1812,12 +1902,13 @@ MODRET ls_stat(cmd_rec *cmd) {
       if (session.xfer.file_size)
         pr_response_add(R_DUP, "%s %s (%" PR_LU "/%" PR_LU ")",
           session.xfer.direction == PR_NETIO_IO_RD ? C_STOR : C_RETR,
-          session.xfer.path, session.xfer.file_size, session.xfer.total_bytes);
+          session.xfer.path, (pr_off_t) session.xfer.file_size,
+          (pr_off_t) session.xfer.total_bytes);
 
       else
         pr_response_add(R_DUP, "%s %s (%" PR_LU ")",
           session.xfer.direction == PR_NETIO_IO_RD ? C_STOR : C_RETR,
-          session.xfer.path, session.xfer.total_bytes);
+          session.xfer.path, (pr_off_t) session.xfer.total_bytes);
 
     } else
       pr_response_add(R_DUP, "No data connection");
@@ -1847,7 +1938,7 @@ MODRET ls_stat(cmd_rec *cmd) {
     list_options = c->argv[0];
     list_strict_opts = *((unsigned char *) c->argv[1]);
 
-    list_ndepth.max = *((unsigned char *) c->argv[2]);
+    list_ndepth.max = *((unsigned int *) c->argv[2]);
 
     /* We add one to the configured maxdepth in order to allow it to
      * function properly: if one configures a maxdepth of 2, one should
@@ -1858,8 +1949,8 @@ MODRET ls_stat(cmd_rec *cmd) {
     if (list_ndepth.max)
       list_ndepth.max += 1;
 
-    list_nfiles.max = *((unsigned char *) c->argv[3]);
-    list_ndirs.max = *((unsigned char *) c->argv[4]);
+    list_nfiles.max = *((unsigned int *) c->argv[3]);
+    list_ndirs.max = *((unsigned int *) c->argv[4]);
   }
 
   fakeuser = get_param_ptr(CURRENT_CONF, "DirFakeUser", FALSE);
@@ -1888,9 +1979,9 @@ MODRET ls_stat(cmd_rec *cmd) {
   opt_a = opt_l = opt_STAT = 1;
 
   pr_response_add(R_211, "Status of %s:", arg && *arg ? arg : ".");
-  dolist(cmd, cmd->arg, FALSE);
+  res = dolist(cmd, arg && *arg ? arg : ".", FALSE);
   pr_response_add(R_211, "End of Status");
-  return HANDLED(cmd);
+  return (res == -1 ? ERROR(cmd) : HANDLED(cmd));
 }
 
 MODRET ls_list(cmd_rec *cmd) {
@@ -1907,7 +1998,7 @@ MODRET ls_list(cmd_rec *cmd) {
  */
 
 MODRET ls_nlst(cmd_rec *cmd) {
-  char *target,line[PR_TUNABLE_PATH_MAX + 1] = {'\0'};
+  char *target, buf[PR_TUNABLE_PATH_MAX + 1] = {'\0'};
   config_rec *c = NULL;
   int count = 0, res = 0, hidden = 0;
   int glob_flags = GLOB_PERIOD;
@@ -1927,7 +2018,7 @@ MODRET ls_nlst(cmd_rec *cmd) {
     list_options = c->argv[0];
     list_strict_opts = *((unsigned char *) c->argv[1]);
 
-    list_ndepth.max = *((unsigned char *) c->argv[2]);
+    list_ndepth.max = *((unsigned int *) c->argv[2]);
 
     /* We add one to the configured maxdepth in order to allow it to
      * function properly: if one configures a maxdepth of 2, one should
@@ -1938,13 +2029,13 @@ MODRET ls_nlst(cmd_rec *cmd) {
     if (list_ndepth.max)
       list_ndepth.max += 1;
 
-    list_nfiles.max = *((unsigned char *) c->argv[3]);
-    list_ndirs.max = *((unsigned char *) c->argv[4]);
+    list_nfiles.max = *((unsigned int *) c->argv[3]);
+    list_ndirs.max = *((unsigned int *) c->argv[4]);
   }
 
   /* Clear the listing option flags. */
-  opt_a = opt_C = opt_d = opt_F = opt_n = opt_r = opt_R = opt_t = opt_STAT =
-    opt_L = 0;
+  opt_a = opt_C = opt_d = opt_F = opt_n = opt_r = opt_R = opt_S = opt_t =
+    opt_STAT = opt_L = 0;
 
   if (!list_strict_opts) {
     parse_list_opts(&target, &glob_flags, FALSE);
@@ -1983,19 +2074,50 @@ MODRET ls_nlst(cmd_rec *cmd) {
 
     p++;
 
-    while (*p && *p !='/' && i < PR_TUNABLE_PATH_MAX)
+    while (*p && *p != '/' && i < PR_TUNABLE_PATH_MAX)
       pb[i++] = *p++;
     pb[i] = '\0';
 
-    if ((pw = auth_getpwnam(cmd->tmp_pool,i ? pb : session.user))) {
+    pw = pr_auth_getpwnam(cmd->tmp_pool, i ? pb : session.user);
+    if (pw) {
       snprintf(pb, sizeof(pb), "%s%s", pw->pw_dir, p);
-      sstrncpy(line, pb, sizeof(line));
-      target = line;
+      sstrncpy(buf, pb, sizeof(buf));
+      target = buf;
     }
   }
 
-  /* If the target is a glob, get the listing of files/dirs to send
-   */
+  /* Clean the path. */
+  if (*target != '/') {
+    size_t cwdlen = strlen(pr_fs_getcwd());
+
+    pr_fs_clean_path(pdircat(cmd->tmp_pool, pr_fs_getcwd(), target, NULL),
+      buf, sizeof(buf));
+
+    target = buf;
+
+    /* If the given target was not an absolute path, advance past the
+     * current working directory prefix in the cleaned up target path.
+     */
+    target += cwdlen;
+
+    /* If the length of the current working directory (cwdlen) is one,
+     * it means that the current working directory is the root ('/'),
+     * and so we don't want to advance past that into the file name
+     * portion of the path.
+     */
+    if (cwdlen > 1)
+      target += 1;
+
+  } else {
+    pr_fs_clean_path(target, buf, sizeof(buf));
+    target = buf;
+  }
+
+  /* Remove any trailing separators. */
+  while (target[strlen(target)-1] == '/')
+    target[strlen(target)-1] = '\0';
+
+  /* If the target is a glob, get the listing of files/dirs to send. */
   if (use_globbing && strpbrk(target, "{[*?") != NULL) {
     glob_t g;
     char **path,*p;
@@ -2003,7 +2125,7 @@ MODRET ls_nlst(cmd_rec *cmd) {
     /* Make sure the glob_t is initialized */
     memset(&g, '\0', sizeof(glob_t));
 
-    if (pr_fs_glob(target, GLOB_PERIOD,NULL, &g) != 0) {
+    if (pr_fs_glob(target, GLOB_PERIOD, NULL, &g) != 0) {
       pr_response_add_err(R_450, "No files found");
       return ERROR(cmd);
     }
@@ -2030,7 +2152,7 @@ MODRET ls_nlst(cmd_rec *cmd) {
           if (hidden)
             continue;
 
-          res = nlstfile(cmd,p);
+          res = nlstfile(cmd, p);
         }
 
         if (res > 0)
@@ -2073,6 +2195,7 @@ MODRET ls_nlst(cmd_rec *cmd) {
     /* Make sure the target is a file or directory, and that we have access
      * to it.
      */
+    pr_fs_clear_cache();
     if (pr_fsio_stat(target, &st) < 0) {
       pr_response_add_err(R_450, "%s: %s", cmd->arg, strerror(errno));
       return ERROR(cmd);
@@ -2082,7 +2205,8 @@ MODRET ls_nlst(cmd_rec *cmd) {
       res = nlstfile(cmd, target);
 
     else if (S_ISDIR(st.st_mode)) {
-      if (access_check(target, R_OK) != 0) {
+      if (pr_fsio_access(target, R_OK, session.uid, session.gid,
+          session.gids) != 0) {
         pr_response_add_err(R_450, "%s: %s", cmd->arg, strerror(errno));
         return ERROR(cmd);
       }
@@ -2103,16 +2227,10 @@ MODRET ls_nlst(cmd_rec *cmd) {
     res = -1;
 
   } else {
-    if (res == 0 && !count && (session.sf_flags & SF_XFER) == 0) {
-      pr_response_add_err(R_450, "No files found");
-      res = -1;
-
-    } else if (session.sf_flags & SF_XFER)
-
-      /* Note that the data connection is NOT cleared here, as an error in
-       * NLST still leaves data ready for another command.
-       */
-      ls_done(cmd);
+    /* Note that the data connection is NOT cleared here, as an error in
+     * NLST still leaves data ready for another command.
+     */
+    ls_done(cmd);
   }
 
   return (res < 0 ? ERROR(cmd) : HANDLED(cmd));
@@ -2305,10 +2423,6 @@ MODRET set_useglobbing(cmd_rec *cmd) {
   return HANDLED(cmd);
 }
 
-MODRET set_lsdefaultoptions(cmd_rec *cmd) {
-  CONF_ERROR(cmd, "deprecated. Use ListOptions instead");
-}
-
 /* Initialization routines
  */
 
@@ -2332,10 +2446,6 @@ static conftable ls_conftab[] = {
   { "ListOptions",	set_listoptions,			NULL },
   { "ShowSymlinks",	set_showsymlinks,			NULL },
   { "UseGlobbing",	set_useglobbing,			NULL },
-
-  /* Deprecated */
-  { "LsDefaultOptions",	set_lsdefaultoptions,			NULL },
-
   { NULL,		NULL,					NULL }
 };
 
