@@ -25,7 +25,7 @@
 
 /*
  * "SITE" commands module for ProFTPD
- * $Id: mod_site.c,v 1.41 2004/02/23 23:27:15 castaglia Exp $
+ * $Id: mod_site.c,v 1.44 2004/12/12 05:59:27 castaglia Exp $
  */
 
 #include "conf.h"
@@ -117,7 +117,8 @@ MODRET site_chgrp(cmd_rec *cmd) {
   if (tmp && *tmp) {
 
     /* Try the parameter as a user name. */
-    if ((gid = auth_name_gid(cmd->tmp_pool, cmd->argv[1])) == -1) {
+    gid = pr_auth_name2gid(cmd->tmp_pool, cmd->argv[1]);
+    if (gid == (gid_t) -1) {
       pr_response_add_err(R_550, "%s: %s", arg, strerror(EINVAL));
       return ERROR(cmd);
     }
@@ -430,10 +431,9 @@ MODRET site_pre_cmd(cmd_rec *cmd) {
 MODRET site_cmd(cmd_rec *cmd) {
   char *cp = NULL;
   cmd_rec *tmpcmd = NULL;
-  MODRET res;
 
   /* Make a copy of the cmd structure for passing to call_module */
-  tmpcmd = pcalloc(cmd->tmp_pool, sizeof(cmd_rec));
+  tmpcmd = pcalloc(cmd->pool, sizeof(cmd_rec));
   memcpy(tmpcmd, cmd, sizeof(cmd_rec));
 
   tmpcmd->argc--;
@@ -441,15 +441,11 @@ MODRET site_cmd(cmd_rec *cmd) {
 
   if (tmpcmd->argc)
     for (cp = tmpcmd->argv[0]; *cp; cp++)
-      *cp = toupper(*cp);
+      *cp = toupper((int) *cp);
 
-  res = site_dispatch(tmpcmd);
+  tmpcmd->notes = cmd->notes;
 
-  /* Copy private data back to original cmd */
-  cmd->private = tmpcmd->private;
-  cmd->privarr = tmpcmd->privarr;
-
-  return res;
+  return site_dispatch(tmpcmd);
 }
 
 MODRET site_post_cmd(cmd_rec *cmd) {
