@@ -2,7 +2,7 @@
  * ProFTPD - FTP server daemon
  * Copyright (c) 1997, 1998 Public Flood Software
  * Copyright (c) 1999, 2000 MacGyver aka Habeeb J. Dihu <macgyver@tos.net>
- * Copyright (c) 2001, 2002, 2003 The ProFTPD Project team
+ * Copyright (c) 2001-2005 The ProFTPD Project team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@
  */
 
 /* Authentication front-end for ProFTPD
- * $Id: auth.c,v 1.36 2004/07/20 17:39:59 castaglia Exp $
+ * $Id: auth.c,v 1.44 2005/06/14 18:11:12 castaglia Exp $
  */
 
 #include "conf.h"
@@ -48,7 +48,7 @@ static cmd_rec *make_cmd(pool *cp, int argc, ...) {
   c->stash_index = -1;
 
   if (argc) {
-    c->argv = pcalloc(cp, sizeof(void *) * argc);
+    c->argv = pcalloc(cp, sizeof(void *) * (argc + 1));
 
     va_start(args, argc);
 
@@ -56,6 +56,8 @@ static cmd_rec *make_cmd(pool *cp, int argc, ...) {
       c->argv[i] = (void *) va_arg(args, char *);
 
     va_end(args);
+
+    c->argv[argc] = NULL;
   }
 
   return c;
@@ -88,7 +90,7 @@ static modret_t *dispatch_auth(cmd_rec *cmd, char *match) {
   return mr;
 }
 
-void auth_setpwent(pool *p) {
+void pr_auth_setpwent(pool *p) {
   cmd_rec *cmd = NULL;
   modret_t *mr = NULL;
 
@@ -103,7 +105,7 @@ void auth_setpwent(pool *p) {
   return;
 }
 
-void auth_endpwent(pool *p) {
+void pr_auth_endpwent(pool *p) {
   cmd_rec *cmd = NULL;
   modret_t *mr = NULL;
 
@@ -118,7 +120,7 @@ void auth_endpwent(pool *p) {
   return;
 }
 
-void auth_setgrent(pool *p) {
+void pr_auth_setgrent(pool *p) {
   cmd_rec *cmd = NULL;
   modret_t *mr = NULL;
 
@@ -133,7 +135,7 @@ void auth_setgrent(pool *p) {
   return;
 }
 
-void auth_endgrent(pool *p) {
+void pr_auth_endgrent(pool *p) {
   cmd_rec *cmd = NULL;
   modret_t *mr = NULL;
 
@@ -148,7 +150,7 @@ void auth_endgrent(pool *p) {
   return;
 }
 
-struct passwd *auth_getpwent(pool *p) {
+struct passwd *pr_auth_getpwent(pool *p) {
   cmd_rec *cmd = NULL;
   modret_t *mr = NULL;
   struct passwd *res = NULL;
@@ -182,7 +184,7 @@ struct passwd *auth_getpwent(pool *p) {
   return res;
 }
 
-struct group *auth_getgrent(pool *p) {
+struct group *pr_auth_getgrent(pool *p) {
   cmd_rec *cmd = NULL;
   modret_t *mr = NULL;
   struct group *res = NULL;
@@ -211,7 +213,7 @@ struct group *auth_getgrent(pool *p) {
   return res;
 }
 
-struct passwd *auth_getpwnam(pool *p, const char *name) {
+struct passwd *pr_auth_getpwnam(pool *p, const char *name) {
   cmd_rec *cmd = NULL;
   modret_t *mr = NULL;
   struct passwd *res = NULL;
@@ -244,15 +246,17 @@ struct passwd *auth_getpwnam(pool *p, const char *name) {
     return NULL;
   }
 
+  pr_log_debug(DEBUG10, "retrieved UID %lu for user '%s'",
+    (unsigned long) res->pw_uid, name);
   return res;
 }
 
-struct passwd *auth_getpwuid(pool *p, uid_t uid) {
+struct passwd *pr_auth_getpwuid(pool *p, uid_t uid) {
   cmd_rec *cmd = NULL;
   modret_t *mr = NULL;
   struct passwd *res = NULL;
 
-  cmd = make_cmd(p, 1, (void *) uid);
+  cmd = make_cmd(p, 1, (void *) &uid);
   mr = dispatch_auth(cmd, "getpwuid");
 
   if (MODRET_ISHANDLED(mr) && MODRET_HASDATA(mr))
@@ -265,7 +269,7 @@ struct passwd *auth_getpwuid(pool *p, uid_t uid) {
 
   /* Sanity check */
   if (res == NULL) {
-    pr_log_pri(PR_LOG_NOTICE, "no such UID '%lu'", (unsigned long)uid);
+    pr_log_pri(PR_LOG_NOTICE, "no such UID %lu", (unsigned long) uid);
     return NULL;
   }
 
@@ -280,10 +284,12 @@ struct passwd *auth_getpwuid(pool *p, uid_t uid) {
     return NULL;
   }
 
+  pr_log_debug(DEBUG10, "retrieved user '%s' for UID %lu",
+    res->pw_name, (unsigned long) uid);
   return res;
 }
 
-struct group *auth_getgrnam(pool *p, const char *name) {
+struct group *pr_auth_getgrnam(pool *p, const char *name) {
   cmd_rec *cmd = NULL;
   modret_t *mr = NULL;
   struct group *res = NULL;
@@ -311,15 +317,17 @@ struct group *auth_getgrnam(pool *p, const char *name) {
     return NULL;
   }
 
+  pr_log_debug(DEBUG10, "retrieved GID %lu for group '%s'",
+    (unsigned long) res->gr_gid, name);
   return res;
 }
 
-struct group *auth_getgrgid(pool *p, gid_t gid) {
+struct group *pr_auth_getgrgid(pool *p, gid_t gid) {
   cmd_rec *cmd = NULL;
   modret_t *mr = NULL;
   struct group *res = NULL;
 
-  cmd = make_cmd(p, 1, (void *) gid);
+  cmd = make_cmd(p, 1, (void *) &gid);
   mr = dispatch_auth(cmd, "getgrgid");
 
   if (MODRET_ISHANDLED(mr) && MODRET_HASDATA(mr))
@@ -332,7 +340,7 @@ struct group *auth_getgrgid(pool *p, gid_t gid) {
 
   /* Sanity check */
   if (res == NULL) {
-    pr_log_pri(PR_LOG_NOTICE, "no such GID '%lu'", (unsigned long)gid);
+    pr_log_pri(PR_LOG_NOTICE, "no such GID %lu", (unsigned long) gid);
     return NULL;
   }
 
@@ -342,10 +350,12 @@ struct group *auth_getgrgid(pool *p, gid_t gid) {
     return NULL;
   }
 
+  pr_log_debug(DEBUG10, "retrieved group '%s' for GID %lu",
+    res->gr_name, (unsigned long) gid);
   return res;
 }
 
-int auth_authenticate(pool *p, const char *name, const char *pw) {
+int pr_auth_authenticate(pool *p, const char *name, const char *pw) {
   cmd_rec *cmd = NULL;
   modret_t *mr = NULL;
   int res = PR_AUTH_NOPWD;
@@ -367,7 +377,7 @@ int auth_authenticate(pool *p, const char *name, const char *pw) {
   return res;
 }
 
-int auth_check(pool *p, const char *cpw, const char *name, const char *pw) {
+int pr_auth_check(pool *p, const char *cpw, const char *name, const char *pw) {
   cmd_rec *cmd = NULL;
   modret_t *mr = NULL;
   int res = PR_AUTH_BADPWD;
@@ -386,7 +396,29 @@ int auth_check(pool *p, const char *cpw, const char *name, const char *pw) {
   return res;
 }
 
-const char *auth_uid_name(pool *p, uid_t uid) {
+int pr_auth_requires_pass(pool *p, const char *name) {
+  cmd_rec *cmd;
+  modret_t *mr;
+  int res = TRUE;
+
+  cmd = make_cmd(p, 1, name);
+  mr = dispatch_auth(cmd, "requires_pass");
+
+  if (MODRET_ISHANDLED(mr))
+    res = FALSE;
+
+  else if (MODRET_ISERROR(mr))
+    res = MODRET_ERROR(mr);
+
+  if (cmd->tmp_pool) {
+    destroy_pool(cmd->tmp_pool);
+    cmd->tmp_pool = NULL;
+  }
+
+  return res;
+}
+
+const char *pr_auth_uid2name(pool *p, uid_t uid) {
   cmd_rec *cmd = NULL;
   modret_t *mr = NULL;
   static char namebuf[64];
@@ -394,8 +426,8 @@ const char *auth_uid_name(pool *p, uid_t uid) {
 
   memset(namebuf, '\0', sizeof(namebuf));
 
-  cmd = make_cmd(p, 1, (void *) uid);
-  mr = dispatch_auth(cmd, "uid_name");
+  cmd = make_cmd(p, 1, (void *) &uid);
+  mr = dispatch_auth(cmd, "uid2name");
 
   if (MODRET_ISHANDLED(mr) && MODRET_HASDATA(mr)) {
     res = mr->data;
@@ -411,7 +443,7 @@ const char *auth_uid_name(pool *p, uid_t uid) {
   return res;
 }
 
-const char *auth_gid_name(pool *p, gid_t gid) {
+const char *pr_auth_gid2name(pool *p, gid_t gid) {
   cmd_rec *cmd = NULL;
   modret_t *mr = NULL;
   static char namebuf[64];
@@ -419,8 +451,8 @@ const char *auth_gid_name(pool *p, gid_t gid) {
 
   memset(namebuf, '\0', sizeof(namebuf));
 
-  cmd = make_cmd(p, 1, (void *) gid);
-  mr = dispatch_auth(cmd, "gid_name");
+  cmd = make_cmd(p, 1, (void *) &gid);
+  mr = dispatch_auth(cmd, "gid2name");
 
   if (MODRET_ISHANDLED(mr) && MODRET_HASDATA(mr)) {
     res = mr->data;
@@ -436,16 +468,16 @@ const char *auth_gid_name(pool *p, gid_t gid) {
   return res;
 }
 
-uid_t auth_name_uid(pool *p, const char *name) {
+uid_t pr_auth_name2uid(pool *p, const char *name) {
   cmd_rec *cmd = NULL;
   modret_t *mr = NULL;
-  uid_t res = -1;
+  uid_t res = (uid_t) -1;
 
   cmd = make_cmd(p, 1, name);
-  mr = dispatch_auth(cmd, "name_uid");
+  mr = dispatch_auth(cmd, "name2uid");
 
   if (MODRET_ISHANDLED(mr))
-    res = (uid_t) mr->data;
+    res = *((uid_t *) mr->data);
   else
     errno = EINVAL;
 
@@ -457,16 +489,16 @@ uid_t auth_name_uid(pool *p, const char *name) {
   return res;
 }
 
-gid_t auth_name_gid(pool *p, const char *name) {
+gid_t pr_auth_name2gid(pool *p, const char *name) {
   cmd_rec *cmd = NULL;
   modret_t *mr = NULL;
-  gid_t res = -1;
+  gid_t res = (gid_t) -1;
 
   cmd = make_cmd(p, 1, name);
-  mr = dispatch_auth(cmd, "name_gid");
+  mr = dispatch_auth(cmd, "name2gid");
 
   if (MODRET_ISHANDLED(mr))
-    res = (gid_t) mr->data;
+    res = *((gid_t *) mr->data);
   else
     errno = EINVAL;
 
@@ -478,8 +510,8 @@ gid_t auth_name_gid(pool *p, const char *name) {
   return res;
 }
 
-int auth_getgroups(pool *p, const char *name, array_header **group_ids,
-               array_header **group_names) {
+int pr_auth_getgroups(pool *p, const char *name, array_header **group_ids,
+    array_header **group_names) {
 
   cmd_rec *cmd = NULL;
   modret_t *mr = NULL;
@@ -498,7 +530,7 @@ int auth_getgroups(pool *p, const char *name, array_header **group_ids,
   mr = dispatch_auth(cmd, "getgroups");
 
   if (MODRET_ISHANDLED(mr) && MODRET_HASDATA(mr)) {
-    res = (int) mr->data;
+    res = *((int *) mr->data);
 
     /* Note: the number of groups returned should, barring error,
      * always be at least 1, as per getgroups(2) behavior.  This one
@@ -545,6 +577,162 @@ int auth_getgroups(pool *p, const char *name, array_header **group_ids,
   return res;
 }
 
+/* Helper function for pr_auth_get_anon_config(), for handling the
+ * (horrible) AnonymousGroup directive.
+ */
+static config_rec *auth_anonymous_group(pool *p, char *user) {
+  config_rec *c;
+  int ret = 0;
+
+  /* Retrieve the session group membership information, so that this check
+   * may work properly.
+   */
+  if (!session.gids && !session.groups &&
+      (ret = pr_auth_getgroups(p, user, &session.gids, &session.groups)) < 1)
+    pr_log_debug(DEBUG2, "no supplemental groups found for user '%s'", user);
+
+  c = find_config(main_server->conf, CONF_PARAM, "AnonymousGroup", FALSE);
+
+  if (c)
+    do {
+      ret = pr_expr_eval_group_and((char **) c->argv);
+
+    } while (!ret &&
+      (c = find_config_next(c, c->next, CONF_PARAM, "AnonymousGroup",
+        FALSE)) != NULL);
+
+  return ret ? c : NULL;
+}
+
+/* This is one messy function.  Yuck.  Yay legacy code. */
+config_rec *pr_auth_get_anon_config(pool *p, char **login_name,
+    char **user_name, char **anon_name) {
+  config_rec *c = NULL, *topc = NULL;
+  char *config_user_name, *config_anon_name = NULL;
+  unsigned char is_alias = FALSE, force_anon = FALSE, *auth_alias_only = NULL;
+
+  /* Precendence rules:
+   *   1. Search for UserAlias directive.
+   *   2. Search for Anonymous directive.
+   *   3. Normal user login
+   */
+
+  config_user_name = get_param_ptr(main_server->conf, "UserName", FALSE);
+  if (config_user_name && user_name)
+    *user_name = config_user_name;
+
+  c = find_config(main_server->conf, CONF_PARAM, "UserAlias", TRUE);
+  if (c) {
+    do {
+      if (strcmp(c->argv[0], "*") == 0 ||
+          strcmp(c->argv[0], *login_name) == 0) {
+        is_alias = TRUE;
+        break;
+      }
+
+    } while ((c = find_config_next(c, c->next, CONF_PARAM, "UserAlias",
+      TRUE)) != NULL);
+  }
+
+  /* This is where things get messy, rapidly. */
+  topc = c;
+
+  while (c && c->parent &&
+    (auth_alias_only = get_param_ptr(c->parent->set, "AuthAliasOnly", FALSE))) {
+
+    /* while() loops should always handle signals. */
+    pr_signals_handle();
+
+    /* If AuthAliasOnly is on, ignore this one and continue. */
+    if (auth_alias_only &&
+        *auth_alias_only == TRUE) {
+      c = find_config_next(c, c->next, CONF_PARAM, "UserAlias", TRUE);
+      continue;
+    }
+
+    is_alias = FALSE;
+
+    find_config_set_top(topc);
+    c = find_config_next(c, c->next, CONF_PARAM, "UserAlias", TRUE);
+
+    if (c &&
+        (strcmp(c->argv[0], "*") == 0 ||
+         strcmp(c->argv[0], *login_name) == 0))
+      is_alias = TRUE;
+  }
+
+  if (c) {
+    *login_name = c->argv[1];
+
+    /* If the alias is applied inside an <Anonymous> context, we have found
+     * our anon block.
+     */
+    if (c->parent &&
+        c->parent->config_type == CONF_ANON)
+      c = c->parent;
+    else
+      c = NULL;
+  }
+
+  /* Next, search for an anonymous entry. */
+
+  if (!c)
+    c = find_config(main_server->conf, CONF_ANON, NULL, FALSE);
+  else
+    find_config_set_top(c);
+
+  if (c) {
+    do {
+      config_anon_name = get_param_ptr(c->subset, "UserName", FALSE);
+
+      if (!config_anon_name)
+        config_anon_name = config_user_name;
+
+      if (config_anon_name &&
+          strcmp(config_anon_name, *login_name) == 0) {
+         if (anon_name)
+           *anon_name = config_anon_name;
+         break;
+      }
+ 
+    } while ((c = find_config_next(c, c->next, CONF_ANON, NULL,
+      FALSE)) != NULL);
+  }
+
+  if (!c) {
+    c = auth_anonymous_group(p, *login_name);
+
+    if (c)
+      force_anon = TRUE;
+  }
+
+  if (!is_alias && !force_anon) {
+    auth_alias_only = get_param_ptr(c ? c->subset : main_server->conf,
+      "AuthAliasOnly", FALSE);
+
+    if (auth_alias_only &&
+        *auth_alias_only == TRUE) {
+      if (c && c->config_type == CONF_ANON)
+        c = NULL;
+      else
+        *login_name = NULL;
+
+      auth_alias_only = get_param_ptr(main_server->conf, "AuthAliasOnly",
+        FALSE);
+      if (*login_name &&
+          auth_alias_only &&
+          *auth_alias_only == TRUE)
+        *login_name = NULL;
+
+      if ((!login_name || !c) &&
+          anon_name)
+        *anon_name = NULL;
+    }
+  }
+
+  return c;
+}
+
 int set_groups(pool *p, gid_t primary_gid, array_header *suppl_gids) {
   int res = 0;
   pool *tmp_pool = NULL;
@@ -609,7 +797,8 @@ int set_groups(pool *p, gid_t primary_gid, array_header *suppl_gids) {
     strgids);
 
   /* Set the supplemental groups. */
-  if ((res = setgroups(nproc_gids, proc_gids)) < 0) {
+  res = setgroups(nproc_gids, proc_gids);
+  if (res < 0) {
     destroy_pool(tmp_pool);
     return res;
   }
@@ -618,7 +807,8 @@ int set_groups(pool *p, gid_t primary_gid, array_header *suppl_gids) {
 #ifndef PR_DEVEL_COREDUMP
   /* Set the primary GID of the process.
    */
-  if ((res = setgid(primary_gid)) < 0) {
+  res = setgid(primary_gid);
+  if (res < 0) {
     if (tmp_pool)
       destroy_pool(tmp_pool);
     return res;
