@@ -24,7 +24,7 @@
 
 /* Routines to work with ProFTPD bindings
  *
- * $Id: bindings.c,v 1.17 2003/09/29 17:25:19 castaglia Exp $
+ * $Id: bindings.c,v 1.30 2005/06/21 20:54:23 castaglia Exp $
  */
 
 #include "conf.h"
@@ -32,19 +32,19 @@
 /* Some convenience macros */
 #define PR_CLOSE_NAMEBIND(n, a, p) \
   if ((res = pr_namebind_close((n), (a), (p))) < 0) \
-    log_pri(PR_LOG_NOTICE, \
+    pr_log_pri(PR_LOG_NOTICE, \
       "%s:%d: notice, unable to close namebind '%s': %s", \
       __FILE__, __LINE__, (n), strerror(errno))
 
 #define PR_CREATE_NAMEBIND(s, n, a, p) \
   if ((res = pr_namebind_create((s), (n), (a), (p))) < 0) \
-    log_pri(PR_LOG_NOTICE, \
+    pr_log_pri(PR_LOG_NOTICE, \
       "%s:%d: notice: unable to create namebind '%s': %s", \
       __FILE__, __LINE__, (n), strerror(errno))
 
 #define PR_OPEN_NAMEBIND(n, a, p) \
   if ((res = pr_namebind_open((n), (a), (p))) < 0) \
-    log_pri(PR_LOG_NOTICE, \
+    pr_log_pri(PR_LOG_NOTICE, \
       "%s:%d: notice: unable to open namebind '%s': %s", \
       __FILE__, __LINE__, (n), strerror(errno))
 
@@ -57,6 +57,10 @@ static pool *binding_pool = NULL;
 static pr_ipbind_t *ipbind_default_server = NULL,
                    *ipbind_localhost_server = NULL;
 
+#ifdef PR_USE_IPV6
+static int ipv6_supported = TRUE;
+#endif /* PR_USE_IPV6 */
+
 /* Server cleanup callback function */
 static void server_cleanup_cb(void *conn) {
   *((conn_t **) conn) = NULL;
@@ -87,12 +91,11 @@ static unsigned int ipbind_hash_addr(pr_netaddr_t *addr) {
  * pr_ipbind_accept_conn() need merely check those listeners, rather than
  * scanning the entire table itself?
  */
-static conn_t *listener_list = NULL;
-static unsigned int listener_listlen = 0;
+
+static array_header *listener_list = NULL;
 
 conn_t *pr_ipbind_accept_conn(fd_set *readfds, int *listenfd) {
-  conn_t *listener = listener_list;
-  int fd = -1;
+  conn_t **listeners = listener_list->elts;
   register unsigned int i = 0;
 
   /* sanity checks */
@@ -106,21 +109,21 @@ conn_t *pr_ipbind_accept_conn(fd_set *readfds, int *listenfd) {
     return NULL;
   }
 
-  for (i = 0, listener = listener_list; i < listener_listlen;
-      i++, listener = listener->next) {
-    pr_signals_handle();
+  for (i = 0; i < listener_list->nelts; i++) {
+    conn_t *listener = listeners[i];
 
+    pr_signals_handle();
     if (FD_ISSET(listener->listen_fd, readfds) &&
         listener->mode == CM_LISTEN) {
+      int fd = pr_inet_accept_nowait(listener->pool, listener);
 
-      if ((fd = pr_inet_accept_nowait(listener->pool, listener)) == -1) {
-
+      if (fd == -1) {
         /* Handle errors gracefully.  If we're here, then
          * ipbind->ib_server->listen contains either error information, or
          * we just got caught in a blocking condition.
          */
         if (listener->mode == CM_ERROR) {
-          log_pri(PR_LOG_ERR, "error: unable to accept an incoming "
+          pr_log_pri(PR_LOG_ERR, "error: unable to accept an incoming "
             "connection (%s)", strerror(listener->xerrno));
           listener->xerrno = 0;
           listener->mode = CM_LISTEN;
@@ -133,6 +136,7 @@ conn_t *pr_ipbind_accept_conn(fd_set *readfds, int *listenfd) {
     }
   }
 
+  errno = ENOENT;
   return NULL;
 }
 
@@ -146,16 +150,16 @@ int pr_ipbind_add_binds(server_rec *serv) {
   if (!serv)
     return -1;
 
-  c = find_config(serv->conf, CONF_PARAM, "Bind", FALSE);
+  c = find_config(serv->conf, CONF_PARAM, "_bind", FALSE);
 
   while (c) {
     listen_conn = NULL;
 
     addr = pr_netaddr_get_addr(serv->pool, c->argv[0], NULL);
     if (!addr) {
-      log_pri(PR_LOG_NOTICE, "notice: unable to determine IP address of '%s'",
-        (char *) c->argv[0]);
-      c = find_config_next(c, c->next, CONF_PARAM, "Bind", FALSE);
+      pr_log_pri(PR_LOG_NOTICE,
+       "notice: unable to determine IP address of '%s'", (char *) c->argv[0]);
+      c = find_config_next(c, c->next, CONF_PARAM, "_bind", FALSE);
       continue;
     }
 
@@ -175,8 +179,8 @@ int pr_ipbind_add_binds(server_rec *serv) {
       PR_OPEN_IPBIND(addr, serv->ServerPort, serv->listen, FALSE, FALSE, TRUE);
     }
 
-    /* move on to the next Bind directive */
-    c = find_config_next(c, c->next, CONF_PARAM, "Bind", FALSE);
+    /* Move on to the next bind directive */
+    c = find_config_next(c, c->next, CONF_PARAM, "_bind", FALSE);
   }
 
   /* done */
@@ -195,7 +199,7 @@ int pr_ipbind_close(pr_netaddr_t *addr, unsigned int port,
     i = ipbind_hash_addr(addr);
 
     if (ipbind_table[i] == NULL) {
-      log_pri(PR_LOG_NOTICE, "notice: no ipbind found for %s:%d",
+      pr_log_pri(PR_LOG_NOTICE, "notice: no ipbind found for %s:%d",
         pr_netaddr_get_ipstr(addr), port);
       errno = ENOENT;
       return -1;
@@ -210,7 +214,7 @@ int pr_ipbind_close(pr_netaddr_t *addr, unsigned int port,
     }
 
     if (!have_ipbind) {
-      log_pri(PR_LOG_NOTICE, "notice: no ipbind found for %s:%d",
+      pr_log_pri(PR_LOG_NOTICE, "notice: no ipbind found for %s:%d",
         pr_netaddr_get_ipstr(addr), port);
       errno = ENOENT;
       return -1;
@@ -300,15 +304,18 @@ int pr_ipbind_close(pr_netaddr_t *addr, unsigned int port,
 
 /* Need a way to close all listening fds in a child process. */
 int pr_ipbind_close_listeners(void) {
-  conn_t *listener = listener_list;
+  conn_t **listeners;
   register unsigned int i = 0;
 
   /* sanity checks */
-  if (!listener)
+  if (!listener_list ||
+      listener_list->nelts == 0)
     return 0;
 
-  for (i = 0, listener = listener_list; i < listener_listlen;
-      i++, listener = listener->next) {
+  listeners = listener_list->elts;
+  for (i = 0; i < listener_list->nelts; i++) {
+    conn_t *listener = listeners[i];
+
     pr_signals_handle();
 
     if (listener->listen_fd != -1) {
@@ -341,7 +348,7 @@ int pr_ipbind_create(server_rec *server, pr_netaddr_t *addr) {
         ipbind->ib_port == server->ServerPort) {
 
       /* An ipbind already exists for this IP address */
-      log_pri(PR_LOG_NOTICE, "notice: '%s' (%s:%u) already bound to '%s'",
+      pr_log_pri(PR_LOG_NOTICE, "notice: '%s' (%s:%u) already bound to '%s'",
         server->ServerName, pr_netaddr_get_ipstr(addr), server->ServerPort,
         ipbind->ib_server->ServerName);
 
@@ -350,9 +357,11 @@ int pr_ipbind_create(server_rec *server, pr_netaddr_t *addr) {
     }
   }
 
-  if (!binding_pool)
+  if (!binding_pool) {
     /* Initialize the working pool, if not present */
     binding_pool = make_sub_pool(permanent_pool);
+    pr_pool_tag(binding_pool, "Bindings Pool");
+  }
 
   ipbind = pcalloc(server->pool, sizeof(pr_ipbind_t));
   ipbind->ib_server = server;
@@ -400,18 +409,57 @@ pr_ipbind_t *pr_ipbind_find(pr_netaddr_t *addr, unsigned int port,
   return NULL;
 }
 
+pr_ipbind_t *pr_ipbind_get(pr_ipbind_t *prev) {
+  static unsigned int i = 0;
+
+  if (prev) {
+
+    /* If there's another ipbind in this chain, simply return that. */
+    if (prev->ib_next)
+      return prev->ib_next;
+
+    /* If the increment is at the maximum size, return NULL (no more chains
+     * to be examined).
+     */
+    if (i == PR_BINDINGS_TABLE_SIZE)
+      return NULL;
+
+    /* Increment the index. At this point, we know that the given pointer is
+     * the last in the chain, and that there are more chains in the table
+     * to be examined.
+     */
+    i++;
+
+  } else
+    /* Reset the index if prev is NULL. */
+    i = 0;
+
+  /* Search for the next non-empty chain in the table. */
+  for (; i < PR_BINDINGS_TABLE_SIZE; i++) {
+    if (ipbind_table[i])
+      return ipbind_table[i];
+  }
+
+  return NULL;
+}
+
 server_rec *pr_ipbind_get_server(pr_netaddr_t *addr, unsigned int port) {
   pr_ipbind_t *ipbind = NULL;
 
   /* If we've got a binding configured for this exact address, return it
    * straightaway.
    */
-  if ((ipbind = pr_ipbind_find(addr, port, TRUE)) != NULL)
+  ipbind = pr_ipbind_find(addr, port, TRUE);
+  if (ipbind != NULL)
     return ipbind->ib_server;
 
-  /* Use the default server, if set */
-  if (ipbind_default_server && ipbind_default_server->ib_isactive)
+  /* Use the default server, if set. */
+  if (ipbind_default_server && ipbind_default_server->ib_isactive) {
+    pr_log_debug(DEBUG7, "no matching vhost found for %s#%u, using "
+      "DefaultServer '%s'", pr_netaddr_get_ipstr(addr), port,
+      ipbind_default_server->ib_server->ServerName);
     return ipbind_default_server->ib_server;
+  }
 
   /* Not found in binding list, and no DefaultServer, so see if it's the
    * loopback address
@@ -432,9 +480,21 @@ int pr_ipbind_listen(fd_set *readfds) {
 
   FD_ZERO(readfds);
 
+  if (!binding_pool) {
+    /* Initialize the working pool, if not present */
+    binding_pool = make_sub_pool(permanent_pool);
+    pr_pool_tag(binding_pool, "Bindings Pool");
+  }
+
   /* Reset the listener list. */
-  listener_list = NULL;
-  listener_listlen = 0;
+  if (!listener_list)
+    listener_list = make_array(binding_pool, 1, sizeof(conn_t *));
+
+  else
+    /* Nasty hack to "clear" the list by making it think it has no
+     * elements.
+     */
+    listener_list->nelts = 0;
 
   /* Slower than the hash lookup, but...we have to check each and every
    * ipbind in the table.
@@ -463,9 +523,7 @@ int pr_ipbind_listen(fd_set *readfds) {
             maxfd = ipbind->ib_listener->listen_fd;
 
           /* Add this to the listener list as well. */
-          ipbind->ib_listener->next = listener_list;
-          listener_list = ipbind->ib_listener;
-          listener_listlen++;
+          *((conn_t **) push_array(listener_list)) = ipbind->ib_listener;
         }
       }
     }
@@ -496,6 +554,7 @@ int pr_ipbind_open(pr_netaddr_t *addr, unsigned int port, conn_t *listen_conn,
     listen_conn->next = NULL;
 
   ipbind->ib_listener = ipbind->ib_server->listen = listen_conn;
+  ipbind->ib_listener = listen_conn;
   ipbind->ib_isdefault = isdefault;
   ipbind->ib_islocalhost = islocalhost;
 
@@ -573,7 +632,8 @@ int pr_namebind_create(server_rec *server, const char *name,
   }
 
   /* First, find the ipbind to hold this namebind. */
-  if ((ipbind = pr_ipbind_find(addr, port, FALSE)) == NULL) {
+  ipbind = pr_ipbind_find(addr, port, FALSE);
+  if (ipbind == NULL) {
     errno = ENOENT;
     return -1;
   }
@@ -727,6 +787,7 @@ void free_bindings(void) {
   if (binding_pool) {
     destroy_pool(binding_pool);
     binding_pool = NULL;
+    listener_list = NULL;
   }
 
   memset(ipbind_table, 0, sizeof(ipbind_table));
@@ -742,10 +803,10 @@ static void init_inetd_bindings(void) {
 
   /* Fill in all the important connection information. */
   if (pr_inet_get_conn_info(main_server->listen, STDIN_FILENO) == -1) {
-    log_pri(PR_LOG_ERR, "fatal: %s", strerror(errno));
+    pr_log_pri(PR_LOG_ERR, "fatal: %s", strerror(errno));
 
     if (errno == ENOTSOCK)
-      log_pri(PR_LOG_ERR, "(Running from command line? "
+      pr_log_pri(PR_LOG_ERR, "(Running from command line? "
         "Use `ServerType standalone' in config file!)");
     exit(1);
   }
@@ -788,18 +849,9 @@ static void init_inetd_bindings(void) {
 
 static void init_standalone_bindings(void) {
   int res = 0;
-  config_rec *c = NULL;
   server_rec *serv = NULL;
   unsigned char *default_server = NULL, is_default = FALSE;
 
-  /* Check for a configured DefaultAddress for the main_server */
-  if ((c = find_config(main_server->conf, CONF_PARAM, "DefaultAddress",
-      FALSE)) != NULL) {
-    log_debug(DEBUG0, "setting default server address to %s",
-      pr_netaddr_get_ipstr((pr_netaddr_t *) c->argv[0]));
-    main_server->addr = (pr_netaddr_t *) c->argv[0];
-  }
-
   /* If a port is set to zero, the address/port is not bound to a socket
    * at all.
    */
@@ -807,12 +859,22 @@ static void init_standalone_bindings(void) {
 
     /* If SocketBindTight is off, then pr_inet_create_connection() will
      * create and bind to a wildcard socket.  However, should it be an
-     * IPv4 or an IPv6 wildcard socket?  Check the main_server->addr's
-     * family, and use that.
+     * IPv4 or an IPv6 wildcard socket?
      */
-    if (!SocketBindTight)
+    if (!SocketBindTight) {
+#ifdef PR_USE_IPV6
+      if (ipv6_supported) {
+        pr_inet_set_default_family(NULL, AF_INET6);
+
+      } else {
+        pr_inet_set_default_family(NULL,
+          pr_netaddr_get_family(main_server->addr));
+      }
+#else
       pr_inet_set_default_family(NULL,
         pr_netaddr_get_family(main_server->addr));
+#endif /* PR_USE_IPV6 */
+    }
 
     main_server->listen =
       pr_inet_create_connection(main_server->pool, server_list, -1,
@@ -843,9 +905,18 @@ static void init_standalone_bindings(void) {
         is_default = TRUE;
 
       if (serv->ServerPort) {
-        if (!SocketBindTight)
-          pr_inet_set_default_family(NULL,
-            pr_netaddr_get_family(serv->addr));
+        if (!SocketBindTight) {
+#ifdef PR_USE_IPV6
+          if (ipv6_supported) {
+            pr_inet_set_default_family(NULL, AF_INET6);
+
+          } else {
+            pr_inet_set_default_family(NULL, pr_netaddr_get_family(serv->addr));
+          }
+#else
+          pr_inet_set_default_family(NULL, pr_netaddr_get_family(serv->addr));
+#endif /* PR_USE_IPV6 */
+        }
 
         serv->listen = pr_inet_create_connection(serv->pool, server_list, -1,
           (SocketBindTight ? serv->addr : NULL), serv->ServerPort, FALSE);
@@ -895,6 +966,20 @@ static void init_standalone_bindings(void) {
 }
 
 void init_bindings(void) {
+
+#ifdef PR_USE_IPV6
+  int sock;
+
+  /* Check to see whether we can actually create an IPv6 socket. */
+  sock = socket(AF_INET6, SOCK_STREAM, IPPROTO_TCP);
+  if (sock < 0) {
+    ipv6_supported = FALSE;
+
+  } else {
+    close(sock);
+  }
+#endif /* PR_USE_IPV6 */
+
   if (ServerType == SERVER_INETD)
     init_inetd_bindings();
 
