@@ -2,7 +2,7 @@
  * ProFTPD - FTP server daemon
  * Copyright (c) 1997, 1998 Public Flood Software
  * Copyright (c) 1999, 2000 MacGyver aka Habeeb J. Dihu <macgyver@tos.net>
- * Copyright (c) 2001, 2002, 2003 The ProFTPD Project team
+ * Copyright (c) 2001-2004 The ProFTPD Project team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,7 +26,7 @@
 
 /*
  * Data connection management functions
- * $Id: data.c,v 1.79 2004/08/25 18:08:46 castaglia Exp $
+ * $Id: data.c,v 1.89 2006/02/08 01:25:46 castaglia Exp $
  */
 
 #include "conf.h"
@@ -78,7 +78,7 @@ static RETSIGTYPE data_urgent(int sig) {
   signal(SIGURG, data_urgent);
 }
 
-static int xlate_ascii_read(char *buf, int *bufsize, int *adjlen) {
+static int xfrm_ascii_read(char *buf, int *bufsize, int *adjlen) {
   char *dest = buf,*src = buf;
   int thislen = *bufsize;
 
@@ -117,7 +117,7 @@ static int xlate_ascii_read(char *buf, int *bufsize, int *adjlen) {
  */
 
 static int have_dangling_cr = FALSE;
-static unsigned int xlate_ascii_write(char **buf, unsigned int *buflen,
+static unsigned int xfrm_ascii_write(char **buf, unsigned int *buflen,
     unsigned int bufsize) {
   char *tmpbuf = *buf;
   unsigned int tmplen = *buflen;
@@ -223,8 +223,8 @@ static void data_new_xfer(char *filename, int direction) {
 
   session.xfer.filename = pstrdup(session.xfer.p, filename);
   session.xfer.direction = direction;
-  session.xfer.bufsize = PR_TUNABLE_BUFFER_SIZE;
-  session.xfer.buf = pcalloc(session.xfer.p, PR_TUNABLE_BUFFER_SIZE + 1);
+  session.xfer.bufsize = PR_TUNABLE_XFER_BUFFER_SIZE;
+  session.xfer.buf = pcalloc(session.xfer.p, PR_TUNABLE_XFER_BUFFER_SIZE + 1);
   session.xfer.buf++;	/* leave room for ascii translation */
   session.xfer.buflen = 0;
 }
@@ -240,7 +240,7 @@ static int data_pasv_open(char *reason, off_t size) {
    * open from taking too long
    */
   if (TimeoutStalled)
-    add_timer(TimeoutStalled, TIMER_STALLED, NULL, stalled_timeout_cb);
+    pr_timer_add(TimeoutStalled, TIMER_STALLED, NULL, stalled_timeout_cb);
 
   /* We save the state of our current disposition for doing reverse
    * lookups, and then set it to what the configuration wants it to
@@ -279,7 +279,7 @@ static int data_pasv_open(char *reason, off_t size) {
     if (session.xfer.xfer_type != STOR_UNIQUE) {
       if (size)
         pr_response_send(R_150, "Opening %s mode data connection for %s "
-          "(%" PR_LU " bytes)", MODE_STRING, reason, size);
+          "(%" PR_LU " bytes)", MODE_STRING, reason, (pr_off_t) size);
       else
         pr_response_send(R_150, "Opening %s mode data connection for %s",
           MODE_STRING, reason);
@@ -337,7 +337,7 @@ static int data_active_open(char *reason, off_t size) {
    * open from taking too long
    */
   if (TimeoutStalled)
-    add_timer(TimeoutStalled, TIMER_STALLED, NULL, stalled_timeout_cb);
+    pr_timer_add(TimeoutStalled, TIMER_STALLED, NULL, stalled_timeout_cb);
 
   rev = pr_netaddr_set_reverse_dns(ServerUseReverseDNS);
 
@@ -379,7 +379,7 @@ static int data_active_open(char *reason, off_t size) {
     if (session.xfer.xfer_type != STOR_UNIQUE) {
       if (size)
         pr_response_send(R_150, "Opening %s mode data connection for %s "
-          "(%" PR_LU " bytes)", MODE_STRING, reason, size);
+          "(%" PR_LU " bytes)", MODE_STRING, reason, (pr_off_t) size);
       else
         pr_response_send(R_150, "Opening %s mode data connection for %s",
           MODE_STRING, reason);
@@ -481,11 +481,21 @@ int pr_data_open(char *filename, char *reason, int direction, off_t size) {
   if (res >= 0) {
     struct sigaction act;
 
-    if (pr_netio_postopen(session.d->instrm) < 0)
+    if (pr_netio_postopen(session.d->instrm) < 0) {
+      pr_response_add_err(R_425, "Unable to build data connection: %s",
+        strerror(session.d->xerrno));
+      destroy_pool(session.d->pool);
+      session.d = NULL;
       return -1;
+    }
 
-    if (pr_netio_postopen(session.d->outstrm) < 0)
+    if (pr_netio_postopen(session.d->outstrm) < 0) {
+      pr_response_add_err(R_425, "Unable to build data connection: %s",
+        strerror(session.d->xerrno));
+      destroy_pool(session.d->pool);
+      session.d = NULL;
       return -1;
+    }
 
     memset(&session.xfer.start_time, '\0', sizeof(session.xfer.start_time));
     gettimeofday(&session.xfer.start_time, NULL);
@@ -499,7 +509,7 @@ int pr_data_open(char *filename, char *reason, int direction, off_t size) {
     session.sf_flags |= SF_XFER;
 
     if (TimeoutNoXfer)
-      reset_timer(TIMER_NOXFER, ANY_MODULE);
+      pr_timer_reset(TIMER_NOXFER, ANY_MODULE);
 
     /* Allow aborts. */
 
@@ -546,10 +556,10 @@ void pr_data_close(int quiet) {
   signal(SIGURG, SIG_IGN);
 
   if (TimeoutNoXfer)
-    reset_timer(TIMER_NOXFER, ANY_MODULE);
+    pr_timer_reset(TIMER_NOXFER, ANY_MODULE);
 
   if (TimeoutStalled)
-    remove_timer(TIMER_STALLED, ANY_MODULE);
+    pr_timer_remove(TIMER_STALLED, ANY_MODULE);
 
   session.sf_flags &= (SF_ALL^SF_PASSIVE);
   session.sf_flags &= (SF_ALL^(SF_ABORT|SF_XFER|SF_PASSIVE|SF_ASCII_OVERRIDE));
@@ -590,15 +600,20 @@ void pr_data_abort(int err, int quiet) {
   nstrm = NULL;
 
   if (session.d) {
-    pr_inet_lingering_close(session.pool, session.d, timeout_linger);
+    if (!true_abort)
+      pr_inet_lingering_close(session.pool, session.d, timeout_linger);
+
+    else
+      pr_inet_lingering_abort(session.pool, session.d, timeout_linger);
+
     session.d = NULL;
   }
 
   if (TimeoutNoXfer)
-    reset_timer(TIMER_NOXFER, ANY_MODULE);
+    pr_timer_reset(TIMER_NOXFER, ANY_MODULE);
 
   if (TimeoutStalled)
-    remove_timer(TIMER_STALLED, ANY_MODULE);
+    pr_timer_remove(TIMER_STALLED, ANY_MODULE);
 
   session.sf_flags &= (SF_ALL^SF_PASSIVE);
   session.sf_flags &= (SF_ALL^(SF_XFER|SF_PASSIVE|SF_ASCII_OVERRIDE));
@@ -608,7 +623,7 @@ void pr_data_abort(int err, int quiet) {
   signal(SIGURG, SIG_IGN);
 
   if (TimeoutNoXfer)
-    reset_timer(TIMER_NOXFER, ANY_MODULE);
+    pr_timer_reset(TIMER_NOXFER, ANY_MODULE);
 
   if (!quiet) {
     char	*respcode = R_426;
@@ -750,15 +765,23 @@ void pr_data_abort(int err, int quiet) {
 #endif
     }
 
-    if ( msg == NULL && (msg = strerror(err)) == NULL ) {
-      if ( snprintf(msgbuf, sizeof msgbuf,
-		    "Unknown or out of range errno [%d].",
-		    err) > 0 )
+    if (msg == NULL &&
+        (msg = strerror(err)) == NULL ) {
+
+      if (snprintf(msgbuf, sizeof(msgbuf),
+          "Unknown or out of range errno [%d]", err) > 0)
 	msg = msgbuf;
     }
 
-    pr_response_add_err(respcode, fmt ? fmt : "Transfer aborted. %s",
-      msg ? msg : "");
+    pr_log_pri(PR_LOG_NOTICE, "notice: user %s: aborting transfer: %s",
+      session.user, msg);
+
+    /* If we are aborting, then a 426 response has already been sent,
+     * and we don't want to add another to the error queue.
+     */
+    if (!true_abort)
+      pr_response_add_err(respcode, fmt ? fmt : "Transfer aborted. %s",
+        msg ? msg : "");
   }
 
   if (true_abort)
@@ -781,64 +804,71 @@ int pr_data_xfer(char *cl_buf, int cl_size) {
 
     if (session.d) {
       if (session.sf_flags & (SF_ASCII|SF_ASCII_OVERRIDE)) {
-        int adjlen,buflen;
+        int adjlen, buflen;
+
 	do {
 	  buflen = session.xfer.buflen;        /* how much remains in buf */
 	  adjlen = 0;
-	
-	  if ((len = pr_netio_read(session.d->instrm, buf + buflen,
-		  session.xfer.bufsize - buflen, 1)) > 0) {
+
+          len = pr_netio_read(session.d->instrm, buf + buflen,
+            session.xfer.bufsize - buflen, 1);
+          if (len < 0)
+            return -1;
+
+          if (len > 0) {
 	    buflen += len;
 
 	    if (TimeoutStalled)
-	      reset_timer(TIMER_STALLED, ANY_MODULE);
+	      pr_timer_reset(TIMER_STALLED, ANY_MODULE);
 	  }
 
-	  /* if buflen > 0, data remains in the buffer to be copied. */
-	  if (len >= 0 && buflen > 0) {
+	  /* If buflen > 0, data remains in the buffer to be copied. */
+	  if (len >= 0 &&
+              buflen > 0) {
 
 	    /* Perform translation:
+             *
 	     * buflen is returned as the modified buffer length after
 	     *        translation
 	     * adjlen is returned as the number of characters unprocessed in
 	     *        the buffer (to be dealt with later)
 	     *
-	     * We skip the call to xlate_ascii_read() in one case:
+	     * We skip the call to xfrm_ascii_read() in one case:
 	     * when we have one character in the buffer and have reached
-	     * end of data, this is so that xlate_ascii_read() won't sit
+	     * end of data, this is so that xfrm_ascii_read() won't sit
 	     * forever waiting for the next character after a final '\r'.
 	     */
 	    if (len > 0 || buflen > 1)
-	      xlate_ascii_read(buf, &buflen, &adjlen);
+	      xfrm_ascii_read(buf, &buflen, &adjlen);
 	
-	    /* now copy everything we can into cl_buf */
+	    /* Now copy everything we can into cl_buf */
 	    if (buflen > cl_size) {
-	      /* because we have to cut our buffer short, make sure this
+	      /* Because we have to cut our buffer short, make sure this
 	       * is made up for later by increasing adjlen.
 	       */
 	      adjlen += (buflen - cl_size);
 	      buflen = cl_size;
 	    }
-  	    memcpy(cl_buf,buf,buflen);
+  	    memcpy(cl_buf, buf, buflen);
 	
-	    /* copy whatever remains at the end of session.xfer.buf to the
-	     * head of the buffer and adjust buf accordingly
+	    /* Copy whatever remains at the end of session.xfer.buf to the
+	     * head of the buffer and adjust buf accordingly.
 	     *
 	     * adjlen is now the total bytes still waiting in buf, if
-	     * anything remains, copy it to the start of the buffer
+	     * anything remains, copy it to the start of the buffer.
 	     */
 	
 	    if (adjlen > 0)
-	      memcpy(buf,buf+buflen,adjlen);
+	      memcpy(buf, buf+buflen, adjlen);
 
-	    /* store everything back in session.xfer */
+	    /* Store everything back in session.xfer. */
 	    session.xfer.buflen = adjlen;
 	    total += buflen;
 	  }
 	
 	  /* Restart if data was returned by pr_netio_read() (len > 0) but
 	   * no data was copied to the client buffer (buflen = 0).
-	   * This indicates that xlate_ascii_read() needs more data
+	   * This indicates that xfrm_ascii_read() needs more data
 	   * in order to translate, so we need to call pr_netio_read() again.
            */
 	} while (len > 0 && buflen == 0);
@@ -850,9 +880,9 @@ int pr_data_xfer(char *cl_buf, int cl_size) {
       } else if ((len = pr_netio_read(session.d->instrm, cl_buf,
           cl_size, 1)) > 0) {
 
-        /* Non-ascii mode doesn't need to use session.xfer.buf */
+        /* Non-ASCII mode doesn't need to use session.xfer.buf */
         if (TimeoutStalled)
-          reset_timer(TIMER_STALLED, ANY_MODULE);
+          pr_timer_reset(TIMER_STALLED, ANY_MODULE);
 
         total += len;
       }
@@ -864,8 +894,8 @@ int pr_data_xfer(char *cl_buf, int cl_size) {
       int buflen = cl_size;
       unsigned int xferbuflen;
 
-      if (buflen > PR_TUNABLE_BUFFER_SIZE)
-        buflen = PR_TUNABLE_BUFFER_SIZE;
+      if (buflen > PR_TUNABLE_XFER_BUFFER_SIZE)
+        buflen = PR_TUNABLE_XFER_BUFFER_SIZE;
 
       xferbuflen = buflen;
 
@@ -879,15 +909,15 @@ int pr_data_xfer(char *cl_buf, int cl_size) {
          * will be adjusted so that it contains the length of data in
          * the internal buffer, including any added CRs.
          */
-        xlate_ascii_write(&session.xfer.buf, &xferbuflen, session.xfer.bufsize);
+        xfrm_ascii_write(&session.xfer.buf, &xferbuflen, session.xfer.bufsize);
       }
 
       if (pr_netio_write(session.d->outstrm, session.xfer.buf,
-          xferbuflen) == -1)
+          xferbuflen) < 0)
         return -1;
 
       if (TimeoutStalled)
-        reset_timer(TIMER_STALLED, ANY_MODULE);
+        pr_timer_reset(TIMER_STALLED, ANY_MODULE);
 
       cl_size -= buflen;
       cl_buf += buflen;
@@ -897,8 +927,9 @@ int pr_data_xfer(char *cl_buf, int cl_size) {
     len = total;
   }
 
-  if (total && TimeoutIdle)
-    reset_timer(TIMER_IDLE, ANY_MODULE);
+  if (total &&
+      TimeoutIdle)
+    pr_timer_reset(TIMER_IDLE, ANY_MODULE);
 
   session.xfer.total_bytes += total;
   session.total_bytes += total;
@@ -910,7 +941,7 @@ int pr_data_xfer(char *cl_buf, int cl_size) {
  * ASCII translation is not performed.
  * return 0 if reading and data connection closes, or -1 if error
  */
-pr_sendfile_t pr_data_sendfile(int retr_fd, off_t *offset, size_t count) {
+pr_sendfile_t pr_data_sendfile(int retr_fd, off_t *offset, off_t count) {
   int flags, error;
   pr_sendfile_t len = 0, total = 0;
 #if defined(HAVE_AIX_SENDFILE)
@@ -921,10 +952,11 @@ pr_sendfile_t pr_data_sendfile(int retr_fd, off_t *offset, size_t count) {
   if (session.xfer.direction == PR_NETIO_IO_RD)
     return -1;
 
-  if ((flags = fcntl(PR_NETIO_FD(session.d->outstrm), F_GETFL)) == -1)
+  flags = fcntl(PR_NETIO_FD(session.d->outstrm), F_GETFL);
+  if (flags == -1)
     return -1;
 
-  /* set fd to blocking-mode for sendfile() */
+  /* Set fd to blocking-mode for sendfile() */
   if (flags & O_NONBLOCK)
     if (fcntl(PR_NETIO_FD(session.d->outstrm), F_SETFL, flags^O_NONBLOCK) == -1)
       return -1;
@@ -935,14 +967,41 @@ pr_sendfile_t pr_data_sendfile(int retr_fd, off_t *offset, size_t count) {
 
     /* Linux semantics are fairly straightforward in a glibc 2.x world:
      *
-     * #include <sys/sendfile.h>
+     *   #include <sys/sendfile.h>
      *
-     * ssize_t sendfile(int out_fd, int in_fd, off_t *offset, size_t count)
+     *   ssize_t sendfile(int out_fd, int in_fd, off_t *offset, size_t count)
+     *
+     * Unfortunately, this API does not allow for an off_t number of bytes
+     * to be sent, only a size_t.  This means we need to make sure that
+     * the given count does not exceed the maximum value for a size_t.  Even
+     * worse, the return value is a ssize_t, not a size_t, which means
+     * the maximum value used can only be of the maximum _signed_ value,
+     * not the maximum unsigned value.  This means calling sendfile() more
+     * times.  How annoying.
      */
+
+#if defined(HAVE_LINUX_SENDFILE)
+    if (count > INT_MAX)
+      count = INT_MAX;
+
+#elif defined(HAVE_SOLARIS_SENDFILE)
+# if SIZEOF_SIZE_T == SIZEOF_INT
+    if (count > INT_MAX)
+      count = INT_MAX;
+# elif SIZEOF_SIZE_T == SIZEOF_LONG
+    if (count > LONG_MAX)
+      count = LONG_MAX;
+# elif SIZEOF_SIZE_T == SIZEOF_LONG_LONG
+    if (count > LLONG_MAX)
+      count = LLONG_MAX;
+# endif
+#endif /* !HAVE_SOLARIS_SENDFILE */
+
     len = sendfile(PR_NETIO_FD(session.d->outstrm), retr_fd, offset, count);
 
-    if (len != -1 && len < count) {
-      /* under linux semantics, this occurs when a signal has interrupted
+    if (len != -1 &&
+        len < count) {
+      /* Under Linux semantics, this occurs when a signal has interrupted
        * sendfile().
        */
 
@@ -951,10 +1010,10 @@ pr_sendfile_t pr_data_sendfile(int retr_fd, off_t *offset, size_t count) {
       /* Only reset the timers if data have actually been written out. */
       if (len > 0) {
         if (TimeoutStalled)
-          reset_timer(TIMER_STALLED, ANY_MODULE);
+          pr_timer_reset(TIMER_STALLED, ANY_MODULE);
 
         if (TimeoutIdle)
-          reset_timer(TIMER_IDLE, ANY_MODULE);
+          pr_timer_reset(TIMER_IDLE, ANY_MODULE);
       }
 
       session.xfer.total_bytes += len;
@@ -975,13 +1034,30 @@ pr_sendfile_t pr_data_sendfile(int retr_fd, off_t *offset, size_t count) {
     /* BSD semantics for sendfile are flexible...it'd be nice if we could
      * standardize on something like it.  The semantics are:
      *
-     * #include <sys/types.h>
-     * #include <sys/socket.h>
-     * #include <sys/uio.h>
+     *   #include <sys/types.h>
+     *   #include <sys/socket.h>
+     *   #include <sys/uio.h>
      *
-     * int sendfile(int in_fd, int out_fd, off_t offset, size_t count,
-     *              struct sf_hdtr *hdtr, off_t *len, int flags)
+     *   int sendfile(int in_fd, int out_fd, off_t offset, size_t count,
+     *                struct sf_hdtr *hdtr, off_t *len, int flags)
+     *
+     *  The comments above, about size_t versus off_t, apply here as
+     *  well.  Except that BSD's sendfile() uses an off_t * for returning
+     *  the number of bytes sent, so we can use the maximum unsigned
+     *  value.
      */
+
+#if SIZEOF_SIZE_T == SIZEOF_INT
+    if (count > UINT_MAX)
+      count = UINT_MAX;
+#elif SIZEOF_SIZE_T == SIZEOF_LONG
+    if (count > ULONG_MAX)
+      count = ULONG_MAX;
+#elif SIZEOF_SIZE_T == SIZEOF_LONG_LONG
+    if (count > ULLONG_MAX)
+      count = ULLONG_MAX;
+#endif
+
     if (sendfile(retr_fd, PR_NETIO_FD(session.d->outstrm), *offset, count,
         NULL, &len, 0) == -1) {
 
@@ -1013,23 +1089,25 @@ pr_sendfile_t pr_data_sendfile(int retr_fd, off_t *offset, size_t count) {
        * -1 being returned and EINTR being set, what ACTUALLY happens is
        * that errno is cleared and the number of bytes written is returned.
        *
-       * For obvious reasons, HP/UX sendfile is not supported yet - jss
+       * For obvious reasons, HP/UX sendfile is not supported yet.
        */
       if (errno == EINTR) {
         pr_signals_handle();
 
-	/* If we got everything in this transaction, we're done.
-	 */
-	if ((count -= len) <= 0)
-	  break;
-	
+        /* If we got everything in this transaction, we're done. */
+        if (len >= count)
+          break;
+
+        else 
+          count -= len;
+
 	*offset += len;
 	
 	if (TimeoutStalled)
-	  reset_timer(TIMER_STALLED, ANY_MODULE);
+	  pr_timer_reset(TIMER_STALLED, ANY_MODULE);
 	
 	if (TimeoutIdle)
-	  reset_timer(TIMER_IDLE, ANY_MODULE);
+	  pr_timer_reset(TIMER_IDLE, ANY_MODULE);
 	
 	session.xfer.total_bytes += len;
 	session.total_bytes += len;
@@ -1052,10 +1130,10 @@ pr_sendfile_t pr_data_sendfile(int retr_fd, off_t *offset, size_t count) {
     fcntl(PR_NETIO_FD(session.d->outstrm), F_SETFL, flags);
 
   if (TimeoutStalled)
-    reset_timer(TIMER_STALLED, ANY_MODULE);
+    pr_timer_reset(TIMER_STALLED, ANY_MODULE);
 
   if (TimeoutIdle)
-    reset_timer(TIMER_IDLE, ANY_MODULE);
+    pr_timer_reset(TIMER_IDLE, ANY_MODULE);
 
   session.xfer.total_bytes += len;
   session.total_bytes += len;
