@@ -2,7 +2,7 @@
  * ProFTPD - FTP server daemon
  * Copyright (c) 1997, 1998 Public Flood Software
  * Copyright (c) 1999, 2000 MacGyver aka Habeeb J. Dihu <macgyver@tos.net>
- * Copyright (c) 2001, 2002, 2003 The ProFTPD Project team
+ * Copyright (c) 2001-2005 The ProFTPD Project team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@
  */
 
 /* Inet support functions, many wrappers for netdb functions
- * $Id: inet.c,v 1.86 2004/04/22 01:32:22 castaglia Exp $
+ * $Id: inet.c,v 1.94 2005/11/29 02:42:19 castaglia Exp $
  */
 
 #include "conf.h"
@@ -37,9 +37,9 @@ extern server_rec *main_server;
 static pool *inet_pool = NULL;
 
 static int ip_proto = IPPROTO_IP;
-#ifdef USE_IPV6
+#ifdef PR_USE_IPV6
 static int ipv6_proto = IPPROTO_IPV6;
-#endif /* USE_IPV6 */
+#endif /* PR_USE_IPV6 */
 static int tcp_proto = IPPROTO_TCP;
 
 static int inet_errno = 0;		/* Holds errno */
@@ -69,8 +69,8 @@ int pr_inet_set_default_family(pool *p, int family) {
   return old_family;
 }
 
-/* Find a service and return it's port number. */
-int pr_inet_getservport(pool *p, char *serv, char *proto) {
+/* Find a service and return its port number. */
+int pr_inet_getservport(pool *p, const char *serv, const char *proto) {
   struct servent *servent = getservbyname(serv, proto);
 
   /* getservbyname returns the port in network byte order. */
@@ -89,9 +89,9 @@ char *pr_inet_validate(char *buf) {
 
     /* Per RFC requirements, these are all that are valid from a DNS. */
     if (!isalnum((int) *p) && *p != '.' && *p != '-'
-#ifdef USE_IPV6
+#ifdef PR_USE_IPV6
         && *p != ':'
-#endif /* USE_IPV6 */
+#endif /* PR_USE_IPV6 */
         ) {
 
       /* We set it to _ because we know that's an invalid, yet safe, option
@@ -208,11 +208,11 @@ static conn_t *inet_initialize_connection(pool *p, xaset_t *servers, int fd,
     /* If no default family has been set, then default to IPv6 (if IPv6
      * support is enabled), otherwise use IPv4.
      */
-#ifdef USE_IPV6
+#ifdef PR_USE_IPV6
     addr_family = AF_INET6;
 #else
     addr_family = AF_INET;
-#endif /* USE_IPV6 */
+#endif /* PR_USE_IPV6 */
   }
 
   /* If fd == -1, there is no currently open socket, so create one.
@@ -234,6 +234,7 @@ static conn_t *inet_initialize_connection(pool *p, xaset_t *servers, int fd,
 #if defined(SOLARIS2) || defined(FREEBSD2) || defined(FREEBSD3) || \
     defined(FREEBSD4) || defined(FREEBSD5) || defined(__OpenBSD__) || \
     defined(__NetBSD__) || defined(DARWIN6) || defined(DARWIN7) || \
+    defined(DARWIN8) || \
     defined(SCO3) || defined(CYGWIN) || defined(SYSV4_2MP) || \
     defined(SYSV5UNIXWARE7)
 # ifdef SOLARIS2
@@ -251,6 +252,7 @@ static conn_t *inet_initialize_connection(pool *p, xaset_t *servers, int fd,
 #if defined(SOLARIS2) || defined(FREEBSD2) || defined(FREEBSD3) || \
     defined(FREEBSD4) || defined(FREEBSD5) || defined(__OpenBSD__) || \
     defined(__NetBSD__) || defined(DARWIN6) || defined(DARWIN7) || \
+    defined(DARWIN8) || \
     defined(SCO3) || defined(CYGWIN) || defined(SYSV4_2MP) || \
     defined(SYSV5UNIXWARE7)
 # ifdef SOLARIS2
@@ -286,34 +288,33 @@ static conn_t *inet_initialize_connection(pool *p, xaset_t *servers, int fd,
     if (bind_addr)
       pr_netaddr_set_sockaddr(&na, pr_netaddr_get_sockaddr(bind_addr));
 
-    else {
+    else
       pr_netaddr_set_sockaddr_any(&na);
 
-#if defined(USE_IPV6) && defined(IPV6_V6ONLY)
-      if (addr_family == AF_INET6) {
-        int off = 0;
+#if defined(PR_USE_IPV6) && defined(IPV6_V6ONLY)
+    if (addr_family == AF_INET6) {
+      int on = 0;
 
 # ifdef SOL_IP
-        int level = SOL_IP;
+      int level = SOL_IP;
 # else
-        int level = ipv6_proto;
+      int level = ipv6_proto;
 # endif /* SOL_IP */
 
-      	/* If creating a wildcard socket IPv6 socket, make sure that it
-         * will accept IPv4 connections as well.  This is the default on
-         * Linux and Solaris; BSD usually defaults to allowing only IPv6
-         * (depending on the net.inet6.ip6.v6only sysctl value).
-         *
-         * Ideally, this setsockopt() call would be configurable via the
-         * SocketOptions directive.
-         */
+      /* If creating a wildcard socket IPv6 socket, make sure that it
+       * will accept IPv4 connections as well.  This is the default on
+       * Linux and Solaris; BSD usually defaults to allowing only IPv6
+       * (depending on the net.inet6.ip6.v6only sysctl value).
+       *
+       * Ideally, this setsockopt() call would be configurable via the
+       * SocketOptions directive.
+       */
 
-        if (setsockopt(fd, level, IPV6_V6ONLY, (void *) &off, sizeof(off)) < 0)
-          pr_log_pri(PR_LOG_NOTICE, "error setting IPV6_V6ONLY: %s",
-            strerror(errno));
-      }
-#endif /* USE_IPV6 and IPV6_V6ONLY */
+      if (setsockopt(fd, level, IPV6_V6ONLY, (void *) &on, sizeof(on)) < 0)
+        pr_log_pri(PR_LOG_NOTICE, "error setting IPV6_V6ONLY: %s",
+          strerror(errno));
     }
+#endif /* PR_USE_IPV6 and IPV6_V6ONLY */
 
     pr_netaddr_set_port(&na, htons(port));
 
@@ -349,7 +350,7 @@ static conn_t *inet_initialize_connection(pool *p, xaset_t *servers, int fd,
         pr_signals_unblock();
       }
 
-      timer_sleep(1);
+      pr_timer_sleep(1);
 
       if (port != INPORT_ANY && port < 1024) {
         pr_signals_block();
@@ -513,6 +514,29 @@ void pr_inet_lingering_close(pool *p, conn_t *c, long linger) {
   destroy_pool(c->pool);
 }
 
+/* Similar to a lingering close, perform a lingering abort. */
+void pr_inet_lingering_abort(pool *p, conn_t *c, long linger) {
+  pr_inet_set_block(p, c);
+
+  if (c->instrm)
+    pr_netio_lingering_abort(c->instrm, linger);
+
+  /* Only close the output stream if it is actually a different stream
+   * than the input stream.
+   *
+   * Note: we do not call pr_netio_lingering_abort() on the input stream
+   * since doing so would result in two 426 responses sent; we only
+   * want and need one.
+   */
+  if (c->outstrm != c->instrm)
+    pr_netio_lingering_close(c->outstrm, linger);
+
+  c->instrm = NULL;
+  c->outstrm = NULL;
+
+  destroy_pool(c->pool);
+}
+
 int pr_inet_set_proto_opts(pool *p, conn_t *c, int mss, int nodelay,
     int lowdelay, int throughput, int nopush) {
   int tos = 0;
@@ -641,7 +665,7 @@ int pr_inet_set_socket_opts(pool *p, conn_t *c, int rcvbuf, int sndbuf) {
     if (sndbuf && sndbuf > csndbuf)
       if (setsockopt(c->wfd, SOL_SOCKET, SO_SNDBUF, (void *) &sndbuf,
           sizeof(sndbuf)) < 0)
-        pr_log_pri(PR_LOG_NOTICE, "error setting SO_SNBUF: %s",
+        pr_log_pri(PR_LOG_NOTICE, "error setting SO_SNDBUF: %s",
           strerror(errno));
 
     c->sndbuf = (sndbuf ? sndbuf : csndbuf);
@@ -769,8 +793,8 @@ int pr_inet_listen(pool *p, conn_t *c, int backlog) {
         continue;
       }
 
-      pr_log_pri(PR_LOG_ERR, "listen() failed in inet_listen(): %s",
-        strerror(errno));
+      pr_log_pri(PR_LOG_ERR, "unable to listen on %s#%u: %s",
+        pr_netaddr_get_ipstr(c->local_addr), c->local_port, strerror(errno));
       end_login(1);
 
     } else
@@ -994,11 +1018,11 @@ int pr_inet_get_conn_info(conn_t *c, int fd) {
   /* Initialize the netaddr. */
   pr_netaddr_clear(&na);
 
-#ifdef USE_IPV6
+#ifdef PR_USE_IPV6
   pr_netaddr_set_family(&na, AF_INET6);
 #else
   pr_netaddr_set_family(&na, AF_INET);
-#endif /* USE_IPV6 */
+#endif /* PR_USE_IPV6 */
   nalen = pr_netaddr_get_sockaddr_len(&na);
 
   if (getsockname(fd, pr_netaddr_get_sockaddr(&na), &nalen) != -1) {
@@ -1019,11 +1043,11 @@ int pr_inet_get_conn_info(conn_t *c, int fd) {
     return -1;
 
   /* "Reset" the pr_netaddr_t struct for the getpeername(2) call. */
-#ifdef USE_IPV6
+#ifdef PR_USE_IPV6
   pr_netaddr_set_family(&na, AF_INET6);
 #else
   pr_netaddr_set_family(&na, AF_INET);
-#endif /* USE_IPV6 */
+#endif /* PR_USE_IPV6 */
   nalen = pr_netaddr_get_sockaddr_len(&na);
 
   if (getpeername(fd, pr_netaddr_get_sockaddr(&na), &nalen) != -1) {
@@ -1209,11 +1233,11 @@ void init_inet(void) {
   if (pr != NULL)
     ip_proto = pr->p_proto;
 
-#ifdef USE_IPV6
+#ifdef PR_USE_IPV6
   pr = getprotobyname("ipv6"); 
   if (pr != NULL)
     ipv6_proto = pr->p_proto;
-#endif /* USE_IPV6 */
+#endif /* PR_USE_IPV6 */
 
   pr = getprotobyname("tcp");
   if (pr != NULL)
