@@ -2,7 +2,7 @@
  * ProFTPD - FTP server daemon
  * Copyright (c) 1997, 1998 Public Flood Software
  * Copyright (c) 1999, 2000 MacGyver aka Habeeb J. Dihu <macgyver@tos.net>
- * Copyright (c) 2001, 2002, 2003 The ProFTPD Project team
+ * Copyright (c) 2001-2005 The ProFTPD Project team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@
  */
 
 /* ProFTPD logging support.
- * $Id: log.c,v 1.61 2003/10/08 18:13:03 castaglia Exp $
+ * $Id: log.c,v 1.76 2005/11/14 16:33:06 castaglia Exp $
  */
 
 #include "conf.h"
@@ -44,85 +44,8 @@ static char systemlog_fn[PR_TUNABLE_PATH_MAX] = {'\0'};
 static char systemlog_host[256] = {'\0'};
 static int systemlog_fd = -1;
 
-static int xfer_fd = -1;
-
 int syslog_sockfd = -1;
 
-char *fmt_time(time_t t) {
-  static char buf[30];
-  static char *mons[] =
-  { "Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec" };
-  static char *days[] =
-  { "Sun","Mon","Tue","Wed","Thu","Fri","Sat" };
-  struct tm *tr;
-
-  memset(buf, '\0', sizeof(buf));
-  if ((tr = localtime(&t)) != NULL) {
-    snprintf(buf,sizeof(buf), "%s %s %2d %02d:%02d:%02d %d",
-            days[tr->tm_wday],
-            mons[tr->tm_mon],
-            tr->tm_mday,
-            tr->tm_hour,
-            tr->tm_min,
-            tr->tm_sec,
-            tr->tm_year + 1900);
-  } else
-    buf[0] = '\0';
-  buf[sizeof(buf)-1] = '\0';
-
-  return buf;
-}
-
-void log_close_xfer(void) {
-  if (xfer_fd != -1)
-    close(xfer_fd);
-
-  xfer_fd = -1;
-}
-
-int log_open_xfer(const char *path) {
-
-  if (!path) {
-    if (xfer_fd != -1)
-      log_close_xfer();
-    return 0;
-  }
-
-  if (xfer_fd == -1) {
-    log_debug(DEBUG6, "opening TransferLog '%s'", path);
-    log_openfile(path, &xfer_fd, LOG_XFER_MODE);
-  }
-
-  return xfer_fd;
-}
-
-int log_xfer(long xfertime, const char *remhost, off_t fsize, char *fname,
-    char xfertype, char direction, char access_mode, char *user,
-    char abort_flag) {
-
-  char buf[LOGBUFFER_SIZE] = {'\0'}, fbuf[LOGBUFFER_SIZE] = {'\0'};
-  register unsigned int i = 0;
-
-  if (xfer_fd == -1 || !remhost || !user || !fname)
-    return 0;
-
-  for (i = 0; (i + 1 < sizeof(fbuf)) && fname[i] != '\0'; i++) {
-    fbuf[i] = (isspace((int) fname[i]) || iscntrl((int) fname[i])) ? '_' :
-      fname[i];
-  }
-  fbuf[i] = '\0';
-
-  snprintf(buf, sizeof(buf),
-    "%s %ld %s %" PR_LU " %s %c _ %c %c %s ftp %c %s %c\n",
-    fmt_time(time(NULL)), xfertime, remhost, fsize, fbuf, xfertype, direction,
-    access_mode, user, session.ident_lookups == TRUE ? '1' : '0',
-    (session.ident_lookups == TRUE && strcmp(session.ident_user,
-      "UNKNOWN")) ? session.ident_user : "*", abort_flag);
-  buf[sizeof(buf)-1] = '\0';
-
-  return write(xfer_fd, buf, strlen(buf));
-}
-
 /* This next function logs an entry to wtmp, it MUST be called as
  * root BEFORE a chroot occurs.
  * Note: This has some portability ifdefs in it.  They *should* work,
@@ -136,7 +59,8 @@ int log_wtmp(char *line, const char *name, const char *host,
   int res = 0;
   static int fd = -1;
 
-#if (defined(SVR4) || defined(__SVR4)) && \
+#if ((defined(SVR4) || defined(__SVR4)) || \
+    (defined(__NetBSD__) && defined(HAVE_UTMPX_H))) && \
     !(defined(LINUX) || defined(__hpux) || defined (_AIX))
   /* This "auxilliary" utmp doesn't exist under linux. */
 #ifdef __sparcv9
@@ -147,8 +71,13 @@ int log_wtmp(char *line, const char *name, const char *host,
 #endif
   static int fdx = -1;
 
-  if (fdx < 0 && (fdx = open(WTMPX_FILE, O_WRONLY|O_APPEND, 0)) < 0) {
-    log_pri(PR_LOG_WARNING, "wtmpx %s: %s", WTMPX_FILE, strerror(errno));
+#if !defined(WTMPX_FILE) && defined(_PATH_WTMPX)
+# define WTMPX_FILE _PATH_WTMPX
+#endif
+
+  if (fdx < 0 &&
+      (fdx = open(WTMPX_FILE, O_WRONLY|O_APPEND, 0)) < 0) {
+    pr_log_pri(PR_LOG_WARNING, "wtmpx %s: %s", WTMPX_FILE, strerror(errno));
     return -1;
   }
 
@@ -158,20 +87,27 @@ int log_wtmp(char *line, const char *name, const char *host,
    * Insane if you ask me.  Unless there's massive uproar, I prefer to err on
    * the side of caution and always null-terminate our strings.
    */
-  if (fstat(fdx,&buf) == 0) {
-    memset(&utx,0,sizeof(utx));
-    sstrncpy(utx.ut_user,name,sizeof(utx.ut_user));
-    sstrncpy(utx.ut_id, "ftp",sizeof(utx.ut_user));
-    sstrncpy(utx.ut_line,line,sizeof(utx.ut_line));
-    sstrncpy(utx.ut_host,host,sizeof(utx.ut_host));
-    utx.ut_syslen = strlen(utx.ut_host)+1;
+  if (fstat(fdx, &buf) == 0) {
+    memset(&utx, 0, sizeof(utx));
+
+    sstrncpy(utx.ut_user, name, sizeof(utx.ut_user));
+    sstrncpy(utx.ut_id, "ftp", sizeof(utx.ut_user));
+    sstrncpy(utx.ut_line, line, sizeof(utx.ut_line));
+    sstrncpy(utx.ut_host, host, sizeof(utx.ut_host));
     utx.ut_pid = getpid();
-#ifdef __sparcv9
+#if defined(__NetBSD__) && defined(HAVE_UTMPX_H)
+    memcpy(&utx.ut_ss, pr_netaddr_get_inaddr(ip), sizeof(utx.ut_ss));
+    gettimeofday(&utx.ut_tv, NULL);
+#else /* SVR4 */
+    utx.ut_syslen = strlen(utx.ut_host)+1;
+#  ifdef __sparcv9
     time(&t);
     utx.ut_tv.tv_sec = (time32_t)t;
-#else
+#  else
     time(&utx.ut_tv.tv_sec);
-#endif
+#  endif
+#endif /* SVR4 */
+
     if (*name)
       utx.ut_type = USER_PROCESS;
     else
@@ -180,57 +116,64 @@ int log_wtmp(char *line, const char *name, const char *host,
     utx.ut_exit.e_termination = 0;
     utx.ut_exit.e_exit = 0;
 #endif /* HAVE_UT_UT_EXIT */
-    if (write(fdx, (char *)&utx,sizeof(utx)) != sizeof(utx))
+    if (write(fdx, (char *)&utx, sizeof(utx)) != sizeof(utx))
       ftruncate(fdx, buf.st_size);
+
   } else {
-    log_debug(DEBUG0, "%s fstat(): %s",WTMPX_FILE,strerror(errno));
+    pr_log_debug(DEBUG0, "%s fstat(): %s", WTMPX_FILE, strerror(errno));
     res = -1;
   }
 
 #else /* Non-SVR4 systems */
 
-  if (fd < 0 && (fd = open(WTMP_FILE,O_WRONLY|O_APPEND,0)) < 0) {
-    log_pri(PR_LOG_WARNING, "wtmp %s: %s", WTMP_FILE, strerror(errno));
+  if (fd < 0 &&
+      (fd = open(WTMP_FILE, O_WRONLY|O_APPEND, 0)) < 0) {
+    pr_log_pri(PR_LOG_WARNING, "wtmp %s: %s", WTMP_FILE, strerror(errno));
     return -1;
   }
 
-  if (fstat(fd,&buf) == 0) {
-    memset(&ut,0,sizeof(ut));
+  if (fstat(fd, &buf) == 0) {
+    memset(&ut, 0, sizeof(ut));
 #ifdef HAVE_UTMAXTYPE
-#ifdef LINUX
+# ifdef LINUX
     if (ip)
-      memcpy(&ut.ut_addr,ip,sizeof(ut.ut_addr));
-#else
-    sstrncpy(ut.ut_id, "ftp",sizeof(ut.ut_id));
-#ifdef HAVE_UT_UT_EXIT
+#  ifndef PR_USE_IPV6
+      memcpy(&ut.ut_addr, pr_netaddr_get_inaddr(ip), sizeof(ut.ut_addr));
+#  else
+      memcpy(&ut.ut_addr_v6, pr_netaddr_get_inaddr(ip), sizeof(ut.ut_addr_v6));
+#  endif /* !PR_USE_IPV6 */
+# else
+    sstrncpy(ut.ut_id, "ftp", sizeof(ut.ut_id));
+#  ifdef HAVE_UT_UT_EXIT
     ut.ut_exit.e_termination = 0;
     ut.ut_exit.e_exit = 0;
-#endif /* HAVE_UT_UT_EXIT */
-#endif
-    sstrncpy(ut.ut_line,line,sizeof(ut.ut_line));
+#  endif /* !HAVE_UT_UT_EXIT */
+# endif /* !LINUX */
+    sstrncpy(ut.ut_line, line, sizeof(ut.ut_line));
     if (name && *name)
-      sstrncpy(ut.ut_user,name,sizeof(ut.ut_user));
+      sstrncpy(ut.ut_user, name, sizeof(ut.ut_user));
     ut.ut_pid = getpid();
     if (name && *name)
       ut.ut_type = USER_PROCESS;
     else
       ut.ut_type = DEAD_PROCESS;
 #else  /* !HAVE_UTMAXTYPE */
-    sstrncpy(ut.ut_line,line,sizeof(ut.ut_line));
+    sstrncpy(ut.ut_line, line, sizeof(ut.ut_line));
     if (name && *name)
-      sstrncpy(ut.ut_name,name,sizeof(ut.ut_name));
+      sstrncpy(ut.ut_name, name, sizeof(ut.ut_name));
 #endif /* HAVE_UTMAXTYPE */
 
 #ifdef HAVE_UT_UT_HOST
     if (host && *host)
-      sstrncpy(ut.ut_host,host,sizeof(ut.ut_host));
+      sstrncpy(ut.ut_host, host, sizeof(ut.ut_host));
 #endif /* HAVE_UT_UT_HOST */
 
     time(&ut.ut_time);
-    if (write(fd, (char *)&ut,sizeof(ut)) != sizeof(ut))
-      ftruncate(fd,buf.st_size);
+    if (write(fd, (char *)&ut, sizeof(ut)) != sizeof(ut))
+      ftruncate(fd, buf.st_size);
+
   } else {
-    log_debug(DEBUG0, "%s fstat(): %s",WTMP_FILE,strerror(errno));
+    pr_log_debug(DEBUG0, "%s fstat(): %s",WTMP_FILE,strerror(errno));
     res = -1;
   }
 #endif /* SVR4 */
@@ -238,7 +181,7 @@ int log_wtmp(char *line, const char *name, const char *host,
   return res;
 }
 
-int log_openfile(const char *log_file, int *log_fd, mode_t log_mode) {
+int pr_log_openfile(const char *log_file, int *log_fd, mode_t log_mode) {
   pool *tmp_pool = NULL;
   char *tmp = NULL, *lf;
   unsigned char have_stat = FALSE, *allow_log_symlinks = NULL;
@@ -252,11 +195,12 @@ int log_openfile(const char *log_file, int *log_fd, mode_t log_mode) {
 
   /* Make a temporary copy of log_file in case it's a constant */
   tmp_pool = make_sub_pool(permanent_pool);
+  pr_pool_tag(tmp_pool, "log_openfile() tmp pool");
   lf = pstrdup(tmp_pool, log_file);
 
   tmp = strrchr(lf, '/');
   if (tmp == NULL) {
-    log_debug(DEBUG0, "inappropriate log file: %s", lf);
+    pr_log_debug(DEBUG0, "inappropriate log file: %s", lf);
     destroy_pool(tmp_pool);
     return -1;
   }
@@ -267,7 +211,7 @@ int log_openfile(const char *log_file, int *log_fd, mode_t log_mode) {
   *tmp = '\0';
 
   if (stat(lf, &sbuf) == -1) {
-    log_debug(DEBUG0, "error: unable to stat() %s: %s", lf,
+    pr_log_debug(DEBUG0, "error: unable to stat() %s: %s", lf,
       strerror(errno));
     destroy_pool(tmp_pool);
     return -1;
@@ -275,16 +219,16 @@ int log_openfile(const char *log_file, int *log_fd, mode_t log_mode) {
 
   /* The path must be in a valid directory */
   if (!S_ISDIR(sbuf.st_mode)) {
-    log_debug(DEBUG0, "error: %s is not a directory", lf);
+    pr_log_debug(DEBUG0, "error: %s is not a directory", lf);
     destroy_pool(tmp_pool);
     return -1;
   }
 
   /* Do not log to world-writeable directories */
   if (sbuf.st_mode & S_IWOTH) {
-    log_pri(PR_LOG_NOTICE, "error: %s is a world writeable directory", lf);
+    pr_log_pri(PR_LOG_NOTICE, "error: %s is a world writeable directory", lf);
     destroy_pool(tmp_pool);
-    return LOG_WRITEABLE_DIR;
+    return PR_LOG_WRITABLE_DIR;
   }
 
   /* Restore the path separator so that checks on the file itself may be
@@ -332,12 +276,12 @@ int log_openfile(const char *log_file, int *log_fd, mode_t log_mode) {
         switch (errno) {
 #ifdef ELOOP
           case ELOOP:
-            return LOG_SYMLINK;
+            return PR_LOG_SYMLINK;
 #endif /* ELOOP */
 
 #ifdef EMLINK
           case EMLINK:
-            return LOG_SYMLINK;
+            return PR_LOG_SYMLINK;
 #endif /* EMLINK */
         }
 
@@ -381,13 +325,13 @@ int log_openfile(const char *log_file, int *log_fd, mode_t log_mode) {
       have_stat = TRUE;
 
     if (!have_stat || S_ISLNK(sbuf.st_mode)) {
-      log_debug(DEBUG0, !have_stat ? "error: unable to stat %s" :
+      pr_log_debug(DEBUG0, !have_stat ? "error: unable to stat %s" :
         "error: %s is a symbolic link", lf);
 
       close(*log_fd);
       *log_fd = -1;
       destroy_pool(tmp_pool);
-      return LOG_SYMLINK;
+      return PR_LOG_SYMLINK;
     }
 
   } else {
@@ -402,6 +346,42 @@ int log_openfile(const char *log_file, int *log_fd, mode_t log_mode) {
   return 0;
 }
 
+int pr_log_writefile(int logfd, const char *ident, const char *fmt, ...) {
+  char buf[PR_TUNABLE_BUFFER_SIZE] = {'\0'};
+  time_t timestamp = time(NULL);
+  struct tm *t = NULL;
+  va_list msg;
+
+  if (logfd < 0) {
+    errno = EINVAL;
+    return -1;
+  }
+
+  t = localtime(&timestamp);
+
+  /* Prepend the timestamp */
+  strftime(buf, sizeof(buf), "%b %d %H:%M:%S ", t);
+  buf[sizeof(buf)-1] = '\0';
+
+  /* Prepend a small header */
+  snprintf(buf + strlen(buf), sizeof(buf) - strlen(buf), "%s[%u]: ",
+    ident, (unsigned int) getpid());
+  buf[sizeof(buf)-1] = '\0';
+
+  /* Affix the message */
+  va_start(msg, fmt);
+  vsnprintf(buf + strlen(buf), sizeof(buf) - strlen(buf), fmt, msg);
+  va_end(msg);
+
+  buf[sizeof(buf)-1] = '\0';
+  buf[strlen(buf)] = '\n';
+
+  if (write(logfd, buf, strlen(buf)) < 0)
+    return -1;
+
+  return 0;
+}
+
 int log_opensyslog(const char *fn) {
   int res = 0;
 
@@ -423,8 +403,8 @@ int log_opensyslog(const char *fn) {
       return -1;
     systemlog_fd = -1;
 
-  } else if ((res = log_openfile(systemlog_fn, &systemlog_fd,
-      LOG_SYSTEM_MODE)) < 0) {
+  } else if ((res = pr_log_openfile(systemlog_fn, &systemlog_fd,
+      PR_LOG_SYSTEM_MODE)) < 0) {
     memset(systemlog_fn, '\0', sizeof(systemlog_fn));
     return res;
   }
@@ -460,20 +440,25 @@ static void log_write(int priority, int f, char *s) {
 
   memset(serverinfo, '\0', sizeof(serverinfo));
 
-  if (main_server && main_server->ServerFQDN) {
+  if (main_server &&
+      main_server->ServerFQDN) {
+    pr_netaddr_t *remote_addr = pr_netaddr_get_sess_remote_addr();
+    const char *remote_name = pr_netaddr_get_sess_remote_name();
+
     snprintf(serverinfo, sizeof(serverinfo), "%s", main_server->ServerFQDN);
     serverinfo[sizeof(serverinfo)-1] = '\0';
 
-    if (session.c && session.c->remote_name) {
+    if (remote_addr && remote_name) {
       snprintf(serverinfo + strlen(serverinfo),
         sizeof(serverinfo) - strlen(serverinfo), " (%s[%s])",
-        session.c->remote_name,
-        pr_netaddr_get_ipstr(session.c->remote_addr));
+        remote_name, pr_netaddr_get_ipstr(remote_addr));
+
       serverinfo[sizeof(serverinfo)-1] = '\0';
     }
   }
 
-  if (logstderr) {
+  if (logstderr ||
+      !main_server) {
     fprintf(stderr, "%s - %s\n", serverinfo, s);
     return;
   }
@@ -508,11 +493,16 @@ static void log_write(int priority, int f, char *s) {
   if (set_facility != -1)
     f = set_facility;
 
-  if (f != facility || !syslog_open)
+  if (!syslog_open) {
     syslog_sockfd = pr_openlog("proftpd", LOG_NDELAY|LOG_PID, f);
 
-  if ((max_priority = get_param_ptr(main_server->conf, "SyslogLevel",
-      FALSE)) != NULL && priority > *max_priority)
+  } else if (f != facility) {
+    (void) pr_setlogfacility(f);
+  }
+
+  max_priority = get_param_ptr(main_server->conf, "SyslogLevel", FALSE);
+  if (max_priority != NULL &&
+      priority > *max_priority)
     return;
 
   if (*serverinfo)
@@ -524,11 +514,12 @@ static void log_write(int priority, int f, char *s) {
     pr_closelog(syslog_sockfd);
     syslog_sockfd = -1;
 
-  } else if (f != facility)
-    syslog_sockfd = pr_openlog("proftpd", LOG_NDELAY|LOG_PID, facility);
+  } else if (f != facility) {
+    (void) pr_setlogfacility(f);
+  }
 }
 
-void log_pri(int priority, const char *fmt, ...) {
+void pr_log_pri(int priority, const char *fmt, ...) {
   char buf[LOGBUFFER_SIZE] = {'\0'};
   va_list msg;
 
@@ -542,11 +533,10 @@ void log_pri(int priority, const char *fmt, ...) {
   log_write(priority, facility, buf);
 }
 
-/* Like log_pri(), but sends the log entry in the LOG_AUTHPRIV
- * facility (presumable it doesn't need to be seen by everyone
+/* Like pr_log_pri(), but sends the log entry in the LOG_AUTHPRIV
+ * facility (presumably it doesn't need to be seen by everyone).
  */
-
-void log_auth(int priority, const char *fmt, ...) {
+void pr_log_auth(int priority, const char *fmt, ...) {
   char buf[LOGBUFFER_SIZE] = {'\0'};
   va_list msg;
 
@@ -572,7 +562,7 @@ void log_stderr(int bool) {
  * numbers mean print more, DEBUG0 (0) == print no debugging log
  * (default)
  */
-int log_setdebuglevel(int level) {
+int pr_log_setdebuglevel(int level) {
   int old_level = debug_level;
   debug_level = level;
   return old_level;
@@ -581,7 +571,7 @@ int log_setdebuglevel(int level) {
 /* Convert a string into the matching syslog level value.  Return -1
  * if no matching level is found.
  */
-int log_str2sysloglevel(const char *name) {
+int pr_log_str2sysloglevel(const char *name) {
 
   if (strcasecmp(name, "emerg") == 0)
     return PR_LOG_EMERG;
@@ -610,7 +600,7 @@ int log_str2sysloglevel(const char *name) {
   return -1;
 }
 
-void log_debug(int level, const char *fmt, ...) {
+void pr_log_debug(int level, const char *fmt, ...) {
   char buf[LOGBUFFER_SIZE] = {'\0'};
   va_list msg;
 
