@@ -23,7 +23,7 @@
  */
 
 /* Home-on-demand support
- * $Id: mkhome.c,v 1.6 2004/05/13 01:13:57 castaglia Exp $
+ * $Id: mkhome.c,v 1.8 2005/06/20 05:41:31 castaglia Exp $
  */
 
 #include "conf.h"
@@ -31,7 +31,7 @@
 
 static int create_dir(const char *dir, uid_t uid, gid_t gid,
     mode_t mode) {
-  mode_t prevmask;
+  mode_t prev_mask;
   struct stat st;
   int res = -1;
 
@@ -51,22 +51,23 @@ static int create_dir(const char *dir, uid_t uid, gid_t gid,
   }
 
   /* The given mode is absolute, not subject to any Umask setting. */
-  prevmask = umask(0);
+  prev_mask = umask(0);
 
   if (pr_fsio_mkdir(dir, mode) < 0) {
+    umask(prev_mask);
     pr_log_pri(PR_LOG_WARNING, "error creating '%s': %s", dir,
       strerror(errno));
     return -1;
   }
 
+  umask(prev_mask);
+
   if (pr_fsio_chown(dir, uid, gid) < 0) {
     pr_log_pri(PR_LOG_WARNING, "error setting ownership of '%s': %s", dir,
       strerror(errno));
     return -1;
   }
 
-  umask(prevmask);
-
   pr_log_debug(DEBUG6, "CreateHome: directory '%s' created", dir);
   return 0;
 }
@@ -86,6 +87,8 @@ static int create_path(pool *p, const char *path, const char *user, uid_t uid,
     return -1;
   }
 
+  pr_event_generate("core.create-home", user);
+
   pr_log_debug(DEBUG3, "creating home directory '%s' for user '%s'", path,
     user);
   tmppath = pstrdup(p, path);
@@ -253,7 +256,7 @@ int create_home(pool *p, const char *home, const char *user, uid_t uid,
     char *skel_dir = c->argv[3];
 
     /* Populate the home directory with files from the configured
-     * skeleton (a al /etc/skel) directory.
+     * skeleton (a la /etc/skel) directory.
      */
 
     pr_log_debug(DEBUG4, "CreateHome: copying skel files from '%s' into '%s'",
