@@ -25,7 +25,7 @@
 
 /*
  * Module handling routines
- * $Id: modules.c,v 1.30 2003/09/08 00:31:32 castaglia Exp $
+ * $Id: modules.c,v 1.49 2004/10/26 23:25:11 castaglia Exp $
  */
 
 #include "conf.h"
@@ -47,23 +47,19 @@ struct stash {
   } ptr;
 };
 
+extern module *static_modules[];
+extern module *loaded_modules;
+
 /* Symbol hashes for each type */
 static xaset_t *symbol_table[PR_TUNABLE_HASH_TABLE_SIZE];
 static pool *symbol_pool = NULL;
 static struct stash *curr_sym = NULL;
 
-static xaset_t *installed_modules = NULL;
-static array_header *mconfarr;			/* masterconf array */
-static array_header *mcmdarr;			/* mastercmd array */
-static array_header *mautharr;			/* masterauth array */
+/* Currently running module */
+module *curr_module = NULL;
 
-conftable *m_conftable; 			/* Master conf table */
-cmdtable *m_cmdtable;				/* Master cmd table */
-authtable *m_authtable;				/* Master auth table */
-
-module *curr_module = NULL;			/* Current running module */
-
-extern module *static_modules[];
+/* Used to track the priority for loaded modules. */
+static unsigned int curr_module_pri = 0;
 
 typedef struct mod_cb {
   struct mod_cb *next, *prev;
@@ -71,12 +67,6 @@ typedef struct mod_cb {
   int (*module_cb)(void);
 } module_cb_t;
 
-static pool *daemon_cb_pool = NULL;
-static xaset_t *daemon_startups = NULL;
-
-static pool *postparse_init_pool = NULL;
-static xaset_t *postparse_inits = NULL;
-
 /* Symbol stash lookup code and management */
 
 /* This wrapper will be used in the future to track when to rehash through
@@ -88,7 +78,7 @@ static struct stash *sym_alloc(void) {
   pool *sub_pool = make_sub_pool(symbol_pool);
   struct stash *sym = pcalloc(sub_pool, sizeof(struct stash));
   sym->sym_pool = sub_pool; 
-
+  pr_pool_tag(sub_pool, "symbol subpool");
   count++;
 
   return sym;
@@ -97,7 +87,7 @@ static struct stash *sym_alloc(void) {
 static int sym_cmp(struct stash *s1, struct stash *s2) {
   int ret;
 
-  ret = strcmp(s1->sym_name,s2->sym_name);
+  ret = strcmp(s1->sym_name, s2->sym_name);
 
   /* Higher priority modules must go BEFORE lower priority in the
    * hash tables.
@@ -117,6 +107,9 @@ static int symtab_hash(const char *name) {
   unsigned char *cp = NULL;
   int total = 0;
 
+  if (!name)
+    return 0;
+
   for (cp = (unsigned char *)name; *cp; cp++)
     total += (int)*cp;
 
@@ -171,7 +164,22 @@ int pr_stash_add_symbol(pr_stash_type_t sym_type, void *data) {
       return -1;
   }
 
-  idx = symtab_hash(sym->sym_name);
+  /* XXX Ugly hack to support mixed cases of directives in config files. */
+  if (sym_type != PR_SYM_CONF)
+    idx = symtab_hash(sym->sym_name);
+
+  else {
+    register unsigned int i;
+    char buf[1024];
+
+    memset(buf, '\0', sizeof(buf));
+    sstrncpy(buf, sym->sym_name, sizeof(buf)-1);
+
+    for (i = 0; i < strlen(buf); i++)
+      buf[i] = tolower((int) buf[i]);
+
+    idx = symtab_hash(buf);
+  }
 
   if (!symbol_table[idx])
     symbol_table[idx] = xaset_create(symbol_pool, (XASET_COMPARE) sym_cmp);
@@ -184,10 +192,11 @@ static struct stash *stash_lookup(pr_stash_type_t sym_type,
     const char *name, int idx) {
   struct stash *sym = NULL;
 
-  if (name && symbol_table[idx]) {
+  if (symbol_table[idx]) {
     for (sym = (struct stash *) symbol_table[idx]->xas_list; sym;
         sym = sym->next)
-      if (sym->sym_type == sym_type && !strcmp(sym->sym_name, name))
+      if (sym->sym_type == sym_type &&
+          (!name || strcasecmp(sym->sym_name, name) == 0))
         break;
   }
 
@@ -202,7 +211,8 @@ static struct stash *stash_lookup_next(pr_stash_type_t sym_type,
   if (symbol_table[idx]) {
     for (sym = (struct stash *) symbol_table[idx]->xas_list; sym;
         sym = sym->next) {
-      if (last_hit && sym->sym_type == sym_type && !strcmp(sym->sym_name, name))
+      if (last_hit && sym->sym_type == sym_type &&
+          (!name || strcasecmp(sym->sym_name, name) == 0))
         break;
       if (sym->ptr.sym_generic == prev)
         last_hit++;
@@ -222,11 +232,34 @@ void *pr_stash_get_symbol(pr_stash_type_t sym_type, const char *name,
 
   else {
 
-    idx = symtab_hash(name);
+    /* XXX Ugly hack to support mixed cases of directives in config files. */
+    if (sym_type != PR_SYM_CONF)
+      idx = symtab_hash(name);
+
+    else {
+      register unsigned int i;
+      char buf[1024];
+
+      memset(buf, '\0', sizeof(buf));
+      sstrncpy(buf, name, sizeof(buf)-1);
+
+      for (i = 0; i < strlen(buf); i++)
+        buf[i] = tolower((int) buf[i]);
+
+      idx = symtab_hash(buf);
+    }
+
     if (idx_cache)
       *idx_cache = idx;
   }
 
+  if (idx >= PR_TUNABLE_HASH_TABLE_SIZE) {
+    if (*idx_cache)
+      *idx_cache = -1;
+
+    return NULL;
+  }
+
   if (prev)
     curr_sym = sym = stash_lookup_next(sym_type, name, idx, prev);
   else
@@ -259,16 +292,32 @@ int pr_stash_remove_symbol(pr_stash_type_t sym_type, const char *sym_name,
     return -1;
   }
 
-  symtab_idx = symtab_hash(sym_name);
+  /* XXX Ugly hack to support mixed cases of directives in config files. */
+  if (sym_type != PR_SYM_CONF)
+    symtab_idx = symtab_hash(sym_name);
+
+  else {
+    register unsigned int i;
+    char buf[1024];
+
+    memset(buf, '\0', sizeof(buf));
+    sstrncpy(buf, sym_name, sizeof(buf)-1);
+
+    for (i = 0; i < strlen(buf); i++)
+      buf[i] = tolower((int) buf[i]);
+
+    symtab_idx = symtab_hash(buf);
+  }
 
   switch (sym_type) {
     case PR_SYM_CONF: {
       int idx = -1;
-      conftable *tab = NULL;
+      conftable *tab;
 
       tab = pr_stash_get_symbol(PR_SYM_CONF, sym_name, NULL, &idx);
 
       while (tab) {
+        pr_signals_handle();
 
         /* Note: this works because of a hack: the symbol look functions
          * set a static pointer, curr_sym, to point to the struct stash
@@ -276,12 +325,14 @@ int pr_stash_remove_symbol(pr_stash_type_t sym_type, const char *sym_name,
          * returns non-NULL.
          */
 
-        if (!sym_module || curr_sym->sym_module == sym_module) {
+        if (!sym_module ||
+            curr_sym->sym_module == sym_module) {
           xaset_remove(symbol_table[symtab_idx], (xasetmember_t *) curr_sym);
           destroy_pool(curr_sym->sym_pool);
+          tab = NULL;
         }
 
-        tab = pr_stash_get_symbol(PR_SYM_CONF, sym_name, NULL, &idx);
+        tab = pr_stash_get_symbol(PR_SYM_CONF, sym_name, tab, &idx);
       }
 
       break;
@@ -289,23 +340,26 @@ int pr_stash_remove_symbol(pr_stash_type_t sym_type, const char *sym_name,
 
     case PR_SYM_CMD: {
       int idx = -1;
-      cmdtable *tab = NULL;
+      cmdtable *tab;
 
       tab = pr_stash_get_symbol(PR_SYM_CMD, sym_name, NULL, &idx);
 
       while (tab) {
+        pr_signals_handle();
 
         /* Note: this works because of a hack: the symbol look functions
          * set a static pointer, curr_sym, to point to the struct stash
          * just looked up.  
          */
 
-        if (!sym_module || curr_sym->sym_module == sym_module) {
+        if (!sym_module ||
+            curr_sym->sym_module == sym_module) {
           xaset_remove(symbol_table[symtab_idx], (xasetmember_t *) curr_sym);
           destroy_pool(curr_sym->sym_pool);
+          tab = NULL;
         }
 
-        tab = pr_stash_get_symbol(PR_SYM_CMD, sym_name, NULL, &idx);
+        tab = pr_stash_get_symbol(PR_SYM_CMD, sym_name, tab, &idx);
       }
 
       break;
@@ -313,23 +367,26 @@ int pr_stash_remove_symbol(pr_stash_type_t sym_type, const char *sym_name,
 
     case PR_SYM_AUTH: {
       int idx = -1;
-      authtable *tab = NULL;
+      authtable *tab;
 
       tab = pr_stash_get_symbol(PR_SYM_AUTH, sym_name, NULL, &idx);
 
       while (tab) {
+        pr_signals_handle();
 
         /* Note: this works because of a hack: the symbol look functions
          * set a static pointer, curr_sym, to point to the struct stash
          * just looked up.  
          */
 
-        if (!sym_module || curr_sym->sym_module == sym_module) {
+        if (!sym_module ||
+            curr_sym->sym_module == sym_module) {
           xaset_remove(symbol_table[symtab_idx], (xasetmember_t *) curr_sym);
           destroy_pool(curr_sym->sym_pool);
+          tab = NULL;
         }
 
-        tab = pr_stash_get_symbol(PR_SYM_AUTH, sym_name, NULL, &idx);
+        tab = pr_stash_get_symbol(PR_SYM_AUTH, sym_name, tab, &idx);
       }
 
       break;
@@ -337,17 +394,21 @@ int pr_stash_remove_symbol(pr_stash_type_t sym_type, const char *sym_name,
 
     case PR_SYM_HOOK: {
       int idx = -1;
-      cmdtable *tab = NULL;
+      cmdtable *tab;
 
       tab = pr_stash_get_symbol(PR_SYM_HOOK, sym_name, NULL, &idx);
 
       while (tab) {
-        if (!sym_module || curr_sym->sym_module == sym_module) {
+        pr_signals_handle();
+
+        if (!sym_module ||
+            curr_sym->sym_module == sym_module) {
           xaset_remove(symbol_table[symtab_idx], (xasetmember_t *) curr_sym);
           destroy_pool(curr_sym->sym_pool);
+          tab = NULL;
         }
 
-        tab = pr_stash_get_symbol(PR_SYM_HOOK, sym_name, NULL, &idx);
+        tab = pr_stash_get_symbol(PR_SYM_HOOK, sym_name, tab, &idx);
       }
 
       break;
@@ -361,57 +422,14 @@ int pr_stash_remove_symbol(pr_stash_type_t sym_type, const char *sym_name,
   return count;
 }
 
-/* functions to manage modular privdata structure inside cmd_rec */
-privdata_t *mod_privdata_alloc(cmd_rec *cmd, char *tag, int size)
-{
-  privdata_t **pp;
-  privdata_t *p;
-
-  if (!tag)
-    return NULL;
-
-  p = pcalloc(cmd->pool,sizeof(privdata_t));
-
-  p->tag = pstrdup(cmd->pool,tag);
-  if (size)
-    p->value.ptr_val = pcalloc(cmd->pool,size);
-  p->m = curr_module;
-
-  if (!cmd->privarr)
-    cmd->privarr = make_array(cmd->pool,2,sizeof(privdata_t*));
-
-  pp = (privdata_t**)push_array(cmd->privarr);
-  *pp = p;
-
-  cmd->private = (privdata_t*)cmd->privarr->elts;
-  return p;
-}
-
-privdata_t *mod_privdata_find(cmd_rec *cmd, char *tag, module *m)
-{
-  int i;
-  privdata_t **p;
-
-  if (!tag)
-    return NULL;
-
-  if (!m)
-    m = curr_module;
-
-  for (i = 0, p = (privdata_t**)cmd->privarr->elts; i < cmd->privarr->nelts; i++, p++) {
-    if (!strcmp((*p)->tag,tag) && (m == ANY_MODULE || (*p)->m == m))
-      break;
-  }
-
-  return (i < cmd->privarr->nelts ? *p : NULL);
-}
-
 modret_t *call_module(module *m, modret_t *(*func)(cmd_rec *), cmd_rec *cmd) {
   modret_t *res;
   module *prev_module = curr_module;
 
-  if (!cmd->tmp_pool)
+  if (!cmd->tmp_pool) {
     cmd->tmp_pool = make_sub_pool(cmd->pool);
+    pr_pool_tag(cmd->tmp_pool, "call_module() cmd tmp_pool");
+  }
 
   curr_module = m;
   res = func(cmd);
@@ -426,7 +444,7 @@ modret_t *call_module(module *m, modret_t *(*func)(cmd_rec *), cmd_rec *cmd) {
 modret_t *mod_create_data(cmd_rec *cmd,void *d) {
   modret_t *ret;
 
-  ret = pcalloc(cmd->tmp_pool,sizeof(modret_t));
+  ret = pcalloc(cmd->tmp_pool, sizeof(modret_t));
   ret->data = d;
 
   return ret;
@@ -435,13 +453,13 @@ modret_t *mod_create_data(cmd_rec *cmd,void *d) {
 modret_t *mod_create_ret(cmd_rec *cmd, unsigned char err, char *n, char *m) {
   modret_t *ret;
 
-  ret = pcalloc(cmd->tmp_pool,sizeof(modret_t));
+  ret = pcalloc(cmd->tmp_pool, sizeof(modret_t));
   ret->mr_handler_module = curr_module;
   ret->mr_error = err;
   if (n)
-    ret->mr_numeric = pstrdup(cmd->tmp_pool,n);
+    ret->mr_numeric = pstrdup(cmd->tmp_pool, n);
   if (m)
-    ret->mr_message = pstrdup(cmd->tmp_pool,m);
+    ret->mr_message = pstrdup(cmd->tmp_pool, m);
 
   return ret;
 }
@@ -449,7 +467,7 @@ modret_t *mod_create_ret(cmd_rec *cmd, unsigned char err, char *n, char *m) {
 modret_t *mod_create_error(cmd_rec *cmd, int mr_errno) {
   modret_t *ret;
 
-  ret = pcalloc(cmd->tmp_pool,sizeof(modret_t));
+  ret = pcalloc(cmd->tmp_pool, sizeof(modret_t));
   ret->mr_handler_module = curr_module;
   ret->mr_error = mr_errno;
 
@@ -459,15 +477,19 @@ modret_t *mod_create_error(cmd_rec *cmd, int mr_errno) {
 /* Called after forking in order to inform/initialize modules
  * need to know we are a child and have a connection.
  */
-int module_session_init(void) {
-  module *prev_module = curr_module;
-  module *m;
+int modules_session_init(void) {
+  module *prev_module = curr_module, *m;
 
-  for (m = (module*) installed_modules->xas_list; m; m=m->next)
-    if (m && m->module_init_session_cb) {
+  for (m = loaded_modules; m; m = m->next) {
+    if (m && m->sess_init) {
       curr_module = m;
-      m->module_init_session_cb();
+      if (m->sess_init() < 0) {
+        pr_log_pri(PR_LOG_ERR, "mod_%s.c: error initializing session: %s",
+          m->name, strerror(errno));
+        return -1;
+      }
     }
+  }
 
   curr_module = prev_module;
   return 0;
@@ -482,210 +504,210 @@ unsigned char command_exists(char *name) {
   return (cmdtab ? TRUE : FALSE);
 }
 
-unsigned char module_exists(const char *name) {
-  char buf[PR_TUNABLE_BUFFER_SIZE] = {'\0'};
-  register unsigned int i = 0;
-
-  /* Check the list of compiled-in modules. */
-  for (i = 0; static_modules[i]; i++) {
-    memset(buf, '\0', sizeof(buf));
-    snprintf(buf, sizeof(buf), "mod_%s.c", (static_modules[i])->name);
-    buf[sizeof(buf)-1] = '\0';
-
-    if (!strcmp(buf, name))
-      return TRUE;
-  }
-
-  /* default */
-  return FALSE;
+unsigned char pr_module_exists(const char *name) {
+  return pr_module_get(name) != NULL ? TRUE : FALSE;
 }
 
-void list_modules(void) {
+module *pr_module_get(const char *name) {
+  char buf[80] = {'\0'};
+  module *m;
+
+  if (!name) {
+    errno = EINVAL;
+    return NULL;
+  }
+
+  /* Check the list of compiled-in modules. */
+  for (m = loaded_modules; m; m = m->next) {
+    memset(buf, '\0', sizeof(buf));
+    snprintf(buf, sizeof(buf), "mod_%s.c", m->name);
+    buf[sizeof(buf)-1] = '\0';
+
+    if (strcmp(buf, name) == 0)
+      return m;
+  }
+
+  errno = ENOENT;
+  return NULL;
+}
+
+void modules_list(void) {
   register unsigned int i = 0;
-  module *m = NULL;
 
   printf("Compiled-in modules:\n");
   for (i = 0; static_modules[i]; i++) {
-    m = static_modules[i];
+    module *m = static_modules[i];
     printf("  mod_%s.c\n", m->name);
   }
 }
 
-int module_daemon_startup(void) {
-  module_cb_t *di = NULL;
+int pr_module_load(module *m) {
+  char buf[256];
 
-  if (!daemon_startups)
-    return 0;
-
-  for (di = (module_cb_t *) daemon_startups->xas_list; di; di = di->next)
-    di->module_cb();
-
-  return 0;
-}
-
-int module_preparse_init(void) {
-  int numconf = 0,numcmd = 0,numauth = 0;
-  module *m;
-  conftable *conf = NULL;
-  cmdtable *cmd = NULL;
-  authtable *auth = NULL;
-  register unsigned int i = 0;
-
-  installed_modules = xaset_create(permanent_pool, NULL);
-
-  for (i = 0; static_modules[i]; i++) {
-    m = static_modules[i];
-    m->priority = i;
-
-    if (m->api_version < PR_MODULE_API_VERSION) {
-      log_pri(PR_LOG_ERR, "Fatal: module '%s' API version (0x%x) is too old "
-        "(need at least 0x%x)", m->name, m->api_version, PR_MODULE_API_VERSION);
-	exit(1);
-    }
-
-    if (!m->module_init_cb ||
-        (m->module_init_cb() >= 0)) {
-      xaset_insert(installed_modules, (xasetmember_t *) m);
-
-      if (m->conftable)
-        for (conf = m->conftable; conf->directive; conf++)
-          ++numconf;
-
-      if (m->cmdtable)
-        for (cmd = m->cmdtable; cmd->command; cmd++)
-          ++numcmd;
-
-      if (m->authtable)
-        for (auth = m->authtable; auth->name; auth++)
-          ++numauth;
-
-    } else
-      log_pri(PR_LOG_ERR, "error: initialization of 'mod_%s' module failed",
-        m->name);
+  if (!m) {
+    errno = EINVAL;
+    return -1;
   }
 
-  /* Allow for an empty entry */
-  ++numconf;
-  ++numcmd;
-  ++numauth;
+  /* Check the API version the module wants to use. */
+  if (m->api_version < PR_MODULE_API_VERSION) {
+    errno = EACCES;
+    return -1;
+  }
 
-  /* Create an array to store the master conf dispatch table */
-  mconfarr = make_array(permanent_pool, numconf, sizeof(conftable));
-  mcmdarr = make_array(permanent_pool, numcmd, sizeof(cmdtable));
-  mautharr = make_array(permanent_pool, numauth, sizeof(authtable));
+  /* Do not allow multiple modules with the same name. */
+  memset(buf, '\0', sizeof(buf));
+  snprintf(buf, sizeof(buf), "mod_%s.c", m->name);
+  buf[sizeof(buf)-1] = '\0';
 
-  for (m = (module *) installed_modules->xas_list; m; m = m->next) {
+  if (pr_module_get(buf) != NULL) {
+    errno = EEXIST;
+    return -1;
+  }
 
+  /* Invoke the module's initialization routine. */
+  if (!m->init ||
+      m->init() >= 0) {
+
+    /* Assign a priority to this module. */
+    m->priority = curr_module_pri++;
+
+    /* Add the module's config, cmd, and auth tables. */
     if (m->conftable) {
-      for (conf = m->conftable; conf->directive; conf++) {
-        conftable *conftab = (conftable *) push_array(mconfarr);
-        memcpy(conftab, conf, sizeof(conftable));
-        conftab->m = m;
+      conftable *conftab;
 
+      for (conftab = m->conftable; conftab->directive; conftab++) {
+        conftab->m = m;
         pr_stash_add_symbol(PR_SYM_CONF, conftab);
       }
     }
 
     if (m->cmdtable) {
-      for (cmd = m->cmdtable; cmd->command; cmd++) {
-        cmdtable *cmdtab = (cmdtable *) push_array(mcmdarr);
-        memcpy(cmdtab, cmd, sizeof(cmdtable));
+      cmdtable *cmdtab;
+
+      for (cmdtab = m->cmdtable; cmdtab->command; cmdtab++) {
         cmdtab->m = m;
 
-        /* HOOKs and CMDs share the cmdtable type, so check the cmd_type
-         * when adding this symbol to the stash.
-         */
         if (cmdtab->cmd_type == HOOK)
           pr_stash_add_symbol(PR_SYM_HOOK, cmdtab);
 
         else
+          /* All other cmd_types are for CMDs: PRE_CMD, CMD, POST_CMD, etc. */
           pr_stash_add_symbol(PR_SYM_CMD, cmdtab);
       }
     }
 
     if (m->authtable) {
-      for (auth = m->authtable; auth->name; auth++) {
-        authtable *authtab = (authtable *) push_array(mautharr);
-        memcpy(authtab, auth, sizeof(authtable));
-        authtab->m = m;
+      authtable *authtab;
 
+      for (authtab = m->authtable; authtab->name; authtab++) {
+        authtab->m = m;
         pr_stash_add_symbol(PR_SYM_AUTH, authtab);
       }
     }
-  }
 
-  /* add a null entry (pcalloc zeros the memory for us) */
-  push_array(mconfarr);
-  push_array(mcmdarr);
-  push_array(mautharr);
+    /* Add the module to the loaded_modules list. */
+    m->next = loaded_modules;
 
-  m_conftable = (conftable *) mconfarr->elts;
-  m_cmdtable = (cmdtable *) mcmdarr->elts;
-  m_authtable = (authtable *) mautharr->elts;
+    if (loaded_modules)
+      loaded_modules->prev = m;
 
-  return 0;
-}
+    loaded_modules = m;
 
-int module_postparse_init(void) {
-  module_cb_t *pp = NULL;
+    /* Generate an event. */
+    pr_event_generate("core.module-load", buf);
 
-  if (!postparse_inits)
     return 0;
+  }
 
-  for (pp = (module_cb_t *) postparse_inits->xas_list; pp; pp = pp->next)
-    pp->module_cb();
+  errno = EPERM;
+  return -1;
+}
+
+int pr_module_unload(module *m) {
+  char buf[256];
+
+  if (!m) {
+    errno = EINVAL;
+    return -1;
+  }
+
+  /* Make sure this module has been loaded.  We can't unload a module that
+   * has not been loaded, now can we?
+   */
+
+  memset(buf, '\0', sizeof(buf));
+  snprintf(buf, sizeof(buf), "mod_%s.c", m->name);
+  buf[sizeof(buf)-1] = '\0';
+
+  if (pr_module_get(buf) == NULL) {
+    errno = ENOENT;
+    return -1;
+  } 
+
+  /* Generate an event. */
+  pr_event_generate("core.module-unload", buf);
+
+  /* Remove the module from the loaded_modules list. */
+  if (m->prev)
+    m->prev->next = m->next;
+
+  else
+    /* This module is the start of the loaded_modules list (prev is NULL),
+     * so we need to update that pointer, too.
+     */
+    loaded_modules = m->next;
+
+  if (m->next)
+    m->next->prev = m->prev;
+
+  m->prev = m->next = NULL;
+
+  /* Remove the module's config, cmd, and auth tables. */
+  if (m->conftable) {
+    conftable *conftab;
+
+    for (conftab = m->conftable; conftab->directive; conftab++)
+      pr_stash_remove_symbol(PR_SYM_CONF, conftab->directive, conftab->m);
+  }
+
+  if (m->cmdtable) {
+    cmdtable *cmdtab;
+
+    for (cmdtab = m->cmdtable; cmdtab->command; cmdtab++) {
+      if (cmdtab->cmd_type == HOOK)
+        pr_stash_remove_symbol(PR_SYM_HOOK, cmdtab->command, cmdtab->m);
+
+      else
+        /* All other cmd_types are for CMDs: PRE_CMD, CMD, POST_CMD, etc. */
+        pr_stash_remove_symbol(PR_SYM_CMD, cmdtab->command, cmdtab->m);
+    }
+  }
+
+  if (m->authtable) {
+    authtable *authtab;
+
+    for (authtab = m->authtable; authtab->name; authtab++)
+      pr_stash_remove_symbol(PR_SYM_AUTH, authtab->name, authtab->m);
+  }
 
   return 0;
 }
 
-void pr_register_daemon_startup(int (*cb)(void)) {
-  module_cb_t *di = NULL;
+int modules_init(void) {
+  register unsigned int i = 0;
 
-  if (!daemon_cb_pool)
-    daemon_cb_pool = make_sub_pool(permanent_pool);
+  for (i = 0; static_modules[i]; i++) {
+    module *m = static_modules[i];
 
-  if (!daemon_startups)
-    daemon_startups = xaset_create(daemon_cb_pool, NULL);
-
-  di = pcalloc(daemon_cb_pool, sizeof(module_cb_t));
-  di->module_cb = cb;
-
-  xaset_insert(daemon_startups, (xasetmember_t *) di);
-}
-
-void pr_register_postparse_init(int (*cb)(void)) {
-  module_cb_t *pp = NULL;
-
-  if (!postparse_init_pool)
-    postparse_init_pool = make_sub_pool(permanent_pool);
-
-  if (!postparse_inits)
-    postparse_inits = xaset_create(postparse_init_pool, NULL);
-
-  pp = pcalloc(postparse_init_pool, sizeof(module_cb_t));
-  pp->module_cb = cb;
-
-  xaset_insert(postparse_inits, (xasetmember_t *) pp);
-}
-
-void module_remove_daemon_startups(void) {
-  if (daemon_startups)
-    daemon_startups = NULL;
-
-  if (daemon_cb_pool) {
-    destroy_pool(daemon_cb_pool);
-    daemon_cb_pool = NULL;
+    if (pr_module_load(m) < 0) {
+      pr_log_pri(PR_LOG_ERR, "Fatal: unable to load module 'mod_%s.c': %s",
+        m->name, strerror(errno));
+      exit(1);
+    }
   }
-}
 
-void module_remove_postparse_inits(void) {
-  if (postparse_inits)
-    postparse_inits = NULL;
-
-  if (postparse_init_pool) {
-    destroy_pool(postparse_init_pool);
-    postparse_init_pool = NULL;
-  }
+  return 0;
 }
 
 int init_stash(void) {
@@ -693,6 +715,7 @@ int init_stash(void) {
     destroy_pool(symbol_pool);
 
   symbol_pool = make_sub_pool(permanent_pool); 
+  pr_pool_tag(symbol_pool, "Stash Pool");
   memset(symbol_table, '\0', sizeof(symbol_table));
 
   return 0;
