@@ -1,6 +1,6 @@
 /*
  * ProFTPD - FTP server daemon
- * Copyright (c) 2003 The ProFTPD Project team
+ * Copyright (c) 2003-2005 The ProFTPD Project team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
  */
 
 /* Network address routines
- * $Id: netaddr.c,v 1.44 2004/06/30 02:42:13 castaglia Exp $
+ * $Id: netaddr.c,v 1.51 2005/09/19 21:35:38 castaglia Exp $
  */
 
 #include "conf.h"
@@ -33,12 +33,19 @@
 #define IN_IS_ADDR_LOOPBACK(a) \
   ((((long int) (a)->s_addr) & 0xff000000) == 0x7f000000)
 
+static pr_netaddr_t sess_local_addr;
+static int have_sess_local_addr = FALSE;
+
+static pr_netaddr_t sess_remote_addr;
+static char sess_remote_name[PR_TUNABLE_BUFFER_SIZE];
+static int have_sess_remote_addr = FALSE;
+
 /* Do reverse DNS lookups? */
 static int reverse_dns = 1;
 
 /* Provide replacements for needed functions. */
 
-#if !defined(HAVE_GETNAMEINFO) || defined(USE_GETNAMEINFO)
+#if !defined(HAVE_GETNAMEINFO) || defined(PR_USE_GETNAMEINFO)
 int pr_getnameinfo(const struct sockaddr *sa, socklen_t salen, char *host,
     size_t hostlen, char *serv, size_t servlen, int flags) {
 
@@ -80,9 +87,9 @@ int pr_getnameinfo(const struct sockaddr *sa, socklen_t salen, char *host,
 
   return 0;
 }
-#endif /* HAVE_GETNAMEINFO or USE_GETNAMEINFO */
+#endif /* HAVE_GETNAMEINFO or PR_USE_GETNAMEINFO */
 
-#if !defined(HAVE_GETADDRINFO) || defined(USE_GETADDRINFO)
+#if !defined(HAVE_GETADDRINFO) || defined(PR_USE_GETADDRINFO)
 int pr_getaddrinfo(const char *node, const char *service,
     const struct addrinfo *hints, struct addrinfo **res) {
 
@@ -172,7 +179,7 @@ void pr_freeaddrinfo(struct addrinfo *ai) {
   }
   free(ai);
 }
-#endif /* HAVE_GETADDRINFO or USE_GETADDRINFO */
+#endif /* HAVE_GETADDRINFO or PR_USE_GETADDRINFO */
 
 #if !defined(HAVE_INET_NTOP)
 const char *pr_inet_ntop(int af, const void *src, char *dst, size_t len) {
@@ -271,9 +278,9 @@ pr_netaddr_t *pr_netaddr_get_addr(pool *p, const char *name,
     array_header **addrs) {
 
   struct sockaddr_in v4;
-#ifdef USE_IPV6
+#ifdef PR_USE_IPV6
   struct sockaddr_in6 v6;
-#endif /* USE_IPV6 */
+#endif /* PR_USE_IPV6 */
   pr_netaddr_t *na = NULL;
   int res;
 
@@ -294,7 +301,7 @@ pr_netaddr_t *pr_netaddr_get_addr(pool *p, const char *name,
 
   na = (pr_netaddr_t *) pcalloc(p, sizeof(pr_netaddr_t));
 
-#ifdef USE_IPV6
+#ifdef PR_USE_IPV6
   memset(&v6, 0, sizeof(v6));
   v6.sin6_family = AF_INET6;
 
@@ -309,10 +316,11 @@ pr_netaddr_t *pr_netaddr_get_addr(pool *p, const char *name,
     if (addrs)
       *addrs = NULL;
 
-    pr_log_debug(DEBUG10, "'%s' resolved to an IPv6 address", name);
+    pr_log_debug(DEBUG10, "'%s' resolved to IPv6 address %s", name,
+      pr_netaddr_get_ipstr(na));
     return na;
   }
-#endif
+#endif /* PR_USE_IPV6 */
 
   memset(&v4, 0, sizeof(v4));
   v4.sin_family = AF_INET;
@@ -328,7 +336,8 @@ pr_netaddr_t *pr_netaddr_get_addr(pool *p, const char *name,
     if (addrs)
       *addrs = NULL;
 
-    pr_log_debug(DEBUG10, "'%s' resolved to an IPv4 address", name);
+    pr_log_debug(DEBUG10, "'%s' resolved to IPv4 address %s", name,
+      pr_netaddr_get_ipstr(na));
     return na;
 
   } else if (res == 0) {
@@ -345,51 +354,79 @@ pr_netaddr_t *pr_netaddr_get_addr(pool *p, const char *name,
 
     memset(&hints, 0, sizeof(hints));
 
-#ifdef USE_IPV6
-    /* This looks up both IPv4 (as IPv6-mapped) and IPv6 addresses. */
-    hints.ai_family = AF_UNSPEC;
-#else
     hints.ai_family = AF_INET;
-#endif /* USE_IPV6 */
     hints.ai_socktype = SOCK_STREAM;
 
-    pr_log_debug(DEBUG10, "attempting to resolve '%s' via DNS", name);
-
+    pr_log_debug(DEBUG10,
+      "attempting to resolve '%s' to IPv4 address via DNS", name);
     gai_res = pr_getaddrinfo(name, NULL, &hints, &info);
     if (gai_res != 0) {
-      pr_log_pri(PR_LOG_INFO, "getaddrinfo '%s' error: %s", name,
-        res != EAI_SYSTEM ? pr_gai_strerror(gai_res) : strerror(errno));
+      pr_log_pri(PR_LOG_INFO, "IPv4 getaddrinfo '%s' error: %s", name,
+        gai_res != EAI_SYSTEM ? pr_gai_strerror(gai_res) : strerror(errno));
       return NULL;
     }
 
     if (info) {
-      pr_log_debug(DEBUG10, "resolved '%s' to an %s address", name,
-        info->ai_family == AF_INET ? "IPv4" : "IPv6");
-
       /* Copy the first returned addr into na, as the return value. */
       pr_netaddr_set_family(na, info->ai_family);
       pr_netaddr_set_sockaddr(na, info->ai_addr);
 
-      /* If the caller provided a pointer for any additional addresses,
-       * then we cycle through the rest of getaddrinfo(3)'s results and
-       * build a list to return to the caller.
-       */
-      if (addrs) {
-        struct addrinfo *ai;
-        *addrs = make_array(p, 0, sizeof(pr_netaddr_t *));
-
-        for (ai = info->ai_next; ai; ai = ai->ai_next) {
-          pr_netaddr_t **elt = push_array(*addrs);
-
-          *elt = pcalloc(p, sizeof(pr_netaddr_t));
-          pr_netaddr_set_family(*elt, ai->ai_family);
-          pr_netaddr_set_sockaddr(*elt, ai->ai_addr);
-        }
-      }
+      pr_log_debug(DEBUG10, "resolved '%s' to %s address %s", name,
+        info->ai_family == AF_INET ? "IPv4" : "IPv6",
+        pr_netaddr_get_ipstr(na));
 
       pr_freeaddrinfo(info);
-      return na;
     }
+
+#ifdef PR_USE_IPV6
+    if (addrs) {
+      /* Do the call again, this time for IPv6 addresses.
+       *
+       * We make two separate getaddrinfo(3) calls, rather than one
+       * with a hint of AF_UNSPEC, because of certain bugs where the use
+       * of AF_UNSPEC does not function as advertised.  (I suspect this
+       * bug was caused by proftpd's calling pattern, but as I could
+       * not track it down, and as there are reports of AF_UNSPEC not
+       * being as fast as AF_INET/AF_INET6, it just seemed easier to
+       * do it this way.)
+       */
+
+      gai_res = 0;
+
+      memset(&hints, 0, sizeof(hints));
+
+      hints.ai_family = AF_INET6;
+      hints.ai_socktype = SOCK_STREAM;
+
+      pr_log_debug(DEBUG10,
+        "attempting to resolve '%s' to IPv6 address via DNS", name);
+      gai_res = pr_getaddrinfo(name, NULL, &hints, &info);
+      if (gai_res != 0) {
+        pr_log_pri(PR_LOG_INFO, "IPv6 getaddrinfo '%s' error: %s", name,
+          gai_res != EAI_SYSTEM ? pr_gai_strerror(gai_res) : strerror(errno));
+        return na;
+      }
+
+      if (info) {
+        pr_netaddr_t **elt;
+
+        *addrs = make_array(p, 0, sizeof(pr_netaddr_t *));
+        elt = push_array(*addrs);
+
+        *elt = pcalloc(p, sizeof(pr_netaddr_t));
+        pr_netaddr_set_family(*elt, info->ai_family);
+        pr_netaddr_set_sockaddr(*elt, info->ai_addr);
+
+        pr_log_debug(DEBUG10, "resolved '%s' to %s address %s", name,
+          info->ai_family == AF_INET ? "IPv4" : "IPv6",
+          pr_netaddr_get_ipstr(*elt));
+
+        pr_freeaddrinfo(info);
+      }
+    }
+#endif /* PR_USE_IPV6 */
+
+    return na;
   }
 
   pr_log_debug(DEBUG10, "failed to resolve '%s' to an IP address", name);
@@ -417,11 +454,11 @@ int pr_netaddr_set_family(pr_netaddr_t *na, int family) {
       na->na_addr.v4.sin_family = AF_INET;
       break;
 
-#ifdef USE_IPV6
+#ifdef PR_USE_IPV6
     case AF_INET6:
       na->na_addr.v6.sin6_family = AF_INET6;
       break;
-#endif /* USE_IPV6 */
+#endif /* PR_USE_IPV6 */
 
     default:
 #ifdef EAFNOSUPPORT
@@ -446,10 +483,10 @@ size_t pr_netaddr_get_sockaddr_len(const pr_netaddr_t *na) {
     case AF_INET:
       return sizeof(struct sockaddr_in);
  
-#ifdef USE_IPV6
+#ifdef PR_USE_IPV6
     case AF_INET6:
       return sizeof(struct sockaddr_in6);
-#endif /* USE_IPV6 */   
+#endif /* PR_USE_IPV6 */   
   }
 
   errno = EPERM;
@@ -466,10 +503,10 @@ size_t pr_netaddr_get_inaddr_len(const pr_netaddr_t *na) {
     case AF_INET:
       return sizeof(struct in_addr);
 
-#ifdef USE_IPV6
+#ifdef PR_USE_IPV6
     case AF_INET6:
       return sizeof(struct in6_addr);
-#endif /* USE_IPV6 */
+#endif /* PR_USE_IPV6 */
   }
 
   errno = EPERM;
@@ -486,10 +523,10 @@ struct sockaddr *pr_netaddr_get_sockaddr(const pr_netaddr_t *na) {
     case AF_INET:
       return (struct sockaddr *) &na->na_addr.v4;
 
-#ifdef USE_IPV6
+#ifdef PR_USE_IPV6
     case AF_INET6:
       return (struct sockaddr *) &na->na_addr.v6;
-#endif /* USE_IPV6 */
+#endif /* PR_USE_IPV6 */
   }
 
   errno = EPERM;
@@ -508,11 +545,11 @@ int pr_netaddr_set_sockaddr(pr_netaddr_t *na, struct sockaddr *addr) {
       memcpy(&(na->na_addr.v4), addr, sizeof(struct sockaddr_in));
       return 0;
 
-#ifdef USE_IPV6
+#ifdef PR_USE_IPV6
     case AF_INET6:
       memcpy(&(na->na_addr.v6), addr, sizeof(struct sockaddr_in6));
       return 0;
-#endif /* USE_IPV6 */
+#endif /* PR_USE_IPV6 */
   }
 
   errno = EPERM;
@@ -537,7 +574,7 @@ int pr_netaddr_set_sockaddr_any(pr_netaddr_t *na) {
       return 0;
     }
 
-#ifdef USE_IPV6
+#ifdef PR_USE_IPV6
     case AF_INET6:
       na->na_addr.v6.sin6_family = AF_INET6;
 #ifdef SIN6_LEN
@@ -545,7 +582,7 @@ int pr_netaddr_set_sockaddr_any(pr_netaddr_t *na) {
 #endif /* SIN6_LEN */
       memcpy(&na->na_addr.v6.sin6_addr, &in6addr_any, sizeof(struct in6_addr));
       return 0;
-#endif /* USE_IPV6 */
+#endif /* PR_USE_IPV6 */
   }
 
   errno = EPERM;
@@ -562,10 +599,10 @@ void *pr_netaddr_get_inaddr(const pr_netaddr_t *na) {
     case AF_INET:
       return (void *) &na->na_addr.v4.sin_addr;
 
-#ifdef USE_IPV6
+#ifdef PR_USE_IPV6
     case AF_INET6:
       return (void *) &na->na_addr.v6.sin6_addr;
-#endif /* USE_IPV6 */
+#endif /* PR_USE_IPV6 */
   }
 
   errno = EPERM;
@@ -582,10 +619,10 @@ unsigned int pr_netaddr_get_port(const pr_netaddr_t *na) {
     case AF_INET:
       return na->na_addr.v4.sin_port;
 
-#ifdef USE_IPV6
+#ifdef PR_USE_IPV6
     case AF_INET6:
       return na->na_addr.v6.sin6_port;
-#endif /* USE_IPV6 */
+#endif /* PR_USE_IPV6 */
   }
 
   errno = EPERM;
@@ -603,11 +640,11 @@ int pr_netaddr_set_port(pr_netaddr_t *na, unsigned int port) {
       na->na_addr.v4.sin_port = port;
       return 0;
 
-#ifdef USE_IPV6
+#ifdef PR_USE_IPV6
     case AF_INET6:
       na->na_addr.v6.sin6_port = port;
       return 0;
-#endif /* USE_IPV6 */
+#endif /* PR_USE_IPV6 */
   }
 
   errno = EPERM;
@@ -635,11 +672,11 @@ int pr_netaddr_cmp(const pr_netaddr_t *na1, const pr_netaddr_t *na2) {
       return memcmp(&na1->na_addr.v4.sin_addr, &na2->na_addr.v4.sin_addr,
         sizeof(struct in_addr));
 
-#ifdef USE_IPV6
+#ifdef PR_USE_IPV6
     case AF_INET6:
       return memcmp(&na1->na_addr.v6.sin6_addr, &na2->na_addr.v6.sin6_addr,
         sizeof(struct in6_addr));
-#endif /* USE_IPV6 */
+#endif /* PR_USE_IPV6 */
   }
 
   errno = EPERM;
@@ -679,7 +716,7 @@ int pr_netaddr_ncmp(const pr_netaddr_t *na1, const pr_netaddr_t *na2,
       break;
     }
 
-#ifdef USE_IPV6
+#ifdef PR_USE_IPV6
     case AF_INET6: {
       /* Make sure that the given number of bits is not more than supported
        * for IPv6 addresses (128).
@@ -691,7 +728,7 @@ int pr_netaddr_ncmp(const pr_netaddr_t *na1, const pr_netaddr_t *na2,
 
       break;
     }
-#endif /* USE_IPV6 */
+#endif /* PR_USE_IPV6 */
 
     default:
       errno = EPERM;
@@ -779,11 +816,11 @@ int pr_netaddr_fnmatch(pr_netaddr_t *na, const char *pattern, int flags) {
 }
 
 const char *pr_netaddr_get_ipstr(pr_netaddr_t *na) {
-#ifdef USE_IPV6
+#ifdef PR_USE_IPV6
   char buf[INET6_ADDRSTRLEN];
 #else
   char buf[INET_ADDRSTRLEN];
-#endif /* USE_IPV6 */
+#endif /* PR_USE_IPV6 */
   int res = 0;
   
   if (!na) {
@@ -874,7 +911,7 @@ const char *pr_netaddr_get_dnsstr(pr_netaddr_t *na) {
             } 
             break;
 
-#ifdef USE_IPV6
+#ifdef PR_USE_IPV6
           case AF_INET6:
             if (family == AF_INET6) {
               for (checkaddr = hent->h_addr_list; *checkaddr; ++checkaddr) {
@@ -885,7 +922,7 @@ const char *pr_netaddr_get_dnsstr(pr_netaddr_t *na) {
               }
             } 
             break;
-#endif /* USE_IPV6 */
+#endif /* PR_USE_IPV6 */
         }
 
         name = ok ? buf : NULL;
@@ -894,7 +931,10 @@ const char *pr_netaddr_get_dnsstr(pr_netaddr_t *na) {
         pr_log_debug(DEBUG1, "notice: unable to resolve '%s': %s", buf,
           hstrerror(errno));
     }
-  }
+
+  } else
+    pr_log_debug(DEBUG10,
+      "UseReverseDNS off, returning IP address instead of DNS name");
 
   if (!name)
     name = (char *) pr_netaddr_get_ipstr(na);
@@ -945,7 +985,7 @@ int pr_netaddr_loopback(const pr_netaddr_t *na) {
       return IN_IS_ADDR_LOOPBACK(
         (struct in_addr *) pr_netaddr_get_inaddr(na));
 
-#ifdef USE_IPV6
+#ifdef PR_USE_IPV6
     case AF_INET6:
 
       /* XXX *sigh* Different platforms implement the IN6_IS_ADDR macros
@@ -964,7 +1004,7 @@ int pr_netaddr_loopback(const pr_netaddr_t *na) {
       return IN6_IS_ADDR_LOOPBACK(
         ((struct in6_addr *) pr_netaddr_get_inaddr(na))->s6_addr32);
 # endif
-#endif /* USE_IPV6 */
+#endif /* PR_USE_IPV6 */
   }
 
   return FALSE;
@@ -983,7 +1023,7 @@ unsigned int pr_netaddr_get_addrno(const pr_netaddr_t *na) {
     case AF_INET:
       return na->na_addr.v4.sin_addr.s_addr;
 
-#ifdef USE_IPV6
+#ifdef PR_USE_IPV6
     case AF_INET6: {
 
       /* Linux defines s6_addr32 in its netinet/in.h header.
@@ -998,7 +1038,7 @@ unsigned int pr_netaddr_get_addrno(const pr_netaddr_t *na) {
       return 0;
 #endif
     }
-#endif /* USE_IPV6 */
+#endif /* PR_USE_IPV6 */
   }
 
   errno = EPERM;
@@ -1018,7 +1058,7 @@ int pr_netaddr_is_v4mappedv6(const pr_netaddr_t *na) {
       errno = EINVAL;
       return -1;
 
-#ifdef USE_IPV6
+#ifdef PR_USE_IPV6
     case AF_INET6:
 
 # ifndef LINUX
@@ -1028,10 +1068,53 @@ int pr_netaddr_is_v4mappedv6(const pr_netaddr_t *na) {
       return IN6_IS_ADDR_V4MAPPED(
         ((struct in6_addr *) pr_netaddr_get_inaddr(na))->s6_addr32);
 # endif
-#endif /* USE_IPV6 */
+#endif /* PR_USE_IPV6 */
   }
 
   errno = EPERM;
   return -1;
 }
 
+pr_netaddr_t *pr_netaddr_get_sess_local_addr(void) {
+  if (have_sess_local_addr) {
+    return &sess_local_addr;
+  }
+
+  errno = ENOENT;
+  return NULL;
+}
+
+pr_netaddr_t *pr_netaddr_get_sess_remote_addr(void) {
+  if (have_sess_remote_addr) {
+    return &sess_remote_addr;
+  }
+
+  errno = ENOENT;
+  return NULL;
+}
+
+const char *pr_netaddr_get_sess_remote_name(void) {
+  if (have_sess_remote_addr) {
+    return sess_remote_name;
+  }
+
+  errno = ENOENT;
+  return NULL;
+}
+
+void pr_netaddr_set_sess_addrs(void) {
+  pr_netaddr_set_family(&sess_local_addr,
+    pr_netaddr_get_family(session.c->local_addr));
+  pr_netaddr_set_sockaddr(&sess_local_addr,
+    pr_netaddr_get_sockaddr(session.c->local_addr));
+  have_sess_local_addr = TRUE;
+
+  pr_netaddr_set_family(&sess_remote_addr,
+    pr_netaddr_get_family(session.c->remote_addr));
+  pr_netaddr_set_sockaddr(&sess_remote_addr,
+    pr_netaddr_get_sockaddr(session.c->remote_addr));
+
+  memset(sess_remote_name, '\0', sizeof(sess_remote_name));
+  sstrncpy(sess_remote_name, session.c->remote_name, sizeof(sess_remote_name));
+  have_sess_remote_addr = TRUE;
+}
