@@ -25,7 +25,7 @@
 /*
  * ProFTPD scoreboard support.
  *
- * $Id: scoreboard.c,v 1.15 2003/01/02 18:25:27 castaglia Exp $
+ * $Id: scoreboard.c,v 1.31 2004/11/02 18:18:59 castaglia Exp $
  */
 
 #include "conf.h"
@@ -35,8 +35,9 @@
 /* From src/dirtree.c */
 extern char ServerType;
 
+static pid_t scoreboard_opener = 0;
 static int scoreboard_fd = -1;
-static char scoreboard_file[MAX_PATH_LEN] = RUN_DIR "/proftpd.scoreboard";
+static char scoreboard_file[PR_TUNABLE_PATH_MAX] = PR_RUN_DIR "/proftpd.scoreboard";
 
 static off_t current_pos = 0;
 static pr_scoreboard_header_t header;
@@ -48,7 +49,7 @@ static unsigned char scoreboard_write_locked = FALSE;
 
 /* Internal routines */
 
-static char *handle_score_cmd(const char *fmt, va_list cmdap) {
+static char *handle_score_str(const char *fmt, va_list cmdap) {
   static char buf[PR_TUNABLE_SCOREBOARD_BUFFER_SIZE] = {'\0'};
   memset(buf, '\0', sizeof(buf));
   vsnprintf(buf, sizeof(buf), fmt, cmdap);
@@ -59,24 +60,36 @@ static char *handle_score_cmd(const char *fmt, va_list cmdap) {
 static int read_scoreboard_header(pr_scoreboard_header_t *sch) {
   int res = 0;
 
+  /* No interruptions, please. */
+  pr_signals_block();
+ 
   /* NOTE: reading a struct from a file using read(2) -- bad (in general). */
   while ((res = read(scoreboard_fd, sch, sizeof(pr_scoreboard_header_t))) !=
       sizeof(pr_scoreboard_header_t)) {
-    if (res == 0)
+    int rd_errno = errno;
+
+    if (res == 0) {
+      pr_signals_unblock();
+      errno = EIO;
       return -1;
+    }
 
     if (errno == EINTR) {
       pr_signals_handle();
       continue;
+    }
 
-    } else
-      return -1;
+    pr_signals_unblock();
+    errno = rd_errno;
+    return -1;
   }
 
+  pr_signals_unblock();
+
   /* Note: these errors will most likely occur only for inetd-run daemons.
    * Standalone daemons erase the scoreboard on startup.
    */
-
+ 
   if (sch->sch_magic != PR_SCOREBOARD_MAGIC) {
     pr_close_scoreboard();
     return PR_SCORE_ERR_BAD_MAGIC;
@@ -218,57 +231,136 @@ int pr_close_scoreboard(void) {
   if (scoreboard_read_locked || scoreboard_write_locked)
     unlock_scoreboard();
 
-  close(scoreboard_fd);
+  while (close(scoreboard_fd) < 0) {
+    if (errno == EINTR) {
+      pr_signals_handle();
+      continue;
+
+    } else
+      break;
+  }
+
   scoreboard_fd = -1;
+  scoreboard_opener = 0;
 
   return 0;
 }
 
 void pr_delete_scoreboard(void) {
-  if (scoreboard_fd > -1)
-    close(scoreboard_fd);
+  if (scoreboard_fd > -1) {
+    while (close(scoreboard_fd) < 0) {
+      if (errno == EINTR) {
+        pr_signals_handle();
+        continue;
+
+      } else
+        break;
+    }
+  }
+
   scoreboard_fd = -1;
+  scoreboard_opener = 0;
+
+  if (*scoreboard_file) {
+    struct stat st;
+
+    if (stat(scoreboard_file, &st) == 0)
+      pr_log_debug(DEBUG3, "deleting existing scoreboard '%s'",
+        scoreboard_file);
 
-  if (scoreboard_file)
     unlink(scoreboard_file);
+  }
 }
 
 const char *pr_get_scoreboard(void) {
   return scoreboard_file;
 }
 
-int pr_open_scoreboard(int flags, pid_t *daemon_pid) {
+int pr_open_scoreboard(int flags) {
   int res;
   struct stat st;
 
+  /* Try to prevent a file descriptor leak by only opening the scoreboard
+   * file if the scoreboard file descriptor is not already positive, i.e.
+   * if the scoreboard has not already been opened.
+   */
+  if (scoreboard_fd >= 0 && scoreboard_opener == getpid()) {
+    pr_log_debug(DEBUG7, "scoreboard already opened");
+    return 0;
+  }
+
+  pr_log_debug(DEBUG7, "opening scoreboard '%s'", scoreboard_file);
+
   /* Prevent writing to a symlink while avoiding a race condition: open
    * the file name O_RDWR|O_CREAT first, then check to see if it's a symlink.
    * If so, close the file and error out.  If not, truncate as necessary,
    * and continue.
    */
-  if ((scoreboard_fd = open(scoreboard_file, flags|O_CREAT,
-      PR_SCOREBOARD_MODE)) < 0)
-    return -1;
+  
+  while ((scoreboard_fd = open(scoreboard_file, flags|O_CREAT,
+      PR_SCOREBOARD_MODE)) < 0) {
+    if (errno == EINTR) {
+      pr_signals_handle();
+      continue;
+
+    } else
+      return -1;
+  }
 
   /* Make certain that the scoreboard mode will be read-only for everyone
    * except the user owner (this allows for non-root-running daemons to
    * still modify the scoreboard).
    */
-  fchmod(scoreboard_fd, 0644);
+  while (fchmod(scoreboard_fd, 0644) < 0) {
+    if (errno == EINTR) {
+      pr_signals_handle();
+      continue;
 
-  if (fstat(scoreboard_fd, &st) < 0) {
-    close(scoreboard_fd);
+    } else
+      break;
+  }
+
+  while (fstat(scoreboard_fd, &st) < 0) {
+    int st_errno = errno;
+
+    if (errno == EINTR) {
+      pr_signals_handle();
+      continue;
+    }
+
+    while (close(scoreboard_fd) < 0) {
+      if (errno == EINTR) {
+        pr_signals_handle();
+        continue;
+
+      } else
+        break;
+    }
+
+    /* Either way, the scoreboard fd should be marked as closed. */
     scoreboard_fd = -1;
+
+    errno = st_errno;
     return -1;
   }
 
   if (S_ISLNK(st.st_mode)) {
-    close(scoreboard_fd);
-    scoreboard_fd = -1;
+    while (close(scoreboard_fd) < 0) {
+      if (errno == EINTR) {
+        pr_signals_handle();
+        continue;
+
+      } else
+        break;
+    }
+
     errno = EPERM;
+    scoreboard_fd = -1;
     return -1;
   }
 
+  scoreboard_opener = getpid();
+
   /* Check the header of this scoreboard file. */
   if ((res = read_scoreboard_header(&header)) == -1) {
 
@@ -278,27 +370,38 @@ int pr_open_scoreboard(int flags, pid_t *daemon_pid) {
     header.sch_magic = PR_SCOREBOARD_MAGIC;
     header.sch_version = PR_SCOREBOARD_VERSION;
 
-    if (ServerType == SERVER_STANDALONE)
+    if (ServerType == SERVER_STANDALONE) {
       header.sch_pid = getpid();
-    else
+      header.sch_uptime = time(NULL);
+
+    } else {
       header.sch_pid = 0;
+      header.sch_uptime = 0;
+    }
+
+    /* Write-lock the scoreboard file. */
+    if (wlock_scoreboard() < 0)
+      return -1;
 
     while (write(scoreboard_fd, &header, sizeof(header)) != sizeof(header)) {
+      int wr_errno = errno;
+
       if (errno == EINTR) {
         pr_signals_handle();
         continue;
+      }
 
-      } else
-        return -1;
+      unlock_scoreboard();
+
+      errno = wr_errno;
+      return -1;
     }
 
+    unlock_scoreboard();
     return 0;
 
-  } else if (res < 0)
-    return -1;
-
-  if (daemon_pid)
-    *daemon_pid = header.sch_pid;
+  } else
+    return res;
 
   return 0;
 }
@@ -334,7 +437,7 @@ int pr_rewind_scoreboard(void) {
 }
 
 int pr_set_scoreboard(const char *path) {
-  char dir[MAX_PATH_LEN] = {'\0'};
+  char dir[PR_TUNABLE_PATH_MAX] = {'\0'};
   struct stat st;
   char *tmp = NULL;
 
@@ -374,7 +477,11 @@ int pr_scoreboard_add_entry(void) {
   }
 
   /* Write-lock the scoreboard file. */
-  wlock_scoreboard();
+  if (wlock_scoreboard() < 0)
+    return -1;
+
+  /* No interruptions, please. */
+  pr_signals_block();
 
   /* If the scoreboard is open, the file position is already past the
    * header.
@@ -401,9 +508,6 @@ int pr_scoreboard_add_entry(void) {
 
     if (found_slot)
       break;
-
-    if (errno == EINTR)
-      pr_signals_handle();
   }
 
   memset(&entry, '\0', sizeof(entry));
@@ -413,9 +517,11 @@ int pr_scoreboard_add_entry(void) {
   entry.sce_gid = getegid();
 
   if (write_entry() < 0)
-    log_pri(PR_LOG_NOTICE, "error writing scoreboard entry: %s",
+    pr_log_pri(PR_LOG_NOTICE, "error writing scoreboard entry: %s",
       strerror(errno));
 
+  pr_signals_unblock();
+
   /* We can unlock the scoreboard now. */
   unlock_scoreboard();
 
@@ -434,13 +540,21 @@ int pr_scoreboard_del_entry(unsigned char verbose) {
   /* Write-lock this entry */
   wlock_entry();
   if (write_entry() < 0 && verbose)
-    log_pri(PR_LOG_NOTICE, "error deleting scoreboard entry: %s",
+    pr_log_pri(PR_LOG_NOTICE, "error deleting scoreboard entry: %s",
       strerror(errno));
   unlock_entry();
 
   return 0;
 }
 
+pid_t pr_scoreboard_get_daemon_pid(void) {
+  return header.sch_pid;
+}
+
+time_t pr_scoreboard_get_daemon_uptime(void) {
+  return header.sch_uptime;
+}
+
 pr_scoreboard_entry_t *pr_scoreboard_read_entry(void) {
   static pr_scoreboard_entry_t scan_entry;
   int res = 0;
@@ -451,8 +565,12 @@ pr_scoreboard_entry_t *pr_scoreboard_read_entry(void) {
   }
 
   /* Make sure the scoreboard file is read-locked. */
-  if (!scoreboard_read_locked)
-    rlock_scoreboard();
+  if (!scoreboard_read_locked) {
+
+    /* Do not proceed if we cannot lock the scoreboard. */
+    if (rlock_scoreboard() < 0)
+      return NULL; 
+  }
 
   memset(&scan_entry, '\0', sizeof(scan_entry));
 
@@ -507,18 +625,17 @@ int pr_scoreboard_update_entry(pid_t pid, ...) {
         sstrncpy(entry.sce_user, tmp, sizeof(entry.sce_user));
         break;
 
-      case PR_SCORE_CLIENT_ADDR:
-        {
-          p_in_addr_t *remote_ip = va_arg(ap, p_in_addr_t *);
+      case PR_SCORE_CLIENT_ADDR: {
+          pr_netaddr_t *remote_addr = va_arg(ap, pr_netaddr_t *);
 
           snprintf(entry.sce_client_addr, sizeof(entry.sce_client_addr),
-            "%s", remote_ip ? inet_ntoa(*remote_ip) : "(unknown)");
+            "%s", remote_addr ? pr_netaddr_get_ipstr(remote_addr) :
+            "(unknown)");
           entry.sce_client_addr[sizeof(entry.sce_client_addr) - 1] = '\0';
         }
         break;
 
-      case PR_SCORE_CLIENT_NAME:
-        {
+      case PR_SCORE_CLIENT_NAME: {
           char *remote_name = va_arg(ap, char *);
           
           snprintf(entry.sce_client_name, sizeof(entry.sce_client_name),
@@ -540,11 +657,10 @@ int pr_scoreboard_update_entry(pid_t pid, ...) {
         sstrncpy(entry.sce_cwd, tmp, sizeof(entry.sce_cwd));
         break;
 
-      case PR_SCORE_CMD:
-        {
+      case PR_SCORE_CMD: {
           char *cmdstr = NULL;
           tmp = va_arg(ap, char *);
-          cmdstr = handle_score_cmd(tmp, ap);
+          cmdstr = handle_score_str(tmp, ap);
 
           memset(entry.sce_cmd, '\0', sizeof(entry.sce_cmd));
           sstrncpy(entry.sce_cmd, cmdstr, sizeof(entry.sce_cmd));
@@ -552,22 +668,28 @@ int pr_scoreboard_update_entry(pid_t pid, ...) {
         }
         break;
 
-      case PR_SCORE_SERVER_IP:
-        entry.sce_server_ip = va_arg(ap, p_in_addr_t *);
+      case PR_SCORE_CMD_ARG: {
+          char *argstr = NULL;
+          tmp = va_arg(ap, char *);
+          argstr = handle_score_str(tmp, ap);
+
+          memset(entry.sce_cmd_arg, '\0', sizeof(entry.sce_cmd_arg));
+          sstrncpy(entry.sce_cmd_arg, argstr, sizeof(entry.sce_cmd_arg));
+          tmp = va_arg(ap, void *);
+        }
         break;
 
       case PR_SCORE_SERVER_PORT:
         entry.sce_server_port = va_arg(ap, int);
         break;
 
-      case PR_SCORE_SERVER_ADDR:
-        {
-          p_in_addr_t *server_ip = va_arg(ap, p_in_addr_t *);
+      case PR_SCORE_SERVER_ADDR: {
+          pr_netaddr_t *server_addr = va_arg(ap, pr_netaddr_t *);
           int server_port = va_arg(ap, int);
 
           snprintf(entry.sce_server_addr, sizeof(entry.sce_server_addr),
-            "%s:%d", server_ip ? inet_ntoa(*server_ip) : "(uknown)",
-            server_port);
+            "%s:%d", server_addr ? pr_netaddr_get_ipstr(server_addr) :
+            "(unknown)", server_port);
           entry.sce_server_addr[sizeof(entry.sce_server_addr)-1] = '\0';
         }
         break;
@@ -617,7 +739,7 @@ int pr_scoreboard_update_entry(pid_t pid, ...) {
   /* Write-lock this entry */
   wlock_entry();
   if (write_entry() < 0)
-    log_pri(PR_LOG_NOTICE, "error writing scoreboard entry: %s",
+    pr_log_pri(PR_LOG_NOTICE, "error writing scoreboard entry: %s",
       strerror(errno));
   unlock_entry();
 
