@@ -27,7 +27,7 @@
 /* Various basic support routines for ProFTPD, used by all modules
  * and not specific to one or another.
  *
- * $Id: support.c,v 1.61 2003/08/13 06:07:39 castaglia Exp $
+ * $Id: support.c,v 1.78 2005/09/28 02:06:26 castaglia Exp $
  */
 
 #include "conf.h"
@@ -46,14 +46,6 @@
 # include <sys/statfs.h>
 #endif
 
-static pool *exithandler_pool = NULL;
-
-typedef struct exit_obj {
-  struct exit_obj *next, *prev;
-
-  void (*exit_cb)(void);
-} exithandler_t;
-
 typedef struct sched_obj {
   struct sched_obj *next, *prev;
 
@@ -64,7 +56,6 @@ typedef struct sched_obj {
 } sched_t;
 
 static xaset_t *scheds = NULL;
-static xaset_t *exits = NULL;
 
 /* Masks/unmasks all important signals (as opposed to blocking alarms)
  */
@@ -102,41 +93,6 @@ void pr_signals_unblock(void) {
   mask_signals(FALSE);
 }
 
-void pr_exit_register_handler(void (*exit_cb)(void)) {
-  exithandler_t *e = NULL;
-
-  if (!exithandler_pool)
-    exithandler_pool = make_sub_pool(permanent_pool);
-
-  if (!exits)
-    exits = xaset_create(exithandler_pool, NULL);
-
-  e = pcalloc(exithandler_pool, sizeof(exithandler_t));
-  e->exit_cb = exit_cb;
-
-  xaset_insert(exits, (xasetmember_t *) e);
-}
-
-void remove_exit_handlers(void) {
-  if (exits)
-    exits = NULL;
-
-  if (exithandler_pool) {
-    destroy_pool(exithandler_pool);
-    exithandler_pool = NULL;
-  }
-}
-
-void run_exit_handlers(void) {
-  exithandler_t *e = NULL;
-
-  if (!exits)
-    return;
-
-  for (e = (exithandler_t *) exits->xas_list; e; e = e->next)
-    e->exit_cb();
-}
-
 void schedule(void (*f)(void*,void*,void*,void*),int nloops, void *a1,
     void *a2, void *a3, void *a4) {
   pool *p, *sub_pool;
@@ -144,10 +100,11 @@ void schedule(void (*f)(void*,void*,void*,void*),int nloops, void *a1,
 
   if (!scheds) {
    p = make_sub_pool(permanent_pool);
+   pr_pool_tag(p, "Schedules Pool");
    scheds = xaset_create(p, NULL);
 
   } else
-   p = scheds->mempool;
+   p = scheds->pool;
 
   sub_pool = make_sub_pool(p);
 
@@ -213,7 +170,7 @@ int get_name_max(char *dirname, int dir_fd) {
     /* NB: errno may not be set if the failure is due to a limit or option
      * not being supported.
      */
-    log_debug(DEBUG1, msgfmt, dirname ? dirname : "(NULL)", name_max, errno);
+    pr_log_debug(DEBUG1, msgfmt, dirname ? dirname : "(NULL)", name_max, errno);
   }
 
 #else
@@ -244,7 +201,7 @@ char *dir_interpolate(pool *p, const char *path) {
     if (!*user)
       user = session.user;
 
-    pw = auth_getpwnam(p, user);
+    pw = pr_auth_getpwnam(p, user);
 
     if (!pw) {
       errno = ENOENT;
@@ -258,7 +215,7 @@ char *dir_interpolate(pool *p, const char *path) {
 }
 
 /* dir_best_path() creates the "most" fully canonicalized path possible
- * (i.e. if path components at the end don't exist, they are ignored
+ * (i.e. if path components at the end don't exist, they are ignored).
  */
 char *dir_best_path(pool *p, const char *path) {
   char workpath[PR_TUNABLE_PATH_MAX + 1] = {'\0'};
@@ -481,81 +438,33 @@ int exists(char *path) {
   return _exists(path, -1);
 }
 
-/* Perform access check for effective user id, similar to accessx(...,
- * ACC_SELF) on AIX.
- */
-int access_check(char *path, int mode) {
-  mode_t mask;
-  struct stat buf;
+char *pr_str_strip(pool *p, char *str) {
+  char c, *dupstr, *start, *finish;
 
-  pr_fs_clear_cache();
-  if (pr_fsio_stat(path, &buf) < 0) {
-    errno = ENOENT;
-    return -1;
+  if (!p || !str) {
+    errno = EINVAL;
+    return NULL;
   }
 
-  /* If root, always return succeed. */
-  if (session.uid == 0)
-    return 0;
+  /* First, find the non-whitespace start of the given string */
+  for (start = str; isspace((int) *start); start++);
 
-  /* Initialize `mask' to reflect the permission bits that are
-   * applicable for the effective user. `mask' contains the user-bits
-   * if the effective user id equals the id of the file owner. `mask'
-   * contains the group bits if the group id is if the effective user
-   * belongs to the group of the file. `mask' will always contain the
-   * other bits of the permission bits.
+  /* Now, find the non-whitespace end of the given string */
+  for (finish = &str[strlen(str)-1]; isspace((int) *finish); finish--);
+
+  /* finish is now pointing to a non-whitespace character.  So advance one
+   * character forward, and set that to NUL.
    */
-  mask = S_IROTH | S_IWOTH | S_IXOTH;
+  c = *++finish;
+  *finish = '\0';
 
-  if (buf.st_uid == session.uid)
-    mask |= S_IRUSR|S_IWUSR|S_IXUSR;
+  /* The space-stripped string is, then, everything from start to finish. */
+  dupstr = pstrdup(p, start);
+ 
+  /* Restore the given string buffer contents. */
+  *finish = c;
 
-  /* Check the current group, as well as all supplementary groups.
-   * Fortunately, we have this information cached, so accessing it is
-   * almost free.
-   */
-  if (buf.st_gid == session.gid) {
-    mask |= S_IRGRP | S_IWGRP | S_IXGRP;
-
-  } else {
-    if (session.gids) {
-      register unsigned int i = 0;
-
-      for (i = 0; i < session.gids->nelts; i++) {
-        if (buf.st_gid == ((gid_t *) session.gids->elts)[i]) {
-	  mask |= S_IRGRP|S_IWGRP|S_IXGRP;
-	  break;
-        }
-      }
-    }
-  }
-
-  mask &= buf.st_mode;
-
-  /* Perform requested access checks */
-  if (mode & R_OK) {
-    if (!(mask & (S_IRUSR|S_IRGRP|S_IROTH))) {
-      errno = EACCES;
-      return -1;
-    }
-  }
-
-  if (mode & W_OK) {
-    if (!(mask & (S_IWUSR|S_IWGRP|S_IWOTH))) {
-      errno = EACCES;
-      return -1;
-    }
-  }
-
-  if (mode & X_OK) {
-    if (!(mask & (S_IXUSR|S_IXGRP|S_IXOTH))) {
-      errno = EACCES;
-      return -1;
-    }
-  }
-
-  /* F_OK already checked by checking the return value of stat */
-  return 0;
+  return dupstr;
 }
 
 char *strip_end(char *s, char *ch) {
@@ -620,7 +529,7 @@ char *safe_token(char **s) {
   return res;
 }
 
-/* Checks for the existance of SHUTMSG_PATH.  deny and disc are
+/* Checks for the existence of PR_SHUTMSG_PATH.  deny and disc are
  * filled with the times to deny new connections and disconnect
  * existing ones.
  */
@@ -632,7 +541,7 @@ int check_shutmsg(time_t *shut, time_t *deny, time_t *disc, char *msg,
   time_t now,shuttime = (time_t)0;
   struct tm tm;
 
-  if (file_exists(SHUTMSG_PATH) && (fp = fopen(SHUTMSG_PATH, "r"))) {
+  if (file_exists(PR_SHUTMSG_PATH) && (fp = fopen(PR_SHUTMSG_PATH, "r"))) {
     if ((cp = fgets(buf, sizeof(buf),fp)) != NULL) {
       buf[sizeof(buf)-1] = '\0'; CHOP(cp);
 
@@ -723,8 +632,9 @@ char *sreplace(pool *p, char *s, ...) {
   char **mptr,**rptr;
   char *marr[33],*rarr[33];
   char buf[PR_TUNABLE_PATH_MAX] = {'\0'}, *pbuf = NULL;
-  int mlen = 0, rlen = 0;
-  int blen, dyn = 1;
+  size_t mlen = 0, rlen = 0;
+  int blen;
+  int dyn = TRUE;
 
   cp = buf;
   *cp = '\0';
@@ -735,12 +645,54 @@ char *sreplace(pool *p, char *s, ...) {
 
   va_start(args, s);
 
-  while ((m = va_arg(args, char *)) != NULL && mlen < 32) {
+  while ((m = va_arg(args, char *)) != NULL && mlen < sizeof(marr)-1) {
+    char *tmp = NULL;
+    int count = 0;
+
     if ((r = va_arg(args, char *)) == NULL)
       break;
-    blen += (strlen(r) - strlen(m));
-    marr[mlen] = m;
-    rarr[mlen++] = r;
+
+    /* Increase the length of the needed buffer by the difference between
+     * the given match and replacement strings, multiplied by the number
+     * of times the match string occurs in the source string.
+     */
+    tmp = strstr(s, m);
+    while (tmp) {
+      pr_signals_handle();
+      count++;
+      if (count > 8) {
+        /* More than eight instances of the same escape on the same line?
+         * Give me a break.
+         */
+        return s;
+      }
+
+      /* Be sure to increment the pointer returned by strstr(3), to
+       * advance past the beginning of the substring for which we are
+       * looking.  Otherwise, we just loop endlessly, seeing the same
+       * value for tmp over and over.
+       */
+      tmp += strlen(m);
+      tmp = strstr(tmp, m);
+    }
+
+    /* We are only concerned about match/replacement strings that actually
+     * occur in the given string.
+     */
+    if (count) {
+      blen += count * (strlen(r) - strlen(m));
+      if (blen < 0) {
+        /* Integer overflow. In order to overflow this, somebody must be
+         * doing something very strange. The possibility still exists that
+         * we might not catch this overflow in extreme corner cases, but
+         * massive amounts of data (gigabytes) would need to be in s to
+         * trigger this, easily larger than any buffer we might use.
+         */
+        return s;
+      }
+      marr[mlen] = m;
+      rarr[mlen++] = r;
+    }
   }
 
   va_end(args);
@@ -759,7 +711,7 @@ char *sreplace(pool *p, char *s, ...) {
 
   if (!pbuf) {
     cp = pbuf = buf;
-    dyn = 0;
+    dyn = FALSE;
     blen = sizeof(buf);
   }
 
@@ -771,7 +723,7 @@ char *sreplace(pool *p, char *s, ...) {
       if (strncmp(src, *mptr, mlen) == 0) {
         sstrncpy(cp, *rptr, blen - strlen(pbuf));
 	if (((cp + rlen) - pbuf + 1) > blen) {
-	  log_pri(PR_LOG_ERR,
+	  pr_log_pri(PR_LOG_ERR,
 		  "WARNING: attempt to overflow internal ProFTPD buffers");
 	  cp = pbuf + blen - 1;
 	  goto done;
@@ -786,10 +738,11 @@ char *sreplace(pool *p, char *s, ...) {
     }
 
     if (!*mptr) {
-      if ((cp - pbuf + 1) > blen) {
-	log_pri(PR_LOG_ERR,
+      if ((cp - pbuf + 1) >= blen) {
+	pr_log_pri(PR_LOG_ERR,
 		"WARNING: attempt to overflow internal ProFTPD buffers");
 	cp = pbuf + blen - 1;
+	goto done;
       }
       *cp++ = *src++;
     }
@@ -819,7 +772,7 @@ void pr_memscrub(void *ptr, size_t ptrlen) {
 
   while (loop--) {
     *(p++) = memscrub_ctr++;
-    memscrub_ctr += (17 + (unsigned char)((int) p & 0xF));
+    memscrub_ctr += (17 + (unsigned char)((intptr_t) p & 0xF));
   }
 
   if (memchr(ptr, memscrub_ctr, ptrlen))
@@ -840,3 +793,47 @@ char *sstrcat(char *dest, const char *src, size_t n) {
   *d = 0;
   return dest;
 }
+
+struct tm *pr_gmtime(pool *p, const time_t *t) {
+  struct tm *sys_tm, *dup_tm;
+
+  sys_tm = gmtime(t);
+  dup_tm = pcalloc(p, sizeof(struct tm));
+  memcpy(dup_tm, sys_tm, sizeof(struct tm));
+
+  return dup_tm;
+}
+
+struct tm *pr_localtime(pool *p, const time_t *t) {
+  struct tm *sys_tm, *dup_tm;
+
+  sys_tm = localtime(t);
+  dup_tm = pcalloc(p, sizeof(struct tm));
+  memcpy(dup_tm, sys_tm, sizeof(struct tm));
+
+  return dup_tm;
+}
+
+const char *pr_strtime(time_t t) {
+  static char buf[30];
+  static char *mons[] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul",
+    "Aug", "Sep", "Oct", "Nov", "Dec" };
+  static char *days[] = { "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat" };
+  struct tm *tr;
+
+  memset(buf, '\0', sizeof(buf));
+
+  tr = localtime(&t);
+  if (tr != NULL) {
+    snprintf(buf, sizeof(buf), "%s %s %2d %02d:%02d:%02d %d",
+      days[tr->tm_wday], mons[tr->tm_mon], tr->tm_mday, tr->tm_hour,
+      tr->tm_min, tr->tm_sec, tr->tm_year + 1900);
+
+  } else
+    buf[0] = '\0';
+
+  buf[sizeof(buf)-1] = '\0';
+
+  return buf;
+}
+
