@@ -23,7 +23,7 @@
  */
 
 /* ProFTPD xferlog(5) logging support.
- * $Id: xferlog.c,v 1.1 2003/11/09 22:19:47 castaglia Exp $
+ * $Id: xferlog.c,v 1.3 2005/06/14 01:23:45 castaglia Exp $
  */
 
 #include "conf.h"
@@ -50,7 +50,7 @@ int xferlog_open(const char *path) {
 
   if (xferlogfd == -1) {
     pr_log_debug(DEBUG6, "opening TransferLog '%s'", path);
-    pr_log_openfile(path, &xferlogfd, LOG_XFER_MODE);
+    pr_log_openfile(path, &xferlogfd, PR_LOG_XFER_MODE);
   }
 
   return xferlogfd;
@@ -77,7 +77,7 @@ int xferlog_write(long xfertime, const char *remhost, off_t fsize, char *fname,
       pr_strtime(time(NULL)),
       xfertime,
       remhost,
-      fsize,
+      (pr_off_t) fsize,
       fbuf,
       xfertype,
       direction,
