@@ -25,7 +25,7 @@
 /*
  * ProFTPD scoreboard support (modified for use by external utilities).
  *
- * $Id: scoreboard.c,v 1.4 2003/01/05 01:31:35 jwm Exp $
+ * $Id: scoreboard.c,v 1.8 2006/03/22 18:51:45 castaglia Exp $
  */
 
 #include "utils.h"
@@ -33,7 +33,7 @@
 #include <signal.h>
 
 static int util_scoreboard_fd = -1;
-static char util_scoreboard_file[MAX_PATH_LEN] = RUN_DIR "/proftpd.scoreboard";
+static char util_scoreboard_file[PR_TUNABLE_PATH_MAX] = PR_RUN_DIR "/proftpd.scoreboard";
 
 static pr_scoreboard_header_t util_header;
 
@@ -143,7 +143,7 @@ const char *util_get_scoreboard(void) {
   return util_scoreboard_file;
 }
 
-int util_open_scoreboard(int flags, pid_t *daemon_pid) {
+int util_open_scoreboard(int flags) {
   int res;
   struct stat st;
 
@@ -172,27 +172,24 @@ int util_open_scoreboard(int flags, pid_t *daemon_pid) {
   if ((res = read_scoreboard_header(&util_header)) < 0)
     return res;
 
-  if (daemon_pid)
-    *daemon_pid = util_header.sch_pid;
-
   return 0;
 }
 
 int util_set_scoreboard(const char *path) {
-  char dir[MAX_PATH_LEN] = {'\0'};
+  char dir[PR_TUNABLE_PATH_MAX] = {'\0'};
   struct stat st;
   char *tmp = NULL;
 
   util_sstrncpy(dir, path, sizeof(dir));
 
-  if ((tmp = strrchr(dir, '/')) == NULL) {
+  tmp = strrchr(dir, '/');
+  if (tmp == NULL) {
     errno = EINVAL;
     return -1;
   }
   *tmp = '\0';
 
-  /* Parent directory must not be world-writeable */
-
+  /* Parent directory must not be world-writable. */
   if (stat(dir, &st) < 0)
     return -1;
 
@@ -206,10 +203,24 @@ int util_set_scoreboard(const char *path) {
     return -1;
   }
 
+  *tmp = '/';
+
+  /* Make sure file exists. */
+  if (stat(dir, &st) < 0)
+    return -1;
+
   util_sstrncpy(util_scoreboard_file, path, sizeof(util_scoreboard_file));
   return 0;
 }
 
+pid_t util_scoreboard_get_daemon_pid(void) {
+  return util_header.sch_pid;
+}
+
+time_t util_scoreboard_get_daemon_uptime(void) {
+  return util_header.sch_uptime;
+}
+
 pr_scoreboard_entry_t *util_scoreboard_read_entry(void) {
   static pr_scoreboard_entry_t scan_entry;
   int res = 0;
